package javawrapper;

import javax.swing.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Vector;
import java.awt.FlowLayout;
import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.awt.Dimension;
import javax.swing.event.ChangeListener;
import javax.swing.event.ChangeEvent;

import org.jfree.data.xy.DefaultXYZDataset;
import org.jfree.chart.ChartPanel;
import org.jfree.chart.plot.XYPlot;
import org.jfree.chart.plot.DefaultDrawingSupplier;

import datavisualizer.XYTreeRenderer;
import datavisualizer.DataGrapher;
import datavisualizer.XYZDataItem;
import datavisualizer.XYSimpleCellRenderer;
import datavisualizer.XYZSimpleDataset;

/**
 * Displays episodic events and allows for editing.
 * <p>Copyright: Copyright (c) Charles D. Canham 2003</p>
 * <p>Company: Institute of Ecosystem Studies</p>
 * @author Lora E. Murphy
 * @version 1.0
 */

public class EpisodicEventsWindow
    extends JDialog
    implements ActionListener, ChangeListener {

  /**A dataset for trees, one series for each species*/
  protected DefaultXYZDataset m_oTreeDataset = new DefaultXYZDataset();

  /**Renderer for displaying the trees along with episodic event data*/
  protected XYTreeRenderer m_oTreeRenderer;

  /**Plot object for rendering events*/
  protected XYPlot m_oPlot = new XYPlot();

  /** Dataset for displaying harvest events - controls which cells show
   * up which color */
  private XYZSimpleDataset m_oHarvestDataset;

  /** Dataset for displaying mortality episode events - controls which cells
   * show up which color */
  private XYZSimpleDataset m_oMortEpisodeDataset;

  /** Dataset for displaying planting events - controls which cells show
   * up which color */
  private XYZSimpleDataset m_oPlantDataset;

  /** Renderer for displaying all events */
  private XYSimpleCellRenderer m_oRenderer = new XYSimpleCellRenderer();

  /**Copy of the DisturbanceBehaviors object to exchange data with*/
  protected DisturbanceBehaviors m_oDisturbanceBehaviors;

  /**Copy of the PlantingBehaviors object to exchange data with*/
  protected PlantingBehaviors m_oPlantBehaviors;

  /**Copy of the planting initial diameter at 10 cm values*/
  protected float[] mp_fPlantInitialDiam10s;

  /**Copy of harvest data - copy this from m_oDisturbanceBehaviors, let the
   * user make their updates, then copy this back to m_oDisturbanceBehaviors*/
  protected Vector mp_oHarvestData = new Vector(0);

  /**Copy of mortality episode data - copy this from m_oDisturbanceBehaviors,
   * let the user make their updates, then copy this back to
   * m_oDisturbanceBehaviors*/
  protected Vector mp_oMortEpisodeData = new Vector(0);

  /**Copy of planting data - copy this from m_oPlantBehaviors, let the
   * user make their updates, then copy this back to m_oPlantBehaviors*/
  protected Vector mp_oPlantingData = new Vector(0);

  /**For displaying harvest, episodic mortality, and planting events*/
  protected JTabbedPane m_jTabs = new JTabbedPane();

  /**Array of labels displaying species percentages for the current planting*/
  protected JLabel mp_jSpeciesPlantPercentages[];

  /**Array of labels displaying the initial diam10 for each species for all
   * plantings*/
  protected JLabel mp_jSpeciesPlantInitialDiam10[];

  /**Label that displays the cut type for the currently displayed harvest*/
  protected JLabel m_jCutTypeLabel = new JLabel("N/A");

  /**Label that displays the timestep for the currently displayed harvest*/
  protected JLabel m_jHarvestTimestepLabel = new JLabel("N/A");

  /**Label that displays the timestep for the currently displayed planting*/
  protected JLabel m_jPlantTimestepLabel = new JLabel("N/A");

  /**Label that displays the timestep for the currently displayed episodic
   * mortality event*/
  protected JLabel m_jMortEpTimestepLabel = new JLabel("N/A");

  /**Label that displays the cut amount type for the currently displayed harvest*/
  protected JLabel m_jHarvestCutAmountTypeLabel = new JLabel(
      "N/A");

  /**Label that displays the cut amount type for the currently displayed
   * mortality episode event*/
  protected JLabel m_jMortEpCutAmountTypeLabel = new JLabel("N/A");

  /**Label that displays the plant spacing for the currently displayed planting*/
  protected JLabel m_jPlantSpacingLabel = new JLabel("N/A");

  /**Label that displays the plant amount label for the currently displayed planting*/
  protected JLabel m_jPlantAmountLabel = new JLabel("Density (#/ha):");

  /**Label that displays the plant amount value for the currently displayed planting*/
  protected JLabel m_jPlantAmountValueLabel = new JLabel("N/A");

  /**Label that displays number of harvest events currently defined*/
  protected JLabel m_jNumHarvestEvents = new JLabel("0");

  /**Label that displays number of mortality episodes currently defined*/
  protected JLabel m_jNumMortEpEvents = new JLabel("0");

  /**Label that displays number of planting events currently defined*/
  protected JLabel m_jNumPlantingEvents = new JLabel("0");

  /**Label that displays the number of the current harvest event*/
  protected JLabel m_jHarvestNumber = new JLabel("0");

  /**Label that displays the number of the current mortality episode event*/
  protected JLabel m_jMortEpNumber = new JLabel("0");

  /**Label that displays the number of the current planting event*/
  protected JLabel m_jPlantNumber = new JLabel("0");

  /**Label that displays the cut range 1 minimum for the currently displayed harvest*/
  protected JLabel m_jHarvestCutRange1Min = new JLabel("0");

  /**Label that displays the cut range 2 minimum for the currently displayed harvest*/
  protected JLabel m_jHarvestCutRange2Min = new JLabel("0");

  /**Label that displays the cut range 3 minimum for the currently displayed harvest*/
  protected JLabel m_jHarvestCutRange3Min = new JLabel("0");

  /**Label that displays the cut range 4 minimum for the currently displayed harvest*/
  protected JLabel m_jHarvestCutRange4Min = new JLabel("0");

  /**Label that displays the cut range 1 maximum for the currently displayed harvest*/
  protected JLabel m_jHarvestCutRange1Max = new JLabel("0");

  /**Label that displays the cut range 2 maximum for the currently displayed harvest*/
  protected JLabel m_jHarvestCutRange2Max = new JLabel("0");

  /**Label that displays the cut range 3 maximum for the currently displayed harvest*/
  protected JLabel m_jHarvestCutRange3Max = new JLabel("0");

  /**Label that displays the cut range 4 maximum for the currently displayed harvest*/
  protected JLabel m_jHarvestCutRange4Max = new JLabel("0");

  /**Label that displays the cut range 1 amount for the currently displayed harvest*/
  protected JLabel m_jHarvestCutRange1Amt = new JLabel("0");

  /**Label that displays the cut range 2 amount for the currently displayed harvest*/
  protected JLabel m_jHarvestCutRange2Amt = new JLabel("0");

  /**Label that displays the cut range 3 amount for the currently displayed harvest*/
  protected JLabel m_jHarvestCutRange3Amt = new JLabel("0");

  /**Label that displays the cut range 4 amount for the currently displayed harvest*/
  protected JLabel m_jHarvestCutRange4Amt = new JLabel("0");

  /**Label that displays the cut range 1 minimum for the currently displayed mortality episode*/
  protected JLabel m_jMortEpCutRange1Min = new JLabel("0");

  /**Label that displays the cut range 2 minimum for the currently displayed mortality episode*/
  protected JLabel m_jMortEpCutRange2Min = new JLabel("0");

  /**Label that displays the cut range 3 minimum for the currently displayed mortality episode*/
  protected JLabel m_jMortEpCutRange3Min = new JLabel("0");

  /**Label that displays the cut range 4 minimum for the currently displayed mortality episode*/
  protected JLabel m_jMortEpCutRange4Min = new JLabel("0");

  /**Label that displays the cut range 1 maximum for the currently displayed mortality episode*/
  protected JLabel m_jMortEpCutRange1Max = new JLabel("0");

  /**Label that displays the cut range 2 maximum for the currently displayed mortality episode*/
  protected JLabel m_jMortEpCutRange2Max = new JLabel("0");

  /**Label that displays the cut range 3 maximum for the currently displayed mortality episode*/
  protected JLabel m_jMortEpCutRange3Max = new JLabel("0");

  /**Label that displays the cut range 4 maximum for the currently displayed mortality episode*/
  protected JLabel m_jMortEpCutRange4Max = new JLabel("0");

  /**Label that displays the cut range 1 amount for the currently displayed mortality episode*/
  protected JLabel m_jMortEpCutRange1Amt = new JLabel("0");

  /**Label that displays the cut range 2 amount for the currently displayed mortality episode*/
  protected JLabel m_jMortEpCutRange2Amt = new JLabel("0");

  /**Label that displays the cut range 3 amount for the currently displayed mortality episode*/
  protected JLabel m_jMortEpCutRange3Amt = new JLabel("0");

  /**Label that displays the cut range 4 amount for the currently displayed mortality episode*/
  protected JLabel m_jMortEpCutRange4Amt = new JLabel("0");

  /**List of species for the currently displayed harvest*/
  protected DefaultListModel m_jHarvestSpeciesList = new DefaultListModel();

  /**List of species for the currently displayed mortality episode*/
  protected DefaultListModel m_jMortEpSpeciesList = new DefaultListModel();

  /**Panel displaying the chart*/
  protected JPanel m_jChartPanel = new JPanel();

  /**Size of the parent calling window - so we can make sure children fit
   * within this*/
  protected Dimension m_jParentSize;

  /**Number of X cells in the Harvest grid*/
  protected int m_iNumHarvestXCells;

  /**Number of Y cells in the Harvest grid*/
  protected int m_iNumHarvestYCells;

  /**Number of X cells in the Mortality Episode grid*/
  protected int m_iNumMortEpisodeXCells;

  /**Number of Y cells in the Mortality Episode grid*/
  protected int m_iNumMortEpisodeYCells;

  /**Number of X cells in the Planting grid*/
  protected int m_iNumPlantXCells;

  /**Number of Y cells in the Planting grid*/
  protected int m_iNumPlantYCells;

  /**Length of the plot in the X direction, in meters*/
  protected int m_iPlotLengthX;

  /**Length of the plot in the Y direction, in meters*/
  protected int m_iPlotLengthY;

  /**Number of species*/
  protected int m_iNumSpecies;

  /**Color for displaying harvests*/
  public final static java.awt.Color HARVEST_COLOR = new java.awt.Color(255,
      100, 255);

  /**Color for displaying plantings*/
  public final static java.awt.Color PLANT_COLOR = new java.awt.Color(255, 255,
      100);

  /**Color for displaying mortality episode events*/
  public final static java.awt.Color MORTALITY_EPISODE_COLOR = new java.awt.
      Color(150, 225,
            225);

  /**Help ID string*/
  private String m_sHelpID = "windows.episodic_events_window";

  /**
   * Constructor.  Builds the window.
   * @param oOwner Owner window for this dialog.
   * @param oDisturbanceBehaviors DisturbanceBehaviors object to exchange data with.
   * @param oPlantBehaviors PlantingBehaviors object to exchange data with.
   * @throws ModelException passing through from called methods.
   */
  public EpisodicEventsWindow(JFrame oOwner,
                              DisturbanceBehaviors oDisturbanceBehaviors,
                              PlantingBehaviors oPlantBehaviors) throws
      ModelException {
    super(oOwner, "Edit Episodic Events", true);
    int i;

    m_jParentSize = oOwner.getSize();

    m_oDisturbanceBehaviors = oDisturbanceBehaviors;
    m_oPlantBehaviors = oPlantBehaviors;

    //Copy the data from the DisturbanceBehaviors object
    for (i = 0; i < m_oDisturbanceBehaviors.mp_oHarvestCuts.size(); i++) {
      mp_oHarvestData.add(m_oDisturbanceBehaviors.mp_oHarvestCuts.elementAt(i));
    }
    for (i = 0; i < m_oDisturbanceBehaviors.mp_oMortEpisodes.size(); i++) {
      mp_oMortEpisodeData.add(m_oDisturbanceBehaviors.mp_oMortEpisodes.
                              elementAt(i));
    }

    //Copy the data from the PlantingBehaviors object
    for (i = 0; i < m_oPlantBehaviors.mp_oPlantings.size(); i++) {
      mp_oPlantingData.add(m_oPlantBehaviors.mp_oPlantings.elementAt(i));
    }

    //Get the number of species
    m_iNumSpecies = oDisturbanceBehaviors.m_oManager.GetTreePopulation().
        GetNumberOfSpecies();

    mp_fPlantInitialDiam10s = new float[m_iNumSpecies];
    for (i = 0; i < m_iNumSpecies; i++) {
      mp_fPlantInitialDiam10s[i] = new Float(String.valueOf(m_oPlantBehaviors.
          mp_fInitialDiam10.GetValue().elementAt(i))).floatValue();
    }

    MakeTreeDatasetAndRenderer();
    MakeGUI();
    GetPlotAndGridInfo();
    SetUpCharting();
    DisplayHarvest(null);

    //Help ID
    m_oDisturbanceBehaviors.m_oManager.GetHelpBroker().enableHelpKey(this.
        getRootPane(), m_sHelpID, null);
  }

  /**
   * Sets up the chart and displays any trees.
   * @throws ModelException should not be thrown.
   */
  protected void SetUpCharting() throws ModelException {
    int i, j;

    //Create each dataset with two series - an underlying white layer, and
    //another layer upon which to display the event (we only display
    //them one at a time)
    m_oHarvestDataset = new XYZSimpleDataset(2, m_iNumHarvestXCells,
                                       m_iNumHarvestYCells);
    m_oMortEpisodeDataset = new XYZSimpleDataset(2, m_iNumMortEpisodeXCells,
                                       m_iNumMortEpisodeYCells);
    m_oPlantDataset = new XYZSimpleDataset(2, m_iNumPlantXCells,
                                       m_iNumPlantYCells);
    //Set the first layer of each dataset to all true - this is the white
    //underlayer, and default top layer to all false
    for (i = 0; i < m_iNumHarvestXCells; i++) {
      for (j = 0; j < m_iNumHarvestYCells; j++) {
        m_oHarvestDataset.mp_bData[0][i][j] = true;
        m_oHarvestDataset.mp_bData[1][i][j] = false;
      }
    }
    for (i = 0; i < m_iNumMortEpisodeXCells; i++) {
      for (j = 0; j < m_iNumMortEpisodeYCells; j++) {
        m_oMortEpisodeDataset.mp_bData[0][i][j] = true;
        m_oMortEpisodeDataset.mp_bData[1][i][j] = false;
      }
    }
    for (i = 0; i < m_iNumPlantXCells; i++) {
      for (j = 0; j < m_iNumPlantYCells; j++) {
        m_oPlantDataset.mp_bData[0][i][j] = true;
        m_oPlantDataset.mp_bData[1][i][j] = false;
      }
    }

    //Set up the renderer to display selected cells in the underlayer in white
    m_oRenderer.SetColor(255, 255, 255, 0);

    ChartPanel jChart = DataGrapher.MakeOverlaidMap(null,
        m_oTreeDataset, "East->", "", m_iPlotLengthX,
        m_iPlotLengthY, null, m_oTreeRenderer,
        new java.awt.Dimension(Math.min(600,
                                        m_iPlotLengthX * 3),
                               Math.min(600, m_iPlotLengthY * 3)));
    m_jChartPanel.add(jChart);
    m_oPlot = (XYPlot) jChart.getChart().getPlot();
  }

  /**
   * Create the dataset and renderer for the trees.
   */
  protected void MakeTreeDatasetAndRenderer() {
    TreePopulation oPop = m_oDisturbanceBehaviors.m_oManager.GetTreePopulation();
    Plot oPlot = m_oDisturbanceBehaviors.m_oManager.GetPlot();
    int i,j;

    m_oTreeRenderer = new XYTreeRenderer(oPlot.GetPlotXLength(), oPlot.GetPlotYLength());

    //Split out the trees by species, creating XYZDataItems for each
    Vector[] p_oTreesBySpecies = new Vector[oPop.GetNumberOfSpecies()];
    for (i = 0; i < p_oTreesBySpecies.length; i++) {
      p_oTreesBySpecies[i] = new Vector(0);
    }
    Vector p_oTrees = oPop.GetTrees();
    Tree oTree;
    for (i = 0; i < p_oTrees.size(); i++) {
      oTree = (Tree) p_oTrees.elementAt(i);
      int iXCode = oPop.GetFloatDataCode("X", oTree.m_iSpecies, oTree.m_iType),
          iYCode = oPop.GetFloatDataCode("Y", oTree.m_iSpecies, oTree.m_iType),
          iDiamCode = -1;
      if (oTree.m_iType > TreePopulation.SEEDLING) {
        iDiamCode = oPop.GetFloatDataCode("DBH", oTree.m_iSpecies,
                                          oTree.m_iType);
      }
      if (iXCode > -1 && iYCode > -1 && iDiamCode > -1) {
        p_oTreesBySpecies[oTree.m_iSpecies].add(new XYZDataItem(
            oTree.mp_fFloats[iXCode].doubleValue(),
            oTree.mp_fFloats[iYCode].doubleValue(),
            oTree.mp_fFloats[iDiamCode].doubleValue()));
      }
    }

    //Now add these as series to the dataset - empty series OK because they
    //act as placeholders
    XYZDataItem oItem;
    for (i = 0; i < p_oTreesBySpecies.length; i++) {
      double[][] p_oSeries = new double[3][p_oTreesBySpecies[i].size()];
      for (j = 0; j < p_oTreesBySpecies[i].size(); j++) {
        oItem = (XYZDataItem) p_oTreesBySpecies[i].elementAt(j);
        p_oSeries[0][j] = oItem.fX;
        p_oSeries[1][j] = oItem.fY;
        p_oSeries[2][j] = oItem.fZ;
      }
      m_oTreeDataset.addSeries(oPop.GetSpeciesNameFromCode(i), p_oSeries);
    }
  }

  /**
   * Gets information on the plot and the grids.
   * @throws ModelException passing through from called methods.
   */
  protected void GetPlotAndGridInfo() throws ModelException {
    GUIManager oManager = m_oDisturbanceBehaviors.m_oManager;

    //Get the plot lengths
    Plot oPlot = oManager.GetPlot();
    m_iPlotLengthX = (int) oPlot.GetPlotXLength();
    m_iPlotLengthY = (int) oPlot.GetPlotYLength();

    //Get the harvest grid information
    Grid oGrid = oManager.GetGridByName("Harvest Results");
    if (oGrid == null) {
      throw (new ModelException(ErrorGUI.CANT_FIND_OBJECT, "JAVA",
                                "I couldn't find the Harvest Results grid."));
    }

    m_iNumHarvestXCells = (int) java.lang.Math.ceil(m_iPlotLengthX /
            oGrid.GetXCellLength());
    m_iNumHarvestYCells = (int) java.lang.Math.ceil(m_iPlotLengthY /
            oGrid.GetYCellLength());

    //Get the number of harvests
    m_jNumHarvestEvents.setText(String.valueOf(m_oDisturbanceBehaviors.
                                               mp_oHarvestCuts.size()));

    //Get the mortality episode grid information
    oGrid = oManager.GetGridByName("Mortality Episode Results");
    if (oGrid == null) {
      throw (new ModelException(ErrorGUI.CANT_FIND_OBJECT, "JAVA",
                                "I couldn't find the Mortality Episode Results grid."));
    }
    m_iNumMortEpisodeXCells = (int) java.lang.Math.ceil(m_iPlotLengthX /
            oGrid.GetXCellLength());
    m_iNumMortEpisodeYCells = (int) java.lang.Math.ceil(m_iPlotLengthY /
            oGrid.GetYCellLength());

    //Get the number of mortality episodes
    m_jNumMortEpEvents.setText(String.valueOf(m_oDisturbanceBehaviors.
                                              mp_oMortEpisodes.size()));

    //Get the planting grid information
    oGrid = oManager.GetGridByName("Planting Results");
    if (oGrid == null) {
      throw (new ModelException(ErrorGUI.CANT_FIND_OBJECT, "JAVA",
                                "I couldn't find the Planting Episode Results grid."));
    }
    m_iNumPlantXCells = (int) java.lang.Math.ceil(m_iPlotLengthX /
                                                  oGrid.GetXCellLength());
    m_iNumPlantYCells = (int) java.lang.Math.ceil(m_iPlotLengthY /
                                                  oGrid.GetYCellLength());

    //Get the number of planting events
    m_jNumPlantingEvents.setText(String.valueOf(m_oPlantBehaviors.mp_oPlantings.
                                                size()));
  }

  /**
   * Draws the window.
   * @throws ModelException passing through from called methods.
   */
  protected void MakeGUI() throws ModelException {

    //Set all the label fonts
    m_jHarvestCutAmountTypeLabel.setFont(new ModelFont());
    m_jMortEpCutAmountTypeLabel.setFont(new ModelFont());
    m_jPlantAmountLabel.setFont(new ModelFont(java.awt.Font.BOLD));
    m_jPlantAmountValueLabel.setFont(new ModelFont());
    m_jPlantSpacingLabel.setFont(new ModelFont());
    m_jHarvestCutRange1Amt.setFont(new ModelFont());
    m_jHarvestCutRange2Amt.setFont(new ModelFont());
    m_jHarvestCutRange3Amt.setFont(new ModelFont());
    m_jHarvestCutRange4Amt.setFont(new ModelFont());
    m_jHarvestCutRange1Min.setFont(new ModelFont());
    m_jHarvestCutRange2Min.setFont(new ModelFont());
    m_jHarvestCutRange3Min.setFont(new ModelFont());
    m_jHarvestCutRange4Min.setFont(new ModelFont());
    m_jHarvestCutRange1Max.setFont(new ModelFont());
    m_jHarvestCutRange2Max.setFont(new ModelFont());
    m_jHarvestCutRange3Max.setFont(new ModelFont());
    m_jHarvestCutRange4Max.setFont(new ModelFont());
    m_jMortEpCutRange1Amt.setFont(new ModelFont());
    m_jMortEpCutRange2Amt.setFont(new ModelFont());
    m_jMortEpCutRange3Amt.setFont(new ModelFont());
    m_jMortEpCutRange4Amt.setFont(new ModelFont());
    m_jMortEpCutRange1Min.setFont(new ModelFont());
    m_jMortEpCutRange2Min.setFont(new ModelFont());
    m_jMortEpCutRange3Min.setFont(new ModelFont());
    m_jMortEpCutRange4Min.setFont(new ModelFont());
    m_jMortEpCutRange1Max.setFont(new ModelFont());
    m_jMortEpCutRange2Max.setFont(new ModelFont());
    m_jMortEpCutRange3Max.setFont(new ModelFont());
    m_jMortEpCutRange4Max.setFont(new ModelFont());
    m_jCutTypeLabel.setFont(new ModelFont());
    m_jHarvestNumber.setFont(new ModelFont());
    m_jMortEpNumber.setFont(new ModelFont());
    m_jPlantNumber.setFont(new ModelFont());
    m_jHarvestTimestepLabel.setFont(new ModelFont());
    m_jMortEpTimestepLabel.setFont(new ModelFont());
    m_jPlantTimestepLabel.setFont(new ModelFont());
    m_jNumHarvestEvents.setFont(new ModelFont());
    m_jNumPlantingEvents.setFont(new ModelFont());
    m_jNumMortEpEvents.setFont(new ModelFont());

    TreePopulation p_oPop = m_oDisturbanceBehaviors.m_oManager.
        GetTreePopulation();
    int i;
    mp_jSpeciesPlantPercentages = new JLabel[m_iNumSpecies];
    for (i = 0; i < m_iNumSpecies; i++) {
      mp_jSpeciesPlantPercentages[i] = new JLabel("0");
      mp_jSpeciesPlantPercentages[i].setFont(new ModelFont());
    }
    mp_jSpeciesPlantInitialDiam10 = new JLabel[m_iNumSpecies];
    for (i = 0; i < m_iNumSpecies; i++) {
      mp_jSpeciesPlantInitialDiam10[i] = new JLabel(String.valueOf(
          mp_fPlantInitialDiam10s[i]));
      mp_jSpeciesPlantInitialDiam10[i].setFont(new ModelFont());
    }

    JPanel jDataPanel = new JPanel();
    jDataPanel.setLayout(new BoxLayout(jDataPanel, BoxLayout.PAGE_AXIS));
    //Create a panel down the left side that holds information about the
    //harvest being displayed.  Each of the pieces of data gets its own
    //panel within that panel so it is bundled with its label

    //**********************************
     // Harvest panel
     //**********************************
    JPanel jHarvestPanel = new JPanel();
    jHarvestPanel.setLayout(new BoxLayout(jHarvestPanel, BoxLayout.PAGE_AXIS));

    //Display of number of harvest events and the buttons for moving through them
    JButton jHarvestBack = new JButton(new ModelIcon(15, 15,
        ModelIcon.LEFT_TRIANGLE));
    jHarvestBack.setFont(new ModelFont());
    jHarvestBack.setToolTipText("Show previous harvest event");
    jHarvestBack.setActionCommand("HarvestBack");
    jHarvestBack.addActionListener(this);

    JButton jHarvestForward = new JButton(new ModelIcon(15, 15,
        ModelIcon.RIGHT_TRIANGLE));
    jHarvestForward.setFont(new ModelFont());
    jHarvestForward.setToolTipText("Show next harvest event");
    jHarvestForward.setActionCommand("HarvestForward");
    jHarvestForward.addActionListener(this);

    JLabel jHarvestIntro = new JLabel("Showing harvest"),
        jHarvestLabel = new JLabel("of");
    jHarvestIntro.setFont(new ModelFont());
    jHarvestLabel.setFont(new ModelFont());

    JPanel jNumberPanel = new JPanel();
    FlowLayout jLayout1 = new FlowLayout();
    jLayout1.setAlignment(FlowLayout.CENTER);
    jNumberPanel.setLayout(jLayout1);
    jNumberPanel.add(jHarvestBack);
    jNumberPanel.add(jHarvestIntro);
    jNumberPanel.add(m_jHarvestNumber);
    jNumberPanel.add(jHarvestLabel);
    jNumberPanel.add(m_jNumHarvestEvents);
    jNumberPanel.add(jHarvestForward);
    jNumberPanel.setAlignmentX(java.awt.Component.LEFT_ALIGNMENT);
    jHarvestPanel.add(jNumberPanel);

    //New, delete, and edit buttons
    JButton jNewHarvestButton = new JButton("New");
    jNewHarvestButton.setFont(new ModelFont());
    jNewHarvestButton.setToolTipText("Create a new harvest");
    jNewHarvestButton.setActionCommand("NewHarvest");
    jNewHarvestButton.addActionListener(this);

    JButton jHarvestRemove = new JButton("Delete");
    jHarvestRemove.setFont(new ModelFont());
    jHarvestRemove.setToolTipText("Delete the currently displayed harvest");
    jHarvestRemove.setActionCommand("DeleteHarvest");
    jHarvestRemove.addActionListener(this);

    JButton jHarvestEdit = new JButton("Edit");
    jHarvestEdit.setFont(new ModelFont());
    jHarvestEdit.setToolTipText("Edit the currently displayed harvest");
    jHarvestEdit.setActionCommand("EditHarvest");
    jHarvestEdit.addActionListener(this);

    JPanel jHarvestEditPanel = new JPanel();
    jHarvestEditPanel.add(jNewHarvestButton);
    jHarvestEditPanel.add(jHarvestRemove);
    jHarvestEditPanel.add(jHarvestEdit);
    jHarvestEditPanel.setAlignmentX(java.awt.Component.LEFT_ALIGNMENT);
    jHarvestPanel.add(jHarvestEditPanel);

    //Timestep
    JLabel jTimestepLabel = new JLabel("Timestep:");
    jTimestepLabel.setHorizontalAlignment(SwingConstants.LEFT);
    jTimestepLabel.setFont(new ModelFont(java.awt.Font.BOLD));
    FlowLayout jLayout2 = new FlowLayout();
    jLayout2.setAlignment(FlowLayout.LEFT);
    JPanel jTimestepPanel = new JPanel(jLayout2);
    jTimestepPanel.setAlignmentX(java.awt.Component.LEFT_ALIGNMENT);
    jTimestepPanel.add(jTimestepLabel);
    jTimestepPanel.add(m_jHarvestTimestepLabel);
    jHarvestPanel.add(jTimestepPanel);

    //Species
    JLabel jSpeciesLabel = new JLabel("Species applied to:");
    jSpeciesLabel.setAlignmentX(java.awt.Component.LEFT_ALIGNMENT);
    jSpeciesLabel.setFont(new ModelFont(java.awt.Font.BOLD));
    JPanel jSpeciesPanel = new JPanel();
    jSpeciesPanel.setLayout(new BoxLayout(jSpeciesPanel, BoxLayout.PAGE_AXIS));
    jSpeciesPanel.setAlignmentX(java.awt.Component.LEFT_ALIGNMENT);
    jSpeciesPanel.add(jSpeciesLabel);
    JList jSpeciesList = new JList(m_jHarvestSpeciesList);
    jSpeciesList.setFont(new ModelFont());
    jSpeciesList.setVisibleRowCount(4);
    jSpeciesList.setBackground(java.awt.SystemColor.control);
    m_jHarvestSpeciesList.addElement("N/A");
    JScrollPane jPane = new JScrollPane(jSpeciesList);
    jPane.setAlignmentX(java.awt.Component.LEFT_ALIGNMENT);
    jSpeciesPanel.add(jPane);
    jHarvestPanel.add(jSpeciesPanel);

    //Cut type
    JLabel jCutLabel = new JLabel("Cut type:");
    jCutLabel.setFont(new ModelFont(java.awt.Font.BOLD));
    FlowLayout jLayout3 = new FlowLayout();
    jLayout3.setAlignment(FlowLayout.LEFT);
    JPanel jCutTypePanel = new JPanel(jLayout3);
    jCutTypePanel.setAlignmentX(java.awt.Component.LEFT_ALIGNMENT);
    jCutTypePanel.add(jCutLabel);
    jCutTypePanel.add(m_jCutTypeLabel);
    jHarvestPanel.add(jCutTypePanel);

    //Cut amount type
    JLabel jCutAmtLabel = new JLabel("Cut amount type:");
    jCutAmtLabel.setFont(new ModelFont(java.awt.Font.BOLD));
    FlowLayout jLayout4 = new FlowLayout();
    jLayout4.setAlignment(FlowLayout.LEFT);
    JPanel jCutAmountTypePanel = new JPanel(jLayout4);
    jCutAmountTypePanel.setAlignmentX(java.awt.Component.LEFT_ALIGNMENT);
    jCutAmountTypePanel.add(jCutAmtLabel);
    jCutAmountTypePanel.add(m_jHarvestCutAmountTypeLabel);
    jHarvestPanel.add(jCutAmountTypePanel);

    //Cut ranges
    JPanel jCutRangePanel = new JPanel(new GridLayout(3, 5));
    jCutRangePanel.setAlignmentX(java.awt.Component.LEFT_ALIGNMENT);
    JLabel jTempLabel = new JLabel("Min");
    jTempLabel.setFont(new ModelFont(java.awt.Font.BOLD));
    jCutRangePanel.add(jTempLabel);
    jCutRangePanel.add(m_jHarvestCutRange1Min);
    jCutRangePanel.add(m_jHarvestCutRange2Min);
    jCutRangePanel.add(m_jHarvestCutRange3Min);
    jCutRangePanel.add(m_jHarvestCutRange4Min);
    jTempLabel = new JLabel("Max");
    jTempLabel.setFont(new ModelFont(java.awt.Font.BOLD));
    jCutRangePanel.add(jTempLabel);
    jCutRangePanel.add(m_jHarvestCutRange1Max);
    jCutRangePanel.add(m_jHarvestCutRange2Max);
    jCutRangePanel.add(m_jHarvestCutRange3Max);
    jCutRangePanel.add(m_jHarvestCutRange4Max);
    jTempLabel = new JLabel("Amt");
    jTempLabel.setFont(new ModelFont(java.awt.Font.BOLD));
    jCutRangePanel.add(jTempLabel);
    jCutRangePanel.add(m_jHarvestCutRange1Amt);
    jCutRangePanel.add(m_jHarvestCutRange2Amt);
    jCutRangePanel.add(m_jHarvestCutRange3Amt);
    jCutRangePanel.add(m_jHarvestCutRange4Amt);
    jTempLabel = new JLabel("Diameter Range(s) to cut:");
    jTempLabel.setFont(new ModelFont(java.awt.Font.BOLD));
    JPanel jTempPanel2 = new JPanel();
    jTempPanel2.setLayout(new BoxLayout(jTempPanel2, BoxLayout.PAGE_AXIS));
    jTempPanel2.add(jTempLabel);
    jTempPanel2.add(jCutRangePanel);
    jTempPanel2.setBorder(BorderFactory.createEmptyBorder(0, 5, 5, 5));
    jHarvestPanel.add(jTempPanel2);

    //**********************************
     // Planting panel
     //**********************************
    JPanel jPlantPanel = new JPanel();
    jPlantPanel.setLayout(new BoxLayout(jPlantPanel, BoxLayout.PAGE_AXIS));

    //Display of number of planting events and the buttons for moving through them
    JButton jPlantBack = new JButton(new ModelIcon(15, 15,
        ModelIcon.LEFT_TRIANGLE));
    jPlantBack.setFont(new ModelFont());
    jPlantBack.setToolTipText("Show previous planting event");
    jPlantBack.setActionCommand("PlantBack");
    jPlantBack.addActionListener(this);

    JButton jPlantForward = new JButton(new ModelIcon(15, 15,
        ModelIcon.RIGHT_TRIANGLE));
    jPlantForward.setFont(new ModelFont());
    jPlantForward.setToolTipText("Show next planting event");
    jPlantForward.setActionCommand("PlantForward");
    jPlantForward.addActionListener(this);

    JLabel jPlantIntro = new JLabel("Showing planting"),
        jPlantLabel = new JLabel("of");
    jPlantIntro.setFont(new ModelFont());
    jPlantLabel.setFont(new ModelFont());

    //Center everything
    JPanel jNumberPanel2 = new JPanel();
    FlowLayout jLayout5 = new FlowLayout();
    jLayout5.setAlignment(FlowLayout.CENTER);
    jNumberPanel2.setLayout(jLayout5);
    jNumberPanel2.setAlignmentX(java.awt.Component.LEFT_ALIGNMENT);
    jNumberPanel2.add(jPlantBack);
    jNumberPanel2.add(jPlantIntro);
    jNumberPanel2.add(m_jPlantNumber);
    jNumberPanel2.add(jPlantLabel);
    jNumberPanel2.add(m_jNumPlantingEvents);
    jNumberPanel2.add(jPlantForward);

    jPlantPanel.add(jNumberPanel2);

    //New, delete, and edit buttons
    JButton jNewPlantButton = new JButton("New");
    jNewPlantButton.setFont(new ModelFont());
    jNewPlantButton.setToolTipText("Create a new planting");
    jNewPlantButton.setActionCommand("NewPlant");
    jNewPlantButton.addActionListener(this);

    JButton jPlantRemove = new JButton("Delete");
    jPlantRemove.setFont(new ModelFont());
    jPlantRemove.setToolTipText("Delete the currently displayed planting");
    jPlantRemove.setActionCommand("DeletePlant");
    jPlantRemove.addActionListener(this);

    JButton jPlantEdit = new JButton("Edit");
    jPlantEdit.setFont(new ModelFont());
    jPlantEdit.setToolTipText("Edit the currently displayed planting");
    jPlantEdit.setActionCommand("EditPlant");
    jPlantEdit.addActionListener(this);

    JPanel jPlantEditPanel = new JPanel();
    jPlantEditPanel.add(jNewPlantButton);
    jPlantEditPanel.add(jPlantRemove);
    jPlantEditPanel.add(jPlantEdit);
    jPlantEditPanel.setAlignmentX(java.awt.Component.LEFT_ALIGNMENT);
    jPlantPanel.add(jPlantEditPanel);

    //Timestep
    FlowLayout jLayout6 = new FlowLayout();
    jLayout6.setAlignment(FlowLayout.LEFT);
    JPanel jTimestepPanel2 = new JPanel(jLayout6);
    jTimestepPanel2.setAlignmentX(java.awt.Component.LEFT_ALIGNMENT);
    JLabel jTimestepLabel2 = new JLabel("Timestep:");
    jTimestepLabel2.setFont(new ModelFont(java.awt.Font.BOLD));
    jTimestepPanel2.add(jTimestepLabel2);
    jTimestepPanel2.add(m_jPlantTimestepLabel);
    jPlantPanel.add(jTimestepPanel2);

    //Plant spacing type
    jTempLabel = new JLabel("Planting spacing:");
    jTempLabel.setFont(new ModelFont(java.awt.Font.BOLD));
    FlowLayout jLayout7 = new FlowLayout();
    jLayout7.setAlignment(FlowLayout.LEFT);
    JPanel jPlantSpacePanel = new JPanel(jLayout7);
    jPlantSpacePanel.setAlignmentX(java.awt.Component.LEFT_ALIGNMENT);
    jPlantSpacePanel.add(jTempLabel);
    jPlantSpacePanel.add(m_jPlantSpacingLabel);
    jPlantPanel.add(jPlantSpacePanel);

    //Planting amount
    jTempLabel.setFont(new ModelFont(java.awt.Font.BOLD));
    FlowLayout jLayout8 = new FlowLayout();
    jLayout8.setAlignment(FlowLayout.LEFT);
    JPanel jPlantAmountPanel = new JPanel(jLayout8);
    jPlantAmountPanel.setAlignmentX(java.awt.Component.LEFT_ALIGNMENT);
    jPlantAmountPanel.add(m_jPlantAmountLabel);
    jPlantAmountPanel.add(m_jPlantAmountValueLabel);
    jPlantPanel.add(jPlantAmountPanel);

    //Planting percentages and initial diam10s
    JPanel jPercentagePanel = new JPanel(new SpringLayout());
    jPercentagePanel.setAlignmentX(java.awt.Component.LEFT_ALIGNMENT);
    jTempLabel = new JLabel("Species");
    jTempLabel.setFont(new ModelFont(java.awt.Font.BOLD));
    jPercentagePanel.add(jTempLabel);
    jTempLabel = new JLabel("% Total");
    jTempLabel.setFont(new ModelFont(java.awt.Font.BOLD));
    jPercentagePanel.add(jTempLabel);
    MultilineLabel jTempMultiLine = new MultilineLabel("Diam @\n10 cm");
    jTempMultiLine.setFont(new ModelFont(java.awt.Font.BOLD));
    jPercentagePanel.add(jTempMultiLine);
//    jTempLabel = new JLabel("Diam @ 10 cm");
//    jTempLabel.setFont(new ModelFont(java.awt.Font.BOLD));
//    jPercentagePanel.add(jTempLabel);

    for (i = 0; i < m_iNumSpecies; i++) {
      jTempLabel = new JLabel(p_oPop.GetSpeciesNameFromCode(i).replace('_', ' '));
      jTempLabel.setFont(new ModelFont());
      jPercentagePanel.add(jTempLabel);
      jPercentagePanel.add(mp_jSpeciesPlantPercentages[i]);
      jPercentagePanel.add(mp_jSpeciesPlantInitialDiam10[i]);
    }
    SpringUtilities.makeCompactGrid(jPercentagePanel, m_iNumSpecies + 1, 3, 3,
                                    3, 3, 3);

    jPlantPanel.add(jPercentagePanel);

    //Button to edit species initial diameter at 10 cm
    JButton jEditDiam10Button = new JButton("Edit initial diameter at 10 cm");
    jEditDiam10Button.setFont(new ModelFont());
    jEditDiam10Button.setActionCommand("EditDiam10");
    jEditDiam10Button.setToolTipText("Edit initial diameter at 10 cm");
    jEditDiam10Button.addActionListener(this);
    jPlantPanel.add(jEditDiam10Button);

    //**********************************
     // Mortality episode panel
     //**********************************
    JPanel jMortEpPanel = new JPanel();
    jMortEpPanel.setLayout(new BoxLayout(jMortEpPanel, BoxLayout.PAGE_AXIS));

    //Display of number of mortality eposide events and the buttons for moving through them
    JButton jMortEpBack = new JButton(new ModelIcon(15, 15,
        ModelIcon.LEFT_TRIANGLE));
    jMortEpBack.setFont(new ModelFont());
    jMortEpBack.setToolTipText("Show previous mortality episode");
    jMortEpBack.setActionCommand("MortEpisodeBack");
    jMortEpBack.addActionListener(this);

    JButton jMortEpForward = new JButton(new ModelIcon(15, 15,
        ModelIcon.RIGHT_TRIANGLE));
    jMortEpForward.setFont(new ModelFont());
    jMortEpForward.setToolTipText("Show next mortality episode");
    jMortEpForward.setActionCommand("MortEpisodeForward");
    jMortEpForward.addActionListener(this);

    JLabel jMortEpIntro = new JLabel("Showing episode"),
        jMortEpLabel = new JLabel("of");
    jMortEpIntro.setFont(new ModelFont());
    jMortEpLabel.setFont(new ModelFont());

    JPanel jMortEpNumberPanel = new JPanel();
    jLayout1.setAlignment(FlowLayout.CENTER);
    jMortEpNumberPanel.setLayout(jLayout1);
    jMortEpNumberPanel.add(jMortEpBack);
    jMortEpNumberPanel.add(jMortEpIntro);
    jMortEpNumberPanel.add(m_jMortEpNumber);
    jMortEpNumberPanel.add(jMortEpLabel);
    jMortEpNumberPanel.add(m_jNumMortEpEvents);
    jMortEpNumberPanel.add(jMortEpForward);
    jMortEpNumberPanel.setAlignmentX(java.awt.Component.LEFT_ALIGNMENT);
    jMortEpPanel.add(jMortEpNumberPanel);

    //New, delete, and edit buttons
    JButton jNewMortEpButton = new JButton("New");
    jNewMortEpButton.setFont(new ModelFont());
    jNewMortEpButton.setToolTipText(
        "Create a new mortality episode");
    jNewMortEpButton.setActionCommand("NewMortalityEpisode");
    jNewMortEpButton.addActionListener(this);

    JButton jMortEpRemove = new JButton("Delete");
    jMortEpRemove.setFont(new ModelFont());
    jMortEpRemove.setToolTipText(
        "Delete the currently displayed mortality episode");
    jMortEpRemove.setActionCommand("DeleteMortalityEpisode");
    jMortEpRemove.addActionListener(this);

    JButton jMortEpEdit = new JButton("Edit");
    jMortEpEdit.setFont(new ModelFont());
    jMortEpEdit.setToolTipText("Edit the currently displayed mortality episode");
    jMortEpEdit.setActionCommand("EditMortalityEpisode");
    jMortEpEdit.addActionListener(this);

    JPanel jMortEpEditPanel = new JPanel();
    jMortEpEditPanel.add(jNewMortEpButton);
    jMortEpEditPanel.add(jMortEpRemove);
    jMortEpEditPanel.add(jMortEpEdit);
    jMortEpEditPanel.setAlignmentX(java.awt.Component.LEFT_ALIGNMENT);
    jMortEpPanel.add(jMortEpEditPanel);

    //Timestep
    JLabel jMortEpTimestepLabel = new JLabel("Timestep:");
    jMortEpTimestepLabel.setHorizontalAlignment(SwingConstants.LEFT);
    jMortEpTimestepLabel.setFont(new ModelFont(java.awt.Font.BOLD));
    FlowLayout jLayout12 = new FlowLayout();
    jLayout12.setAlignment(FlowLayout.LEFT);
    JPanel jMortEpTimestepPanel = new JPanel(jLayout12);
    jMortEpTimestepPanel.setAlignmentX(java.awt.Component.LEFT_ALIGNMENT);
    jMortEpTimestepPanel.add(jMortEpTimestepLabel);
    jMortEpTimestepPanel.add(m_jMortEpTimestepLabel);
    jMortEpPanel.add(jMortEpTimestepPanel);

    //Species
    JLabel jMortEpSpeciesLabel = new JLabel("Species applied to:");
    jMortEpSpeciesLabel.setAlignmentX(java.awt.Component.LEFT_ALIGNMENT);
    jMortEpSpeciesLabel.setFont(new ModelFont(java.awt.Font.BOLD));
    JPanel jMortEpSpeciesPanel = new JPanel();
    jMortEpSpeciesPanel.setLayout(new BoxLayout(jMortEpSpeciesPanel,
                                                BoxLayout.PAGE_AXIS));
    jMortEpSpeciesPanel.setAlignmentX(java.awt.Component.LEFT_ALIGNMENT);
    jMortEpSpeciesPanel.add(jMortEpSpeciesLabel);
    JList jMortEpSpeciesList = new JList(m_jMortEpSpeciesList);
    jMortEpSpeciesList.setFont(new ModelFont());
    jMortEpSpeciesList.setVisibleRowCount(4);
    jMortEpSpeciesList.setBackground(java.awt.SystemColor.control);
    m_jMortEpSpeciesList.addElement("N/A");
    JScrollPane jMortEpPane = new JScrollPane(jMortEpSpeciesList);
    jMortEpPane.setAlignmentX(java.awt.Component.LEFT_ALIGNMENT);
    jMortEpSpeciesPanel.add(jMortEpPane);
    jMortEpPanel.add(jMortEpSpeciesPanel);

    //Cut amount type
    JLabel jMortEpCutAmtLabel = new JLabel("Amount of mortality type:");
    jMortEpCutAmtLabel.setFont(new ModelFont(java.awt.Font.BOLD));
    FlowLayout jLayout14 = new FlowLayout();
    jLayout14.setAlignment(FlowLayout.LEFT);
    JPanel jMortEpCutAmountTypePanel = new JPanel(jLayout14);
    jMortEpCutAmountTypePanel.setAlignmentX(java.awt.Component.
                                            LEFT_ALIGNMENT);
    jMortEpCutAmountTypePanel.add(jMortEpCutAmtLabel);
    jMortEpCutAmountTypePanel.add(m_jMortEpCutAmountTypeLabel);
    jMortEpPanel.add(jMortEpCutAmountTypePanel);

    //Cut ranges
    JPanel jMortEpCutRangePanel = new JPanel(new GridLayout(3, 5));
    jMortEpCutRangePanel.setAlignmentX(java.awt.Component.LEFT_ALIGNMENT);
    jTempLabel = new JLabel("Min");
    jTempLabel.setFont(new ModelFont(java.awt.Font.BOLD));
    jMortEpCutRangePanel.add(jTempLabel);
    jMortEpCutRangePanel.add(m_jMortEpCutRange1Min);
    jMortEpCutRangePanel.add(m_jMortEpCutRange2Min);
    jMortEpCutRangePanel.add(m_jMortEpCutRange3Min);
    jMortEpCutRangePanel.add(m_jMortEpCutRange4Min);
    jTempLabel = new JLabel("Max");
    jTempLabel.setFont(new ModelFont(java.awt.Font.BOLD));
    jMortEpCutRangePanel.add(jTempLabel);
    jMortEpCutRangePanel.add(m_jMortEpCutRange1Max);
    jMortEpCutRangePanel.add(m_jMortEpCutRange2Max);
    jMortEpCutRangePanel.add(m_jMortEpCutRange3Max);
    jMortEpCutRangePanel.add(m_jMortEpCutRange4Max);
    jTempLabel = new JLabel("Amt");
    jTempLabel.setFont(new ModelFont(java.awt.Font.BOLD));
    jMortEpCutRangePanel.add(jTempLabel);
    jMortEpCutRangePanel.add(m_jMortEpCutRange1Amt);
    jMortEpCutRangePanel.add(m_jMortEpCutRange2Amt);
    jMortEpCutRangePanel.add(m_jMortEpCutRange3Amt);
    jMortEpCutRangePanel.add(m_jMortEpCutRange4Amt);
    jTempLabel = new JLabel("Diameter Range(s) to kill:");
    jTempLabel.setFont(new ModelFont(java.awt.Font.BOLD));
    jTempPanel2 = new JPanel();
    jTempPanel2.setLayout(new BoxLayout(jTempPanel2, BoxLayout.PAGE_AXIS));
    jTempPanel2.add(jTempLabel);
    jTempPanel2.add(jMortEpCutRangePanel);
    jTempPanel2.setBorder(BorderFactory.createEmptyBorder(0, 5, 5, 5));
    jMortEpPanel.add(jTempPanel2);

    //**********************************
     // Legend panel
     //**********************************
    JPanel jLegendPanel = new JPanel(new GridLayout(0, 1));
    if (p_oPop.GetTrees().size() == 0) {
      JLabel jTitle = new JLabel("No trees to display.");
      jTitle.setFont(new ModelFont());
      jLegendPanel.add(jTitle);
    }
    DefaultDrawingSupplier oDrawer = new DefaultDrawingSupplier();
    for (i = 0; i < m_iNumSpecies; i++) {
      JPanel jTemp = new JPanel(new FlowLayout(FlowLayout.LEFT));
      java.awt.Color jSpeciesColor = (java.awt.Color) oDrawer.getNextPaint();
      JButton jColorButton = new JButton(new ModelIcon(15, 15,
          ModelIcon.RECTANGLE, jSpeciesColor));
      jColorButton.setContentAreaFilled(false);
      jColorButton.setPreferredSize(new Dimension(15, 15));
      m_oTreeRenderer.setSeriesPaint(i, jSpeciesColor);
//      JLabel jTempSpLabel = new JLabel(p_oPop.GetSpeciesNameFromCode(i).replace(
//          '_', ' '));
//      jTempSpLabel.setFont(new ModelFont());
      jTempMultiLine = new MultilineLabel(p_oPop.GetSpeciesNameFromCode(i).
                                          replace(
          '_', '\n'));
      jTempMultiLine.setFont(new ModelFont());
      jTemp.add(jColorButton);
      jTemp.add(jTempMultiLine);
      jLegendPanel.add(jTemp);
    }

    //**********************************
    // Put it all together
    //**********************************
    m_jTabs.addTab("Harvests", jHarvestPanel);
    m_jTabs.setFont(new ModelFont(java.awt.Font.BOLD));
    m_jTabs.addTab("Plantings", jPlantPanel);
    m_jTabs.addTab("Mortality Episodes", jMortEpPanel);
    m_jTabs.addChangeListener(this);

    jDataPanel.setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));
    jDataPanel.add(m_jTabs);

    JPanel jContentPanel = new JPanel(new BorderLayout());
    jContentPanel.setLayout(new BorderLayout());
    jContentPanel.add(jDataPanel, BorderLayout.WEST);
    jContentPanel.add(m_jChartPanel, BorderLayout.CENTER);
    jContentPanel.add(jLegendPanel, BorderLayout.EAST);

    getContentPane().setLayout(new BorderLayout());
    getContentPane().add(new JScrollPane(jContentPanel), BorderLayout.CENTER);
    getContentPane().add(new OKCancelButtonPanel(this,
                                                 m_oDisturbanceBehaviors.
                                                 m_oManager.
                                                 GetHelpBroker(), m_sHelpID),
                         BorderLayout.SOUTH);

    /*    getContentPane().setLayout(new BorderLayout());
        getContentPane().add(jDataPanel, BorderLayout.WEST);
        getContentPane().add(new OKCancelButtonPanel(this,
                                                     m_oDisturbanceBehaviors.
                                                     m_oManager.
     GetHelpBroker(), m_sHelpID),
                             BorderLayout.SOUTH);
        getContentPane().add(m_jChartPanel, BorderLayout.CENTER);
        getContentPane().add(jLegendPanel, BorderLayout.EAST); */
  }

  /**
   * Displays the next harvest when the ">>" button is pressed.  If the last
   * harvest is being displayed, then nothing changes.
   * @throws ModelException Passing through from called methods.
   */
  protected void DisplayNextHarvest() throws ModelException {
    int iCurrentHarvest = new Integer(m_jHarvestNumber.getText()).intValue(),
        iTotalHarvests = new Integer(m_jNumHarvestEvents.getText()).
        intValue();

    if (iCurrentHarvest < iTotalHarvests) {
      //Display the next harvest event - the vector index is already right
      //since we add one when displaying
      HarvestData oHarvest = (HarvestData) mp_oHarvestData.elementAt(
          iCurrentHarvest);
      DisplayHarvest(oHarvest);

      //Set the display number
      iCurrentHarvest++;
      m_jHarvestNumber.setText(String.valueOf(iCurrentHarvest));
    }
  }

  /**
   * Displays the next mortality episode when the next button is pressed.  If
   * the last episode is being displayed, then nothing changes.
   * @throws ModelException Passing through from called methods.
   */
  protected void DisplayNextMortalityEpisode() throws ModelException {
    int iCurrentEpisode = new Integer(m_jMortEpNumber.getText()).
        intValue(),
        iTotalEpisodes = new Integer(m_jNumMortEpEvents.getText()).
        intValue();

    if (iCurrentEpisode < iTotalEpisodes) {
      //Display the next mortality episode - the vector index is already right
      //since we add one when displaying
      HarvestData oEpisode = (HarvestData) mp_oMortEpisodeData.elementAt(
          iCurrentEpisode);
      DisplayMortalityEpisode(oEpisode);

      //Set the display number
      iCurrentEpisode++;
      m_jMortEpNumber.setText(String.valueOf(iCurrentEpisode));
    }
  }

  /**
   * Displays the next planting event when the ">>" button is pressed.  If the
   * last planting is being displayed, then nothing changes.
   * @throws ModelException Passing through from called methods.
   */
  protected void DisplayNextPlanting() throws ModelException {
    int iCurrentPlanting = new Integer(m_jPlantNumber.getText()).intValue(),
        iTotalPlantings = new Integer(m_jNumPlantingEvents.getText()).
        intValue();

    if (iCurrentPlanting < iTotalPlantings) {
      //Display the next plant event - the vector index is already right
      //since we add one when displaying
      PlantingData oPlanting = (PlantingData) mp_oPlantingData.elementAt(
          iCurrentPlanting);
      DisplayPlanting(oPlanting);

      //Set the display number
      iCurrentPlanting++;
      m_jPlantNumber.setText(String.valueOf(iCurrentPlanting));
    }
  }

  /**
   * Displays the next harvest when the previous button is pressed.  If the
   * first harvest is being displayed, then nothing changes.
   * @throws ModelException Passing through from called methods.
   */
  protected void DisplayPreviousHarvest() throws ModelException {
    int iCurrentHarvest = new Integer(m_jHarvestNumber.getText()).intValue();

    if (iCurrentHarvest > 1) {
      //Display the previous harvest event - subtract two since the vector
      //index is 0-based
      HarvestData oHarvest = (HarvestData) mp_oHarvestData.elementAt(
          iCurrentHarvest - 2);
      DisplayHarvest(oHarvest);

      //Set the display number
      iCurrentHarvest--;
      m_jHarvestNumber.setText(String.valueOf(iCurrentHarvest));
    }
  }

  /**
   * Displays the previous mortality episode when the previous button is
   * pressed.  If the first episode is being displayed, then nothing changes.
   * @throws ModelException Passing through from called methods.
   */
  protected void DisplayPreviousMortalityEpisode() throws ModelException {
    int iCurrentEpisode = new Integer(m_jMortEpNumber.getText()).
        intValue();

    if (iCurrentEpisode > 1) {
      //Display the previous harvest event - subtract two since the vector
      //index is 0-based
      HarvestData oEpisode = (HarvestData) mp_oMortEpisodeData.elementAt(
          iCurrentEpisode - 2);
      DisplayMortalityEpisode(oEpisode);

      //Set the display number
      iCurrentEpisode--;
      m_jMortEpNumber.setText(String.valueOf(iCurrentEpisode));
    }
  }

  /**
   * Displays the next plant event when the previous button is pressed.  If the
   * first planting is being displayed, then nothing changes.
   * @throws ModelException Passing through from called methods.
   */
  protected void DisplayPreviousPlanting() throws ModelException {
    int iCurrentPlanting = new Integer(m_jPlantNumber.getText()).intValue();

    if (iCurrentPlanting > 1) {
      //Display the previous planting event - subtract two since the vector
      //index is 0-based
      PlantingData oPlanting = (PlantingData) mp_oPlantingData.elementAt(
          iCurrentPlanting - 2);
      DisplayPlanting(oPlanting);

      //Set the display number
      iCurrentPlanting--;
      m_jPlantNumber.setText(String.valueOf(iCurrentPlanting));
    }
  }

  /**
   * Fired when the tabbed pane tab is changed.  This will determine which
   * tab is selected and make sure that the current event on that tab is
   * displayed on the chart.
   * @param e ChangeEvent object.
   */
  public void stateChanged(ChangeEvent e) {
    try {
      if (e.getSource() == m_jTabs) {
        int iSelectedPane = m_jTabs.getSelectedIndex(), iSelectedEvent;
        if (0 == iSelectedPane) {
          //Harvest pane selected
          iSelectedEvent = new Integer(m_jHarvestNumber.getText()).intValue();
          if (iSelectedEvent > 0) {
            HarvestData oData = (HarvestData) mp_oHarvestData.elementAt(
                iSelectedEvent - 1);
            DisplayHarvest(oData);
          }
          else {
            DisplayHarvest(null);
          }
        }
        else if (1 == iSelectedPane) {
          //Planting pane selected
          iSelectedEvent = new Integer(m_jPlantNumber.getText()).intValue();
          if (iSelectedEvent > 0) {
            PlantingData oData = (PlantingData) mp_oPlantingData.elementAt(
                iSelectedEvent - 1);
            DisplayPlanting(oData);
          }
          else {
            DisplayPlanting(null);
          }
        }
        else if (2 == iSelectedPane) {
          //Mortality episode pane selected
          iSelectedEvent = new Integer(m_jMortEpNumber.getText()).intValue();
          if (iSelectedEvent > 0) {
            HarvestData oData = (HarvestData) mp_oMortEpisodeData.elementAt(
                iSelectedEvent - 1);
            DisplayMortalityEpisode(oData);
          }
          else {
            DisplayMortalityEpisode(null);
          }
        }
      }
    }
    catch (ModelException oErr) {
      ErrorGUI oHandler = new ErrorGUI(this);
      oHandler.WriteErrorMessage(oErr);
    }
  }

  /**
   * Sizes a child window to fit within the bounds of this window.
   * @param jChildWindow JDialog The window to size.
   * @return JDialog The sized window.
   */
  private JDialog SizeChildWindow(JDialog jChildWindow) {
    //Resize if too big
    if (jChildWindow.getSize().height > m_jParentSize.height ||
        jChildWindow.getSize().width > m_jParentSize.width) {

      int iWidth = java.lang.Math.min(jChildWindow.getSize().width,
                                      m_jParentSize.width - 10);
      int iHeight = java.lang.Math.min(jChildWindow.getSize().height,
                                       m_jParentSize.height - 10);

      jChildWindow.setBounds(jChildWindow.getBounds().x,
                             jChildWindow.getBounds().y, iWidth, iHeight);
    }

    return jChildWindow;
  }

  /**
   * Responds to window events.
   * @param oEvent Event triggering this call.
   */
  public void actionPerformed(ActionEvent oEvent) {
    if (oEvent.getActionCommand().equals("OK")) {

      AddFinishedData();

      //Close this window
      this.setVisible(false);
      this.dispose();
    }
    else if (oEvent.getActionCommand().equals("Cancel")) {
      //Close this window
      this.setVisible(false);
      this.dispose();
    }
    else if (oEvent.getActionCommand().equals("HarvestForward")) {
      try {
        DisplayNextHarvest();
      }
      catch (ModelException oErr) {
        ErrorGUI oHandler = new ErrorGUI(this);
        oHandler.WriteErrorMessage(oErr);
      }
    }
    else if (oEvent.getActionCommand().equals("HarvestBack")) {
      try {
        DisplayPreviousHarvest();
      }
      catch (ModelException oErr) {
        ErrorGUI oHandler = new ErrorGUI(this);
        oHandler.WriteErrorMessage(oErr);
      }
    }
    else if (oEvent.getActionCommand().equals("MortEpisodeForward")) {
      try {
        DisplayNextMortalityEpisode();
      }
      catch (ModelException oErr) {
        ErrorGUI oHandler = new ErrorGUI(this);
        oHandler.WriteErrorMessage(oErr);
      }
    }
    else if (oEvent.getActionCommand().equals("MortEpisodeBack")) {
      try {
        DisplayPreviousMortalityEpisode();
      }
      catch (ModelException oErr) {
        ErrorGUI oHandler = new ErrorGUI(this);
        oHandler.WriteErrorMessage(oErr);
      }
    }
    else if (oEvent.getActionCommand().equals("PlantForward")) {
      try {
        DisplayNextPlanting();
      }
      catch (ModelException oErr) {
        ErrorGUI oHandler = new ErrorGUI(this);
        oHandler.WriteErrorMessage(oErr);
      }
    }
    else if (oEvent.getActionCommand().equals("PlantBack")) {
      try {
        DisplayPreviousPlanting();
      }
      catch (ModelException oErr) {
        ErrorGUI oHandler = new ErrorGUI(this);
        oHandler.WriteErrorMessage(oErr);
      }
    }
    else if (oEvent.getActionCommand().equals("NewHarvest")) {
      try {
        HarvestEdit oWindow = new HarvestEdit(this);
        oWindow.pack();
        oWindow = (HarvestEdit) SizeChildWindow(oWindow);
        oWindow.setLocationRelativeTo(null);
        oWindow.setVisible(true);
      }
      catch (ModelException oErr) {
        ErrorGUI oHandler = new ErrorGUI(this);
        oHandler.WriteErrorMessage(oErr);
      }
    }
    else if (oEvent.getActionCommand().equals("EditHarvest")) {
      try {

        //Get the harvest object to edit
        int iHarvestNumber = new Integer(m_jHarvestNumber.getText()).intValue();
        iHarvestNumber--; //Since the display is always one more than the index
        if (iHarvestNumber >= 0) {
          HarvestData oHarvest = (HarvestData) mp_oHarvestData.elementAt(
              iHarvestNumber);

          HarvestEdit oWindow = new HarvestEdit(this, oHarvest);
          oWindow.pack();
          oWindow = (HarvestEdit) SizeChildWindow(oWindow);
          oWindow.setLocationRelativeTo(null);
          oWindow.setVisible(true);
        }
        else {
          HarvestEdit oWindow = new HarvestEdit(this);
          oWindow.pack();
          oWindow = (HarvestEdit) SizeChildWindow(oWindow);
          oWindow.setLocationRelativeTo(null);
          oWindow.setVisible(true);
        }
      }
      catch (ModelException oErr) {
        ErrorGUI oHandler = new ErrorGUI(this);
        oHandler.WriteErrorMessage(oErr);
      }
    }

    else if (oEvent.getActionCommand().equals("NewPlant")) {
      try {
        PlantEdit oWindow = new PlantEdit(this);
        oWindow.pack();
        oWindow = (PlantEdit) SizeChildWindow(oWindow);
        oWindow.setLocationRelativeTo(null);
        oWindow.setVisible(true);
      }
      catch (ModelException oErr) {
        ErrorGUI oHandler = new ErrorGUI(this);
        oHandler.WriteErrorMessage(oErr);
      }
    }
    else if (oEvent.getActionCommand().equals("EditPlant")) {
      try {

        //Get the harvest object to edit
        int iPlantNumber = new Integer(m_jPlantNumber.getText()).intValue();
        iPlantNumber--; //Since the display is always one more than the index
        if (iPlantNumber >= 0) {
          PlantingData oPlanting = (PlantingData) mp_oPlantingData.elementAt(
              iPlantNumber);

          PlantEdit oWindow = new PlantEdit(this, oPlanting);
          oWindow.pack();
          oWindow = (PlantEdit) SizeChildWindow(oWindow);
          oWindow.setLocationRelativeTo(null);
          oWindow.setVisible(true);
        }
        else {
          PlantEdit oWindow = new PlantEdit(this);
          oWindow.pack();
          oWindow = (PlantEdit) SizeChildWindow(oWindow);
          oWindow.setLocationRelativeTo(null);
          oWindow.setVisible(true);
        }
      }
      catch (ModelException oErr) {
        ErrorGUI oHandler = new ErrorGUI(this);
        oHandler.WriteErrorMessage(oErr);
      }
    }

    else if (oEvent.getActionCommand().equals("NewMortalityEpisode")) {
      try {
        MortalityEpisodeEdit oWindow = new MortalityEpisodeEdit(this);
        oWindow.pack();
        oWindow = (MortalityEpisodeEdit) SizeChildWindow(oWindow);
        oWindow.setLocationRelativeTo(null);
        oWindow.setVisible(true);
      }
      catch (ModelException oErr) {
        ErrorGUI oHandler = new ErrorGUI(this);
        oHandler.WriteErrorMessage(oErr);
      }
    }
    else if (oEvent.getActionCommand().equals("EditMortalityEpisode")) {
      try {

        //Get the mortality episode to edit
        int iMortEpisodeNumber = new Integer(m_jMortEpNumber.getText()).
            intValue();
        iMortEpisodeNumber--; //Since the display is always one more than the index
        if (iMortEpisodeNumber >= 0) {
          HarvestData oMortEpiside = (HarvestData) mp_oMortEpisodeData.
              elementAt(iMortEpisodeNumber);

          MortalityEpisodeEdit oWindow = new MortalityEpisodeEdit(this,
              oMortEpiside);
          oWindow.pack();
          oWindow = (MortalityEpisodeEdit) SizeChildWindow(oWindow);
          oWindow.setLocationRelativeTo(null);
          oWindow.setVisible(true);
        }
        else {
          MortalityEpisodeEdit oWindow = new MortalityEpisodeEdit(this);
          oWindow.pack();
          oWindow = (MortalityEpisodeEdit) SizeChildWindow(oWindow);
          oWindow.setLocationRelativeTo(null);
          oWindow.setVisible(true);
        }
      }
      catch (ModelException oErr) {
        ErrorGUI oHandler = new ErrorGUI(this);
        oHandler.WriteErrorMessage(oErr);
      }
    }
    else if (oEvent.getActionCommand().equals("EditDiam10")) {
      Diam10Edit oWindow = new Diam10Edit(this);
      oWindow.pack();
      oWindow.setLocationRelativeTo(null);
      oWindow.setVisible(true);
    }
    else if (oEvent.getActionCommand().equals("DeleteHarvest")) {
      try {
        DeleteHarvest();
      }
      catch (ModelException oErr) {
        ErrorGUI oHandler = new ErrorGUI(this);
        oHandler.WriteErrorMessage(oErr);
      }
    }
    else if (oEvent.getActionCommand().equals("DeletePlant")) {
      try {
        DeletePlanting();
      }
      catch (ModelException oErr) {
        ErrorGUI oHandler = new ErrorGUI(this);
        oHandler.WriteErrorMessage(oErr);
      }
    }
    else if (oEvent.getActionCommand().equals("DeleteMortalityEpisode")) {
      try {
        DeleteMortalityEpisode();
      }
      catch (ModelException oErr) {
        ErrorGUI oHandler = new ErrorGUI(this);
        oHandler.WriteErrorMessage(oErr);
      }
    }
  }

  /**
   * Deletes the currently displayed harvest.  If no harvest is displayed
   * (the display number is set to 0), nothing happens.  If a harvest is
   * deleted, the next harvest is displayed, or the previous one if it is the
   * last.
   * @throws ModelException passed through from called methods.
   */
  protected void DeleteHarvest() throws ModelException {
    int iCurrentHarvest = new Integer(m_jHarvestNumber.getText()).intValue() -
        1,
        iTotalHarvests = new Integer(m_jNumHarvestEvents.getText()).
        intValue();

    if (iCurrentHarvest < 0) {
      return;
    }

    mp_oHarvestData.remove(iCurrentHarvest);

    if (iCurrentHarvest < mp_oHarvestData.size()) {
      //Display the next harvest event - the vector index is already right
      HarvestData oHarvest = (HarvestData) mp_oHarvestData.elementAt(
          iCurrentHarvest);
      DisplayHarvest(oHarvest);
    }
    else if (0 == mp_oHarvestData.size()) {
      DisplayHarvest(null);
    }
    else {
      HarvestData oHarvest = (HarvestData) mp_oHarvestData.elementAt(
          mp_oHarvestData.size() - 1);
      DisplayHarvest(oHarvest);
      m_jHarvestNumber.setText(String.valueOf(iCurrentHarvest));
    }

    //Set the total harvest display
    iTotalHarvests--;
    m_jNumHarvestEvents.setText(String.valueOf(iTotalHarvests));
  }

  /**
   * Deletes the currently displayed planting.  If no planting is displayed
   * (the display number is set to 0), nothing happens.  If a planting is
   * deleted, the next planting is displayed, or the previous one if it is the
   * last.
   * @throws ModelException passed through from called methods.
   */
  protected void DeletePlanting() throws ModelException {
    int iCurrentPlanting = new Integer(m_jPlantNumber.getText()).intValue() - 1,
        iTotalPlantings = new Integer(m_jNumPlantingEvents.getText()).
        intValue();

    if (iCurrentPlanting < 0) {
      return;
    }

    mp_oPlantingData.remove(iCurrentPlanting);

    if (iCurrentPlanting < mp_oPlantingData.size()) {
      //Display the next planting event - the vector index is already right
      PlantingData oPlanting = (PlantingData) mp_oPlantingData.elementAt(
          iCurrentPlanting);
      DisplayPlanting(oPlanting);
    }
    else if (0 == mp_oPlantingData.size()) {
      DisplayPlanting(null);
    }
    else {
      PlantingData oPlanting = (PlantingData) mp_oPlantingData.elementAt(
          mp_oPlantingData.size() - 1);
      DisplayPlanting(oPlanting);
      m_jPlantNumber.setText(String.valueOf(iCurrentPlanting));
    }

    //Set the total planting display
    iTotalPlantings--;
    m_jNumPlantingEvents.setText(String.valueOf(iTotalPlantings));
  }

  /**
   * Deletes the currently displayed mortality episode.  If no episode is
   * displayed (the display number is set to 0), nothing happens.  If an
   * episode is deleted, the next episode is displayed, or the previous
   * one if it is the last.
   * @throws ModelException passed through from called methods.
   */
  protected void DeleteMortalityEpisode() throws ModelException {
    int iCurrentEpisode = new Integer(m_jMortEpNumber.getText()).intValue() -
        1,
        iTotalEpisodes = new Integer(m_jNumMortEpEvents.getText()).
        intValue();

    if (iCurrentEpisode < 0) {
      return;
    }

    mp_oMortEpisodeData.remove(iCurrentEpisode);

    if (iCurrentEpisode < mp_oMortEpisodeData.size()) {
      //Display the next mortality episode - the vector index is already right
      HarvestData oEpisode = (HarvestData) mp_oMortEpisodeData.elementAt(
          iCurrentEpisode);
      DisplayMortalityEpisode(oEpisode);
    }
    else if (0 == mp_oMortEpisodeData.size()) {
      DisplayMortalityEpisode(null);
    }
    else {
      HarvestData oEpisode = (HarvestData) mp_oMortEpisodeData.elementAt(
          mp_oMortEpisodeData.size() - 1);
      DisplayMortalityEpisode(oEpisode);
      m_jMortEpNumber.setText(String.valueOf(iCurrentEpisode));
    }

    //Set the total harvest display
    iTotalEpisodes--;
    m_jNumMortEpEvents.setText(String.valueOf(iTotalEpisodes));
  }

  /**
   * Causes a harvest event to be displayed in the window.
   * @param oHarvest Harvest to display, or NULL if no harvest is to be
   * displayed.
   * @throws ModelException passing through from called methods.
   */
  protected void DisplayHarvest(HarvestData oHarvest) throws ModelException {
    ChartPanel jChart = null;
    int i, j;
    //Delete all existing components in the chart panel
    for (i = 0; i < m_jChartPanel.getComponentCount(); i++) {
      m_jChartPanel.remove(i);
    }

    //Set up the renderer
    m_oRenderer.SetColor(HARVEST_COLOR, 1);

    //Set the dataset to have false for every value
    for (i = 0; i < m_iNumHarvestXCells; i++) {
      for (j = 0; j < m_iNumHarvestYCells; j++) {
        m_oHarvestDataset.mp_bData[1][i][j] = false;
      }
    }

    if (oHarvest == null) {

      //Set all the data values to their null settings
      m_jHarvestCutAmountTypeLabel.setText("N/A");
      m_jCutTypeLabel.setText("N/A");
      m_jHarvestSpeciesList.clear();
      m_jHarvestSpeciesList.addElement("N/A");
      m_jHarvestNumber.setText("0");
      m_jHarvestTimestepLabel.setText("0");
      m_jHarvestCutRange1Amt.setText("0");
      m_jHarvestCutRange1Max.setText("0");
      m_jHarvestCutRange1Min.setText("0");
      m_jHarvestCutRange2Amt.setText("0");
      m_jHarvestCutRange2Max.setText("0");
      m_jHarvestCutRange2Min.setText("0");
      m_jHarvestCutRange3Amt.setText("0");
      m_jHarvestCutRange3Max.setText("0");
      m_jHarvestCutRange3Min.setText("0");
      m_jHarvestCutRange4Amt.setText("0");
      m_jHarvestCutRange4Max.setText("0");
      m_jHarvestCutRange4Min.setText("0");
    }
    else {

      //The harvest exists - extract the data

      //Cut amount type
      int iTemp = oHarvest.GetCutAmountType();
      if (iTemp == DisturbanceBehaviors.PERCENTAGE_BASAL_AREA) {
        m_jHarvestCutAmountTypeLabel.setText("% Basal area");
      }
      else if (iTemp == DisturbanceBehaviors.PERCENTAGE_DENSITY) {
        m_jHarvestCutAmountTypeLabel.setText("% Density");
      }
      else if (iTemp == DisturbanceBehaviors.ABSOLUTE_BASAL_AREA) {
        m_jHarvestCutAmountTypeLabel.setText("Amount basal area");
      }
      else if (iTemp == DisturbanceBehaviors.ABSOLUTE_DENSITY) {
        m_jHarvestCutAmountTypeLabel.setText("Amount Density");
      }
      else {
        throw(new ModelException(ErrorGUI.BAD_DATA, "JAVA",
        "The harvest being loaded has an unrecognized cut amount type."));
      }

      //Cut type
      iTemp = oHarvest.GetCutType();
      if (iTemp == DisturbanceBehaviors.CLEAR_CUT) {
        m_jCutTypeLabel.setText("Clear cut");
      }
      else if (iTemp == DisturbanceBehaviors.GAP_CUT) {
        m_jCutTypeLabel.setText("Gap cut");
      }
      else if (iTemp == DisturbanceBehaviors.PARTIAL_CUT) {
        m_jCutTypeLabel.setText("Partial cut");
      }
      else {
        throw(new ModelException(ErrorGUI.BAD_DATA, "JAVA",
        "The harvest being loaded has an unrecognized cut type."));
      }

      //Timestep
      iTemp = oHarvest.GetTimestep();
      m_jHarvestTimestepLabel.setText(String.valueOf(iTemp));

      //Species
      TreePopulation p_oPop = m_oDisturbanceBehaviors.m_oManager.
          GetTreePopulation();
      m_jHarvestSpeciesList.clear();
      for (i = 0; i < oHarvest.GetNumberOfSpecies(); i++) {
        m_jHarvestSpeciesList.addElement(p_oPop.GetSpeciesNameFromCode(oHarvest.
            GetSpecies(i)).replace('_', ' '));
      }

      //Cut range info
      int iNumCutRanges = oHarvest.GetNumberOfCutRanges();
      m_jHarvestCutRange1Amt.setText(String.valueOf(oHarvest.GetCutAmount(0)));
      m_jHarvestCutRange1Max.setText(String.valueOf(oHarvest.GetUpperBound(0)));
      m_jHarvestCutRange1Min.setText(String.valueOf(oHarvest.GetLowerBound(0)));
      if (iNumCutRanges > 1) {
        m_jHarvestCutRange2Amt.setText(String.valueOf(oHarvest.GetCutAmount(1)));
        m_jHarvestCutRange2Max.setText(String.valueOf(oHarvest.GetUpperBound(1)));
        m_jHarvestCutRange2Min.setText(String.valueOf(oHarvest.GetLowerBound(1)));
      }
      else {
        m_jHarvestCutRange2Amt.setText("0");
        m_jHarvestCutRange2Max.setText("0");
        m_jHarvestCutRange2Min.setText("0");
      }
      if (iNumCutRanges > 2) {
        m_jHarvestCutRange3Amt.setText(String.valueOf(oHarvest.GetCutAmount(2)));
        m_jHarvestCutRange3Max.setText(String.valueOf(oHarvest.GetUpperBound(2)));
        m_jHarvestCutRange3Min.setText(String.valueOf(oHarvest.GetLowerBound(2)));
      }
      else {
        m_jHarvestCutRange3Amt.setText("0");
        m_jHarvestCutRange3Max.setText("0");
        m_jHarvestCutRange3Min.setText("0");
      }
      if (iNumCutRanges > 3) {
        m_jHarvestCutRange4Amt.setText(String.valueOf(oHarvest.GetCutAmount(3)));
        m_jHarvestCutRange4Max.setText(String.valueOf(oHarvest.GetUpperBound(3)));
        m_jHarvestCutRange4Min.setText(String.valueOf(oHarvest.GetLowerBound(3)));
      }
      else {
        m_jHarvestCutRange4Amt.setText("0");
        m_jHarvestCutRange4Max.setText("0");
        m_jHarvestCutRange4Min.setText("0");
      }

      //Now add in those values in the harvest data cells with a value of 1
      for (i = 0; i < oHarvest.GetNumberOfCells(); i++) {
        Cell oCell = oHarvest.GetCell(i);
        m_oHarvestDataset.mp_bData[1][oCell.GetX()][oCell.GetY()] = true;
      }
    }

    jChart = DataGrapher.MakeOverlaidMap(m_oHarvestDataset,
        m_oTreeDataset, "East->", "", m_iPlotLengthX,
        m_iPlotLengthY, m_oRenderer, m_oTreeRenderer,
        new java.awt.Dimension(Math.min(600,
                                        m_iPlotLengthX * 3),
                               Math.min(600, m_iPlotLengthY * 3)));
    m_jChartPanel.add(jChart);
    m_jChartPanel.validate();
    m_jChartPanel.repaint();
  }

  /**
   * Causes a mortality episode to be displayed in the window.
   * @param oEpisode Mortality episode to display, or NULL if no episode is
   * to be displayed.
   * @throws ModelException passing through from called methods.
   */
  protected void DisplayMortalityEpisode(HarvestData oEpisode) throws
      ModelException {
    ChartPanel jChart = null;
    int i, j;

    //Set up the renderer
    m_oRenderer.SetColor(MORTALITY_EPISODE_COLOR, 1);

    //Set the dataset to have false for every value
    for (i = 0; i < m_iNumMortEpisodeXCells; i++) {
      for (j = 0; j < m_iNumMortEpisodeYCells; j++) {
        m_oMortEpisodeDataset.mp_bData[1][i][j] = false;
      }
    }

    //Delete all existing components in the chart panel
    for (i = 0; i < m_jChartPanel.getComponentCount(); i++) {
      m_jChartPanel.remove(i);
    }

    if (oEpisode == null) {

      //Set all the data values to their null settings
      m_jMortEpCutAmountTypeLabel.setText("N/A");
      m_jMortEpSpeciesList.clear();
      m_jMortEpSpeciesList.addElement("N/A");
      m_jMortEpNumber.setText("0");
      m_jMortEpTimestepLabel.setText("0");
      m_jMortEpCutRange1Amt.setText("0");
      m_jMortEpCutRange1Max.setText("0");
      m_jMortEpCutRange1Min.setText("0");
      m_jMortEpCutRange2Amt.setText("0");
      m_jMortEpCutRange2Max.setText("0");
      m_jMortEpCutRange2Min.setText("0");
      m_jMortEpCutRange3Amt.setText("0");
      m_jMortEpCutRange3Max.setText("0");
      m_jMortEpCutRange3Min.setText("0");
      m_jMortEpCutRange4Amt.setText("0");
      m_jMortEpCutRange4Max.setText("0");
      m_jMortEpCutRange4Min.setText("0");
    }
    else {

      //The mortality episode exists - extract the data

      //Cut amount type
      int iTemp = oEpisode.GetCutAmountType();
      if (iTemp == DisturbanceBehaviors.PERCENTAGE_BASAL_AREA) {
        m_jMortEpCutAmountTypeLabel.setText("% Basal area");
      }
      else if (iTemp == DisturbanceBehaviors.PERCENTAGE_DENSITY) {
        m_jMortEpCutAmountTypeLabel.setText("% Density");
      }
      else if (iTemp == DisturbanceBehaviors.ABSOLUTE_BASAL_AREA) {
        m_jMortEpCutAmountTypeLabel.setText("Amount basal area");
      }
      else if (iTemp == DisturbanceBehaviors.ABSOLUTE_DENSITY) {
        m_jMortEpCutAmountTypeLabel.setText("Amount Density");
      }
      else {
        throw(new ModelException(ErrorGUI.BAD_DATA, "JAVA",
        "The harvest being loaded has an unrecognized cut amount type."));
      }

      //Timestep
      iTemp = oEpisode.GetTimestep();
      m_jMortEpTimestepLabel.setText(String.valueOf(iTemp));

      //Species
      TreePopulation p_oPop = m_oDisturbanceBehaviors.m_oManager.
          GetTreePopulation();
      m_jMortEpSpeciesList.clear();
      for (i = 0; i < oEpisode.GetNumberOfSpecies(); i++) {
        m_jMortEpSpeciesList.addElement(p_oPop.GetSpeciesNameFromCode(
            oEpisode.
            GetSpecies(i)).replace('_', ' '));
      }

      //Cut range info
      int iNumCutRanges = oEpisode.GetNumberOfCutRanges();
      m_jMortEpCutRange1Amt.setText(String.valueOf(oEpisode.
          GetCutAmount(0)));
      m_jMortEpCutRange1Max.setText(String.valueOf(oEpisode.
          GetUpperBound(0)));
      m_jMortEpCutRange1Min.setText(String.valueOf(oEpisode.
          GetLowerBound(0)));
      if (iNumCutRanges > 1) {
        m_jMortEpCutRange2Amt.setText(String.valueOf(oEpisode.
            GetCutAmount(1)));
        m_jMortEpCutRange2Max.setText(String.valueOf(oEpisode.
            GetUpperBound(1)));
        m_jMortEpCutRange2Min.setText(String.valueOf(oEpisode.
            GetLowerBound(1)));
      }
      else {
        m_jMortEpCutRange2Amt.setText("0");
        m_jMortEpCutRange2Max.setText("0");
        m_jMortEpCutRange2Min.setText("0");
      }
      if (iNumCutRanges > 2) {
        m_jMortEpCutRange3Amt.setText(String.valueOf(oEpisode.
            GetCutAmount(2)));
        m_jMortEpCutRange3Max.setText(String.valueOf(oEpisode.
            GetUpperBound(2)));
        m_jMortEpCutRange3Min.setText(String.valueOf(oEpisode.
            GetLowerBound(2)));
      }
      else {
        m_jMortEpCutRange3Amt.setText("0");
        m_jMortEpCutRange3Max.setText("0");
        m_jMortEpCutRange3Min.setText("0");
      }
      if (iNumCutRanges > 3) {
        m_jMortEpCutRange4Amt.setText(String.valueOf(oEpisode.
            GetCutAmount(3)));
        m_jMortEpCutRange4Max.setText(String.valueOf(oEpisode.
            GetUpperBound(3)));
        m_jMortEpCutRange4Min.setText(String.valueOf(oEpisode.
            GetLowerBound(3)));
      }
      else {
        m_jMortEpCutRange4Amt.setText("0");
        m_jMortEpCutRange4Max.setText("0");
        m_jMortEpCutRange4Min.setText("0");
      }

      //Now set the cells in the mortality episode to true
      for (i = 0; i < oEpisode.GetNumberOfCells(); i++) {
        Cell oCell = oEpisode.GetCell(i);
        m_oMortEpisodeDataset.mp_bData[1][oCell.GetX()][oCell.GetY()] = true;
      }
    }
    jChart = DataGrapher.MakeOverlaidMap(m_oMortEpisodeDataset,
        m_oTreeDataset, "East->", "", m_iPlotLengthX,
        m_iPlotLengthY, m_oRenderer, m_oTreeRenderer,
        new java.awt.Dimension(Math.min(600,
                                        m_iPlotLengthX * 3),
                               Math.min(600, m_iPlotLengthY * 3)));
    m_jChartPanel.add(jChart);
    m_jChartPanel.validate();
    m_jChartPanel.repaint();
  }

  /**
   * Causes a planting event to be displayed in the window.  Planting events
   * are displayed as a transparent texture - this allows harvests to display
   * underneath.
   * @param oPlanting Planting to display, or NULL if no planting is to be
   * displayed.
   * @throws ModelException passing through from called methods.
   */
  protected void DisplayPlanting(PlantingData oPlanting) throws ModelException {
    ChartPanel jChart = null;
    int i, j;

    //Set up the renderer
    m_oRenderer.SetColor(PLANT_COLOR, 1);

    //Set the dataset to have false for every value
    for (i = 0; i < m_iNumPlantXCells; i++) {
      for (j = 0; j < m_iNumPlantYCells; j++) {
        m_oPlantDataset.mp_bData[1][i][j] = false;
      }
    }

    //Delete all existing components in the chart panel
    for (i = 0; i < m_jChartPanel.getComponentCount(); i++) {
      m_jChartPanel.remove(i);
    }

    if (oPlanting == null) {

      //Set all the data values to their null settings
      for (i = 0; i < m_iNumSpecies; i++) {
        mp_jSpeciesPlantPercentages[i].setText("0");
      }
      m_jPlantNumber.setText("0");
      m_jPlantTimestepLabel.setText("0");
      m_jPlantSpacingLabel.setText("N/A");
      m_jPlantAmountLabel.setText("Density (#/ha):");
      m_jPlantAmountValueLabel.setText("N/A");
    }
    else {

      //The planting exists - extract the data

      //Planting percentages
      //Zero out existing values first
      for (i = 0; i < m_iNumSpecies; i++) {
        mp_jSpeciesPlantPercentages[i].setText("0");
      }
      for (i = 0; i < oPlanting.GetNumberOfSpecies(); i++) {
        int iSpecies = oPlanting.GetSpecies(i);
        mp_jSpeciesPlantPercentages[iSpecies].setText(String.valueOf(oPlanting.
            GetAbundance(iSpecies)));
      }

      //Plant spacing
      int iTemp = oPlanting.GetPlantSpacing();
      if (iTemp == PlantingBehaviors.GRIDDED) {
        m_jPlantSpacingLabel.setText("Gridded");
        m_jPlantAmountLabel.setText("Spacing (m):");
        m_jPlantAmountValueLabel.setText(String.valueOf(oPlanting.
            GetSpacingDistance()));
      }
      else if (iTemp == PlantingBehaviors.RANDOM) {
        m_jPlantSpacingLabel.setText("Random");
        m_jPlantAmountLabel.setText("Density (#/ha):");
        m_jPlantAmountValueLabel.setText(String.valueOf(oPlanting.GetDensity()));
      }
      else {
        throw(new ModelException(ErrorGUI.BAD_DATA, "JAVA",
        "The planting event being loaded has an unrecognized plant spacing."));
      }

      //Timestep
      iTemp = oPlanting.GetTimestep();
      m_jPlantTimestepLabel.setText(String.valueOf(iTemp));

      //Now set the cells in the planting to true
      for (i = 0; i < oPlanting.GetNumberOfCells(); i++) {
        Cell oCell = oPlanting.GetCell(i);
        m_oPlantDataset.mp_bData[1][oCell.GetX()][oCell.GetY()] = true;
      }
    }

    jChart = DataGrapher.MakeOverlaidMap(m_oPlantDataset,
        m_oTreeDataset, "East->", "", m_iPlotLengthX,
        m_iPlotLengthY, m_oRenderer, m_oTreeRenderer,
        new java.awt.Dimension(Math.min(600,
                                        m_iPlotLengthX * 3),
                               Math.min(600, m_iPlotLengthY * 3)));
    m_jChartPanel.add(jChart);
    m_jChartPanel.validate();
    m_jChartPanel.repaint();
  }

  /**
   * Gets access to the tree population for benefit of child windows.
   * @return TreePopulation object.
   */
  public TreePopulation GetTreePopulation() {
    return m_oDisturbanceBehaviors.m_oManager.GetTreePopulation();
  }

  /**
   * Gets access to the HelpBroker object for benefit of child windows.
   * @return HelpBroker object.
   */
  public javax.help.HelpBroker GetHelpBroker() {
    return m_oDisturbanceBehaviors.m_oManager.GetHelpBroker();
  }

  /**
   * Takes the final data and adds it back to the behavior groups.  This takes
   * the contents of mp_oHarvestData, mp_oMortEpisodeData, and
   * mp_oPlantingData, and puts them back into the parent behavior groups.
   * Then, if there is a set of data for any of the behaviors, it makes sure
   * that they are enabled.
   */
  protected void AddFinishedData() {
    int i;

    //Harvest data
    m_oDisturbanceBehaviors.mp_oHarvestCuts.clear();
    for (i = 0; i < mp_oHarvestData.size(); i++) {
      m_oDisturbanceBehaviors.mp_oHarvestCuts.add(mp_oHarvestData.elementAt(i));
    }

    //Make sure the harvest behavior is enabled if there's any cut data
    if (m_oDisturbanceBehaviors.mp_oHarvestCuts.size() > 0) {
      m_oDisturbanceBehaviors.GetBehaviorByKey("Harvest").m_bIsEnabled = true;
    }
    else {
      m_oDisturbanceBehaviors.GetBehaviorByKey("Harvest").m_bIsEnabled = false;
    }

    //Mortality episode data
    m_oDisturbanceBehaviors.mp_oMortEpisodes.clear();
    for (i = 0; i < mp_oMortEpisodeData.size(); i++) {
      m_oDisturbanceBehaviors.mp_oMortEpisodes.add(mp_oMortEpisodeData.
          elementAt(i));
    }

    //Make sure the mortality episode behavior is enabled if there's any
    //cut data
    if (m_oDisturbanceBehaviors.mp_oMortEpisodes.size() > 0) {
      m_oDisturbanceBehaviors.GetBehaviorByKey("Episodic Mortality").
          m_bIsEnabled = true;
    }
    else {
      m_oDisturbanceBehaviors.GetBehaviorByKey("Episodic Mortality").
          m_bIsEnabled = false;
    }

    //Planting data
    m_oPlantBehaviors.mp_oPlantings.clear();
    for (i = 0; i < mp_oPlantingData.size(); i++) {
      m_oPlantBehaviors.mp_oPlantings.add(mp_oPlantingData.elementAt(i));
    }

    //Make sure the plant behavior is enabled if there's any plant data
    if (m_oPlantBehaviors.mp_oPlantings.size() > 0) {
      m_oPlantBehaviors.GetBehaviorByKey("Planting").m_bIsEnabled = true;
    }
    else {
      m_oPlantBehaviors.GetBehaviorByKey("Planting").m_bIsEnabled = false;
    }
  }
}

/**
 * A 1.4 file that provides utility methods for
 * creating form- or grid-style layouts with SpringLayout.
 * These utilities are used by several programs, such as
 * SpringBox and SpringCompactGrid.
 * I got this from the Java tutorial (LEM).
 */
class SpringUtilities {

  /**
   *  Used by makeCompactGrid.
   * @param row Row
   * @param col Column
   * @param parent Parent container
   * @param cols Columns
   * @return Constraints
   */
  private static SpringLayout.Constraints getConstraintsForCell(
      int row, int col,
      java.awt.Container parent,
      int cols) {
    SpringLayout layout = (SpringLayout) parent.getLayout();
    java.awt.Component c = parent.getComponent(row * cols + col);
    return layout.getConstraints(c);
  }

  /**
   * Aligns the first <code>rows</code> * <code>cols</code>
   * components of <code>parent</code> in
   * a grid. Each component in a column is as wide as the maximum
   * preferred width of the components in that column;
   * height is similarly determined for each row.
   * The parent is made just big enough to fit them all.
   *
   * @param parent Container with spring layout elements to put into a grid
   * @param rows number of rows
   * @param cols number of columns
   * @param initialX x location to start the grid at
   * @param initialY y location to start the grid at
   * @param xPad x padding between cells
   * @param yPad y padding between cells
   */
  public static void makeCompactGrid(java.awt.Container parent,
                                     int rows, int cols,
                                     int initialX, int initialY,
                                     int xPad, int yPad) {
    SpringLayout layout;
    try {
      layout = (SpringLayout) parent.getLayout();
    }
    catch (ClassCastException exc) {
      System.err.println(
          "The first argument to makeCompactGrid must use SpringLayout.");
      return;
    }

    //Align all cells in each column and make them the same width.
    Spring x = Spring.constant(initialX);
    for (int c = 0; c < cols; c++) {
      Spring width = Spring.constant(0);
      for (int r = 0; r < rows; r++) {
        width = Spring.max(width,
                           getConstraintsForCell(r, c, parent, cols).
                           getWidth());
      }
      for (int r = 0; r < rows; r++) {
        SpringLayout.Constraints constraints =
            getConstraintsForCell(r, c, parent, cols);
        constraints.setX(x);
        constraints.setWidth(width);
      }
      x = Spring.sum(x, Spring.sum(width, Spring.constant(xPad)));
    }

    //Align all cells in each row and make them the same height.
    Spring y = Spring.constant(initialY);
    for (int r = 0; r < rows; r++) {
      Spring height = Spring.constant(0);
      for (int c = 0; c < cols; c++) {
        height = Spring.max(height,
                            getConstraintsForCell(r, c, parent, cols).
                            getHeight());
      }
      for (int c = 0; c < cols; c++) {
        SpringLayout.Constraints constraints =
            getConstraintsForCell(r, c, parent, cols);
        constraints.setY(y);
        constraints.setHeight(height);
      }
      y = Spring.sum(y, Spring.sum(height, Spring.constant(yPad)));
    }

    //Set the parent's size.
    SpringLayout.Constraints pCons = layout.getConstraints(parent);
    pCons.setConstraint(SpringLayout.SOUTH, y);
    pCons.setConstraint(SpringLayout.EAST, x);
  }
}
