package javawrapper;

import junit.framework.*;
import java.io.File;

/**Tests the Allometry class.
 * <p>Copyright: Copyright (c) Charles D. Canham 2005</p>
 * <p>Company: Institute of Ecosystem Studies</p>
 * @author Lora E. Murphy
 * @version 1.0
 */

public class TestAllometry
    extends TestCase {

  /**
   * Test parameter file reading for allometry
   */
  public void testParFileReading() {
    GUIManager oManager = null;
    String sFileName = null;
    try {

      oManager = new GUIManager(null);
      oManager.ClearCurrentData();
      sFileName = WriteXMLFile1();
      oManager.InputXMLParameterFile(sFileName);
      Allometry oAllom = oManager.GetAllometry();

      assertEquals( ( (Float) oAllom.mp_fMaxCanopyHeight.GetValue().elementAt(0)).
                   floatValue(), 45, 0.001);
      assertEquals( ( (Float) oAllom.mp_fMaxCanopyHeight.GetValue().elementAt(1)).
                   floatValue(), 39.48, 0.001);

      assertEquals( ( (Float) oAllom.mp_fSlopeOfAsympCrownRad.GetValue().
                     elementAt(0)).floatValue(), 0.02418, 0.001);
      assertEquals( ( (Float) oAllom.mp_fSlopeOfAsympCrownRad.GetValue().
                     elementAt(1)).floatValue(), 0.0549, 0.001);

      assertEquals( ( (Float) oAllom.mp_fCrownRadExp.GetValue().elementAt(0)).
                   floatValue(), 1.1, 0.001);
      assertEquals( ( (Float) oAllom.mp_fCrownRadExp.GetValue().elementAt(1)).
                   floatValue(), 0.9, 0.001);

      assertEquals( ( (Float) oAllom.mp_fDiam10ToDbhSlope.GetValue().
                     elementAt(0)).floatValue(), 0.7059, 0.001);
      assertEquals( ( (Float) oAllom.mp_fDiam10ToDbhSlope.GetValue().
                     elementAt(1)).floatValue(), 0.8008, 0.001);

     assertEquals( ( (Float) oAllom.mp_fDiam10ToDbhIntercept.GetValue().
                     elementAt(0)).floatValue(), 0.0, 0.001);
      assertEquals( ( (Float) oAllom.mp_fDiam10ToDbhIntercept.GetValue().
                     elementAt(1)).floatValue(), 1.3, 0.001);

      assertEquals( ( (Float) oAllom.mp_fSlopeOfAsympCrownHt.GetValue().
                     elementAt(0)).floatValue(), 0.34, 0.001);
      assertEquals( ( (Float) oAllom.mp_fSlopeOfAsympCrownHt.GetValue().
                     elementAt(1)).floatValue(), 0.389, 0.001);

      assertEquals( ( (Float) oAllom.mp_fCrownHtExp.GetValue().elementAt(0)).
                   floatValue(), 0.9, 0.001);
      assertEquals( ( (Float) oAllom.mp_fCrownHtExp.GetValue().elementAt(1)).
                   floatValue(), 1.0, 0.001);

      assertEquals( ( (Float) oAllom.mp_fSlopeOfHeightDiam10.GetValue().
                     elementAt(0)).floatValue(), 0.03418, 0.001);
      assertEquals( ( (Float) oAllom.mp_fSlopeOfHeightDiam10.GetValue().
                     elementAt(1)).floatValue(), 0.2871, 0.001);

      assertEquals( ( (Float) oAllom.mp_fSlopeOfAsymptoticHeight.GetValue().
                     elementAt(0)).floatValue(), 0.0263, 0.001);
      assertEquals( ( (Float) oAllom.mp_fSlopeOfAsymptoticHeight.GetValue().
                     elementAt(1)).floatValue(), 0.0163, 0.001);

      ModelEnum oEnum = (ModelEnum) oAllom.mp_iWhatSeedlingHDFunction.GetValue().
          elementAt(0);
      assertEquals(oEnum.GetValue(), 2);
      oEnum = (ModelEnum) oAllom.mp_iWhatSeedlingHDFunction.GetValue().
          elementAt(1);
      assertEquals(oEnum.GetValue(), 1);

      oEnum = (ModelEnum) oAllom.mp_iWhatSaplingHDFunction.GetValue().elementAt(
          0);
      assertEquals(oEnum.GetValue(), 1);
      oEnum = (ModelEnum) oAllom.mp_iWhatSaplingHDFunction.GetValue().elementAt(
          1);
      assertEquals(oEnum.GetValue(), 1);

      oEnum = (ModelEnum) oAllom.mp_iWhatAdultHDFunction.GetValue().elementAt(0);
      assertEquals(oEnum.GetValue(), 0);
      oEnum = (ModelEnum) oAllom.mp_iWhatAdultHDFunction.GetValue().elementAt(1);
      assertEquals(oEnum.GetValue(), 2);

      oEnum = (ModelEnum) oAllom.mp_iWhatAdultCRDFunction.GetValue().elementAt(
          0);
      assertEquals(oEnum.GetValue(), 0);
      oEnum = (ModelEnum) oAllom.mp_iWhatAdultCRDFunction.GetValue().elementAt(
          1);
      assertEquals(oEnum.GetValue(), 1);

      oEnum = (ModelEnum) oAllom.mp_iWhatSaplingCRDFunction.GetValue().
          elementAt(0);
      assertEquals(oEnum.GetValue(), 1);
      oEnum = (ModelEnum) oAllom.mp_iWhatSaplingCRDFunction.GetValue().
          elementAt(1);
      assertEquals(oEnum.GetValue(), 0);

      oEnum = (ModelEnum) oAllom.mp_iWhatAdultCHHFunction.GetValue().elementAt(
          0);
      assertEquals(oEnum.GetValue(), 1);
      oEnum = (ModelEnum) oAllom.mp_iWhatAdultCHHFunction.GetValue().elementAt(
          1);
      assertEquals(oEnum.GetValue(), 0);

      oEnum = (ModelEnum) oAllom.mp_iWhatSaplingCHHFunction.GetValue().
          elementAt(0);
      assertEquals(oEnum.GetValue(), 1);
      oEnum = (ModelEnum) oAllom.mp_iWhatSaplingCHHFunction.GetValue().
          elementAt(1);
      assertEquals(oEnum.GetValue(), 1);

      assertEquals( ( (Float) oAllom.mp_fAdultLinearSlope.GetValue().
                     elementAt(0)).floatValue(), 0.96, 0.001);
      assertEquals( ( (Float) oAllom.mp_fAdultLinearSlope.GetValue().
                     elementAt(1)).floatValue(), 1.3, 0.001);

      assertEquals( ( (Float) oAllom.mp_fAdultLinearIntercept.GetValue().
                     elementAt(0)).floatValue(), 0, 0.001);
      assertEquals( ( (Float) oAllom.mp_fAdultLinearIntercept.GetValue().
                     elementAt(1)).floatValue(), -0.9, 0.001);

      assertEquals( ( (Float) oAllom.mp_fSaplingLinearSlope.GetValue().
                     elementAt(0)).floatValue(), 0.492, 0.001);
      assertEquals( ( (Float) oAllom.mp_fSaplingLinearSlope.GetValue().
                     elementAt(1)).floatValue(), 0.0549, 0.001);

      assertEquals( ( (Float) oAllom.mp_fSaplingLinearIntercept.GetValue().
                     elementAt(0)).floatValue(), 1.2, 0.001);
      assertEquals( ( (Float) oAllom.mp_fSaplingLinearIntercept.GetValue().
                     elementAt(1)).floatValue(), 0, 0.001);

      assertEquals( ( (Float) oAllom.mp_fSeedlingLinearSlope.GetValue().
                     elementAt(0)).floatValue(), 0.9629, 0.001);
      assertEquals( ( (Float) oAllom.mp_fSeedlingLinearSlope.GetValue().
                     elementAt(1)).floatValue(), 1.228, 0.001);

      assertEquals( ( (Float) oAllom.mp_fSeedlingLinearIntercept.GetValue().
                     elementAt(0)).floatValue(), 0, 0.001);
      assertEquals( ( (Float) oAllom.mp_fSeedlingLinearIntercept.GetValue().
                     elementAt(1)).floatValue(), -0.9, 0.001);

      assertEquals( ( (Float) oAllom.mp_fAdultReverseLinearSlope.GetValue().
                     elementAt(0)).floatValue(), 2.1, 0.001);
     assertEquals( ( (Float) oAllom.mp_fAdultReverseLinearSlope.GetValue().
                     elementAt(1)).floatValue(), 1.9, 0.001);

      assertEquals( ( (Float) oAllom.mp_fAdultReverseLinearIntercept.GetValue().
                     elementAt(0)).floatValue(), 0.02418, 0.001);
     assertEquals( ( (Float) oAllom.mp_fAdultReverseLinearIntercept.GetValue().
                     elementAt(1)).floatValue(), 0.034, 0.001);

      assertEquals( ( (Float) oAllom.mp_fSaplingReverseLinearSlope.GetValue().
                     elementAt(0)).floatValue(), 1.5, 0.001);
      assertEquals( ( (Float) oAllom.mp_fSaplingReverseLinearSlope.GetValue().
                     elementAt(1)).floatValue(), 1.1, 0.001);

      assertEquals( ( (Float) oAllom.mp_fSaplingReverseLinearIntercept.GetValue().
                     elementAt(0)).floatValue(), 0.761, 0.001);
      assertEquals( ( (Float) oAllom.mp_fSaplingReverseLinearIntercept.GetValue().
                     elementAt(1)).floatValue(), -0.847, 0.001);

      assertEquals( ( (Float) oAllom.mp_fSeedlingReverseLinearSlope.GetValue().
                     elementAt(0)).floatValue(), 1.5, 0.001);
      assertEquals( ( (Float) oAllom.mp_fSeedlingReverseLinearSlope.GetValue().
                     elementAt(1)).floatValue(), 0.02, 0.001);

      assertEquals( ( (Float) oAllom.mp_fSeedlingReverseLinearIntercept.
                     GetValue().elementAt(0)).floatValue(), 0.761, 0.001);
      assertEquals( ( (Float) oAllom.mp_fSeedlingReverseLinearIntercept.
                     GetValue().elementAt(1)).floatValue(), 0.758, 0.001);

      assertEquals( ( (Float) oAllom.mp_fCRCrownRadIntercept.GetValue().
                     elementAt(0)).floatValue(), 0.3, 0.001);
      assertEquals( ( (Float) oAllom.mp_fCRCrownRadIntercept.GetValue().
                     elementAt(1)).floatValue(), 0.4, 0.001);

      assertEquals( ( (Float) oAllom.mp_fCRAsympCrownRad.GetValue().
                     elementAt(0)).floatValue(), 8, 0.001);
      assertEquals( ( (Float) oAllom.mp_fCRAsympCrownRad.GetValue().
                     elementAt(1)).floatValue(), 7, 0.001);

      assertEquals( ( (Float) oAllom.mp_fCRCrownRadShape1.GetValue().
                     elementAt(0)).floatValue(), 0.09, 0.001);
      assertEquals( ( (Float) oAllom.mp_fCRCrownRadShape1.GetValue().
                     elementAt(1)).floatValue(), 0.1, 0.001);

      assertEquals( ( (Float) oAllom.mp_fCRCrownRadShape2.GetValue().
                     elementAt(0)).floatValue(), 1.5, 0.001);
      assertEquals( ( (Float) oAllom.mp_fCRCrownRadShape2.GetValue().
                     elementAt(1)).floatValue(), 2.4, 0.001);

      assertEquals( ( (Float) oAllom.mp_fCRCrownHtIntercept.GetValue().
                     elementAt(0)).floatValue(), 0.6, 0.001);
      assertEquals( ( (Float) oAllom.mp_fCRCrownHtIntercept.GetValue().
                     elementAt(1)).floatValue(), 0.5, 0.001);

      assertEquals( ( (Float) oAllom.mp_fCRAsympCrownHt.GetValue().
                     elementAt(0)).floatValue(), 60, 0.001);
      assertEquals( ( (Float) oAllom.mp_fCRAsympCrownHt.GetValue().
                     elementAt(1)).floatValue(), 12, 0.001);

      assertEquals( ( (Float) oAllom.mp_fCRCrownHtShape1.GetValue().
                     elementAt(0)).floatValue(), 0.3, 0.001);
      assertEquals( ( (Float) oAllom.mp_fCRCrownHtShape1.GetValue().
                     elementAt(1)).floatValue(), 0.1, 0.001);

      assertEquals( ( (Float) oAllom.mp_fCRCrownHtShape2.GetValue().
                     elementAt(0)).floatValue(), 3.2, 0.001);
      assertEquals( ( (Float) oAllom.mp_fCRCrownHtShape2.GetValue().
                     elementAt(1)).floatValue(), 4.1, 0.001);
    }
    catch (ModelException oErr) {
      fail("Allometry parameter file reading failed with message " +
           oErr.getMessage());
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
  }

  /**
   * Test setup when species have changed.
   */
  public void testDoSetup() {
    GUIManager oManager = null;
    String sFileName = null;
    try {

      oManager = new GUIManager(null);
      oManager.ClearCurrentData();
      sFileName = WriteXMLFile1();
      oManager.InputXMLParameterFile(sFileName);

      //Now change the species by adding another
      String[] sNewSpecies = new String[] {
            "Species 1",
            "Species 2",
            "Species 3"};
      oManager.GetTreePopulation().SetSpeciesNames(sNewSpecies);
      Allometry oAllom = oManager.GetAllometry();
      int i;

      //Verify that all the enums have enums where they're supposed to
      assertEquals(3, oAllom.mp_iWhatAdultHDFunction.GetValue().size());
      assertEquals(3, oAllom.mp_iWhatSaplingHDFunction.GetValue().size());
      assertEquals(3, oAllom.mp_iWhatSeedlingHDFunction.GetValue().size());
      assertEquals(3, oAllom.mp_iWhatAdultCRDFunction.GetValue().size());
      assertEquals(3, oAllom.mp_iWhatSaplingCRDFunction.GetValue().size());
      assertEquals(3, oAllom.mp_iWhatAdultCHHFunction.GetValue().size());
      assertEquals(3, oAllom.mp_iWhatSaplingCHHFunction.GetValue().size());

      for (i = 0; i < 3; i++) {
        assertTrue(null != oAllom.mp_iWhatAdultHDFunction.GetValue().elementAt(i));
        assertTrue(null != oAllom.mp_iWhatSaplingHDFunction.GetValue().elementAt(i));
        assertTrue(null != oAllom.mp_iWhatSeedlingHDFunction.GetValue().elementAt(i));
        assertTrue(null != oAllom.mp_iWhatAdultCRDFunction.GetValue().elementAt(i));
        assertTrue(null != oAllom.mp_iWhatSaplingCRDFunction.GetValue().elementAt(i));
        assertTrue(null != oAllom.mp_iWhatAdultCHHFunction.GetValue().elementAt(i));
        assertTrue(null != oAllom.mp_iWhatSaplingCHHFunction.GetValue().elementAt(i));
      }

      //Verify that the first two species values are still good
      assertEquals( ( (Float) oAllom.mp_fMaxCanopyHeight.GetValue().elementAt(0)).
                   floatValue(), 45, 0.001);
      assertEquals( ( (Float) oAllom.mp_fMaxCanopyHeight.GetValue().elementAt(1)).
                   floatValue(), 39.48, 0.001);

      assertEquals( ( (Float) oAllom.mp_fSlopeOfAsympCrownRad.GetValue().
                     elementAt(0)).floatValue(), 0.02418, 0.001);
      assertEquals( ( (Float) oAllom.mp_fSlopeOfAsympCrownRad.GetValue().
                     elementAt(1)).floatValue(), 0.0549, 0.001);

      assertEquals( ( (Float) oAllom.mp_fCrownRadExp.GetValue().elementAt(0)).
                   floatValue(), 1.1, 0.001);
      assertEquals( ( (Float) oAllom.mp_fCrownRadExp.GetValue().elementAt(1)).
                   floatValue(), 0.9, 0.001);

      assertEquals( ( (Float) oAllom.mp_fDiam10ToDbhSlope.GetValue().
                     elementAt(0)).floatValue(), 0.7059, 0.001);
      assertEquals( ( (Float) oAllom.mp_fDiam10ToDbhSlope.GetValue().
                     elementAt(1)).floatValue(), 0.8008, 0.001);

     assertEquals( ( (Float) oAllom.mp_fDiam10ToDbhIntercept.GetValue().
                     elementAt(0)).floatValue(), 0.0, 0.001);
      assertEquals( ( (Float) oAllom.mp_fDiam10ToDbhIntercept.GetValue().
                     elementAt(1)).floatValue(), 1.3, 0.001);

      assertEquals( ( (Float) oAllom.mp_fSlopeOfAsympCrownHt.GetValue().
                     elementAt(0)).floatValue(), 0.34, 0.001);
      assertEquals( ( (Float) oAllom.mp_fSlopeOfAsympCrownHt.GetValue().
                     elementAt(1)).floatValue(), 0.389, 0.001);

      assertEquals( ( (Float) oAllom.mp_fCrownHtExp.GetValue().elementAt(0)).
                   floatValue(), 0.9, 0.001);
      assertEquals( ( (Float) oAllom.mp_fCrownHtExp.GetValue().elementAt(1)).
                   floatValue(), 1.0, 0.001);

      assertEquals( ( (Float) oAllom.mp_fSlopeOfHeightDiam10.GetValue().
                     elementAt(0)).floatValue(), 0.03418, 0.001);
      assertEquals( ( (Float) oAllom.mp_fSlopeOfHeightDiam10.GetValue().
                     elementAt(1)).floatValue(), 0.2871, 0.001);

      assertEquals( ( (Float) oAllom.mp_fSlopeOfAsymptoticHeight.GetValue().
                     elementAt(0)).floatValue(), 0.0263, 0.001);
      assertEquals( ( (Float) oAllom.mp_fSlopeOfAsymptoticHeight.GetValue().
                     elementAt(1)).floatValue(), 0.0163, 0.001);

      ModelEnum oEnum = (ModelEnum) oAllom.mp_iWhatSeedlingHDFunction.GetValue().
          elementAt(0);
      assertEquals(oEnum.GetValue(), 2);
      oEnum = (ModelEnum) oAllom.mp_iWhatSeedlingHDFunction.GetValue().
          elementAt(1);
      assertEquals(oEnum.GetValue(), 1);

      oEnum = (ModelEnum) oAllom.mp_iWhatSaplingHDFunction.GetValue().elementAt(
          0);
      assertEquals(oEnum.GetValue(), 1);
      oEnum = (ModelEnum) oAllom.mp_iWhatSaplingHDFunction.GetValue().elementAt(
          1);
      assertEquals(oEnum.GetValue(), 1);

      oEnum = (ModelEnum) oAllom.mp_iWhatAdultHDFunction.GetValue().elementAt(0);
      assertEquals(oEnum.GetValue(), 0);
      oEnum = (ModelEnum) oAllom.mp_iWhatAdultHDFunction.GetValue().elementAt(1);
      assertEquals(oEnum.GetValue(), 2);

      oEnum = (ModelEnum) oAllom.mp_iWhatAdultCRDFunction.GetValue().elementAt(
          0);
      assertEquals(oEnum.GetValue(), 0);
      oEnum = (ModelEnum) oAllom.mp_iWhatAdultCRDFunction.GetValue().elementAt(
          1);
      assertEquals(oEnum.GetValue(), 1);

      oEnum = (ModelEnum) oAllom.mp_iWhatSaplingCRDFunction.GetValue().
          elementAt(0);
      assertEquals(oEnum.GetValue(), 1);
      oEnum = (ModelEnum) oAllom.mp_iWhatSaplingCRDFunction.GetValue().
          elementAt(1);
      assertEquals(oEnum.GetValue(), 0);

      oEnum = (ModelEnum) oAllom.mp_iWhatAdultCHHFunction.GetValue().elementAt(
          0);
      assertEquals(oEnum.GetValue(), 1);
      oEnum = (ModelEnum) oAllom.mp_iWhatAdultCHHFunction.GetValue().elementAt(
          1);
      assertEquals(oEnum.GetValue(), 0);

      oEnum = (ModelEnum) oAllom.mp_iWhatSaplingCHHFunction.GetValue().
          elementAt(0);
      assertEquals(oEnum.GetValue(), 1);
      oEnum = (ModelEnum) oAllom.mp_iWhatSaplingCHHFunction.GetValue().
          elementAt(1);
      assertEquals(oEnum.GetValue(), 1);

      assertEquals( ( (Float) oAllom.mp_fAdultLinearSlope.GetValue().
                     elementAt(0)).floatValue(), 0.96, 0.001);
      assertEquals( ( (Float) oAllom.mp_fAdultLinearSlope.GetValue().
                     elementAt(1)).floatValue(), 1.3, 0.001);

      assertEquals( ( (Float) oAllom.mp_fAdultLinearIntercept.GetValue().
                     elementAt(0)).floatValue(), 0, 0.001);
      assertEquals( ( (Float) oAllom.mp_fAdultLinearIntercept.GetValue().
                     elementAt(1)).floatValue(), -0.9, 0.001);

      assertEquals( ( (Float) oAllom.mp_fSaplingLinearSlope.GetValue().
                     elementAt(0)).floatValue(), 0.492, 0.001);
      assertEquals( ( (Float) oAllom.mp_fSaplingLinearSlope.GetValue().
                     elementAt(1)).floatValue(), 0.0549, 0.001);

      assertEquals( ( (Float) oAllom.mp_fSaplingLinearIntercept.GetValue().
                     elementAt(0)).floatValue(), 1.2, 0.001);
      assertEquals( ( (Float) oAllom.mp_fSaplingLinearIntercept.GetValue().
                     elementAt(1)).floatValue(), 0, 0.001);

      assertEquals( ( (Float) oAllom.mp_fSeedlingLinearSlope.GetValue().
                     elementAt(0)).floatValue(), 0.9629, 0.001);
      assertEquals( ( (Float) oAllom.mp_fSeedlingLinearSlope.GetValue().
                     elementAt(1)).floatValue(), 1.228, 0.001);

      assertEquals( ( (Float) oAllom.mp_fSeedlingLinearIntercept.GetValue().
                     elementAt(0)).floatValue(), 0, 0.001);
      assertEquals( ( (Float) oAllom.mp_fSeedlingLinearIntercept.GetValue().
                     elementAt(1)).floatValue(), -0.9, 0.001);

      assertEquals( ( (Float) oAllom.mp_fAdultReverseLinearSlope.GetValue().
                     elementAt(0)).floatValue(), 2.1, 0.001);
     assertEquals( ( (Float) oAllom.mp_fAdultReverseLinearSlope.GetValue().
                     elementAt(1)).floatValue(), 1.9, 0.001);

      assertEquals( ( (Float) oAllom.mp_fAdultReverseLinearIntercept.GetValue().
                     elementAt(0)).floatValue(), 0.02418, 0.001);
     assertEquals( ( (Float) oAllom.mp_fAdultReverseLinearIntercept.GetValue().
                     elementAt(1)).floatValue(), 0.034, 0.001);

      assertEquals( ( (Float) oAllom.mp_fSaplingReverseLinearSlope.GetValue().
                     elementAt(0)).floatValue(), 1.5, 0.001);
      assertEquals( ( (Float) oAllom.mp_fSaplingReverseLinearSlope.GetValue().
                     elementAt(1)).floatValue(), 1.1, 0.001);

      assertEquals( ( (Float) oAllom.mp_fSaplingReverseLinearIntercept.GetValue().
                     elementAt(0)).floatValue(), 0.761, 0.001);
      assertEquals( ( (Float) oAllom.mp_fSaplingReverseLinearIntercept.GetValue().
                     elementAt(1)).floatValue(), -0.847, 0.001);

      assertEquals( ( (Float) oAllom.mp_fSeedlingReverseLinearSlope.GetValue().
                     elementAt(0)).floatValue(), 1.5, 0.001);
      assertEquals( ( (Float) oAllom.mp_fSeedlingReverseLinearSlope.GetValue().
                     elementAt(1)).floatValue(), 0.02, 0.001);

      assertEquals( ( (Float) oAllom.mp_fSeedlingReverseLinearIntercept.
                     GetValue().elementAt(0)).floatValue(), 0.761, 0.001);
      assertEquals( ( (Float) oAllom.mp_fSeedlingReverseLinearIntercept.
                     GetValue().elementAt(1)).floatValue(), 0.758, 0.001);

      assertEquals( ( (Float) oAllom.mp_fCRCrownRadIntercept.GetValue().
                     elementAt(0)).floatValue(), 0.3, 0.001);
      assertEquals( ( (Float) oAllom.mp_fCRCrownRadIntercept.GetValue().
                     elementAt(1)).floatValue(), 0.4, 0.001);

      assertEquals( ( (Float) oAllom.mp_fCRAsympCrownRad.GetValue().
                     elementAt(0)).floatValue(), 8, 0.001);
      assertEquals( ( (Float) oAllom.mp_fCRAsympCrownRad.GetValue().
                     elementAt(1)).floatValue(), 7, 0.001);

      assertEquals( ( (Float) oAllom.mp_fCRCrownRadShape1.GetValue().
                     elementAt(0)).floatValue(), 0.09, 0.001);
      assertEquals( ( (Float) oAllom.mp_fCRCrownRadShape1.GetValue().
                     elementAt(1)).floatValue(), 0.1, 0.001);

      assertEquals( ( (Float) oAllom.mp_fCRCrownRadShape2.GetValue().
                     elementAt(0)).floatValue(), 1.5, 0.001);
      assertEquals( ( (Float) oAllom.mp_fCRCrownRadShape2.GetValue().
                     elementAt(1)).floatValue(), 2.4, 0.001);

      assertEquals( ( (Float) oAllom.mp_fCRCrownHtIntercept.GetValue().
                     elementAt(0)).floatValue(), 0.6, 0.001);
      assertEquals( ( (Float) oAllom.mp_fCRCrownHtIntercept.GetValue().
                     elementAt(1)).floatValue(), 0.5, 0.001);

      assertEquals( ( (Float) oAllom.mp_fCRAsympCrownHt.GetValue().
                     elementAt(0)).floatValue(), 60, 0.001);
      assertEquals( ( (Float) oAllom.mp_fCRAsympCrownHt.GetValue().
                     elementAt(1)).floatValue(), 12, 0.001);

      assertEquals( ( (Float) oAllom.mp_fCRCrownHtShape1.GetValue().
                     elementAt(0)).floatValue(), 0.3, 0.001);
      assertEquals( ( (Float) oAllom.mp_fCRCrownHtShape1.GetValue().
                     elementAt(1)).floatValue(), 0.1, 0.001);

      assertEquals( ( (Float) oAllom.mp_fCRCrownHtShape2.GetValue().
                     elementAt(0)).floatValue(), 3.2, 0.001);
      assertEquals( ( (Float) oAllom.mp_fCRCrownHtShape2.GetValue().
                     elementAt(1)).floatValue(), 4.1, 0.001);
    }
    catch (ModelException oErr) {
      fail("Allometry setup testing failed with message " +
           oErr.getMessage());
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }
  }

  /**
   * Tests species changing. Allometry doesn't have special code for this but
   * I'm making sure it gets treated appropriately.
   */
  public void testChangeOfSpecies() {
    GUIManager oManager = null;
    String sFileName = null;
    try {

      oManager = new GUIManager(null);
      oManager.ClearCurrentData();
      sFileName = WriteXMLFile1();
      oManager.InputXMLParameterFile(sFileName);

      //Now change the species by adding another
      String[] sNewSpecies = new String[] {
            "Species 3",
            "Species 2",
            "Species 1"};
      oManager.GetTreePopulation().SetSpeciesNames(sNewSpecies);
      Allometry oAllom = oManager.GetAllometry();
      int i;

      //Verify that all the enums have enums where they're supposed to
      assertEquals(3, oAllom.mp_iWhatAdultHDFunction.GetValue().size());
      assertEquals(3, oAllom.mp_iWhatSaplingHDFunction.GetValue().size());
      assertEquals(3, oAllom.mp_iWhatSeedlingHDFunction.GetValue().size());
      assertEquals(3, oAllom.mp_iWhatAdultCRDFunction.GetValue().size());
      assertEquals(3, oAllom.mp_iWhatSaplingCRDFunction.GetValue().size());
      assertEquals(3, oAllom.mp_iWhatAdultCHHFunction.GetValue().size());
      assertEquals(3, oAllom.mp_iWhatSaplingCHHFunction.GetValue().size());

      for (i = 0; i < 3; i++) {
        assertTrue(null != oAllom.mp_iWhatAdultHDFunction.GetValue().elementAt(i));
        assertTrue(null != oAllom.mp_iWhatSaplingHDFunction.GetValue().elementAt(i));
        assertTrue(null != oAllom.mp_iWhatSeedlingHDFunction.GetValue().elementAt(i));
        assertTrue(null != oAllom.mp_iWhatAdultCRDFunction.GetValue().elementAt(i));
        assertTrue(null != oAllom.mp_iWhatSaplingCRDFunction.GetValue().elementAt(i));
        assertTrue(null != oAllom.mp_iWhatAdultCHHFunction.GetValue().elementAt(i));
        assertTrue(null != oAllom.mp_iWhatSaplingCHHFunction.GetValue().elementAt(i));
      }

      //Verify that the first two species values are still good
      assertEquals( ( (Float) oAllom.mp_fMaxCanopyHeight.GetValue().elementAt(2)).
                   floatValue(), 45, 0.001);
      assertEquals( ( (Float) oAllom.mp_fMaxCanopyHeight.GetValue().elementAt(1)).
                   floatValue(), 39.48, 0.001);

      assertEquals( ( (Float) oAllom.mp_fSlopeOfAsympCrownRad.GetValue().
                     elementAt(2)).floatValue(), 0.02418, 0.001);
      assertEquals( ( (Float) oAllom.mp_fSlopeOfAsympCrownRad.GetValue().
                     elementAt(1)).floatValue(), 0.0549, 0.001);

      assertEquals( ( (Float) oAllom.mp_fCrownRadExp.GetValue().elementAt(2)).
                   floatValue(), 1.1, 0.001);
      assertEquals( ( (Float) oAllom.mp_fCrownRadExp.GetValue().elementAt(1)).
                   floatValue(), 0.9, 0.001);

      assertEquals( ( (Float) oAllom.mp_fDiam10ToDbhSlope.GetValue().
                     elementAt(2)).floatValue(), 0.7059, 0.001);
      assertEquals( ( (Float) oAllom.mp_fDiam10ToDbhSlope.GetValue().
                     elementAt(1)).floatValue(), 0.8008, 0.001);

     assertEquals( ( (Float) oAllom.mp_fDiam10ToDbhIntercept.GetValue().
                     elementAt(2)).floatValue(), 0.0, 0.001);
      assertEquals( ( (Float) oAllom.mp_fDiam10ToDbhIntercept.GetValue().
                     elementAt(1)).floatValue(), 1.3, 0.001);

      assertEquals( ( (Float) oAllom.mp_fSlopeOfAsympCrownHt.GetValue().
                     elementAt(2)).floatValue(), 0.34, 0.001);
      assertEquals( ( (Float) oAllom.mp_fSlopeOfAsympCrownHt.GetValue().
                     elementAt(1)).floatValue(), 0.389, 0.001);

      assertEquals( ( (Float) oAllom.mp_fCrownHtExp.GetValue().elementAt(2)).
                   floatValue(), 0.9, 0.001);
      assertEquals( ( (Float) oAllom.mp_fCrownHtExp.GetValue().elementAt(1)).
                   floatValue(), 1.0, 0.001);

      assertEquals( ( (Float) oAllom.mp_fSlopeOfHeightDiam10.GetValue().
                     elementAt(2)).floatValue(), 0.03418, 0.001);
      assertEquals( ( (Float) oAllom.mp_fSlopeOfHeightDiam10.GetValue().
                     elementAt(1)).floatValue(), 0.2871, 0.001);

      assertEquals( ( (Float) oAllom.mp_fSlopeOfAsymptoticHeight.GetValue().
                     elementAt(2)).floatValue(), 0.0263, 0.001);
      assertEquals( ( (Float) oAllom.mp_fSlopeOfAsymptoticHeight.GetValue().
                     elementAt(1)).floatValue(), 0.0163, 0.001);

      ModelEnum oEnum = (ModelEnum) oAllom.mp_iWhatSeedlingHDFunction.GetValue().
          elementAt(2);
      assertEquals(oEnum.GetValue(), 2);
      oEnum = (ModelEnum) oAllom.mp_iWhatSeedlingHDFunction.GetValue().
          elementAt(1);
      assertEquals(oEnum.GetValue(), 1);

      oEnum = (ModelEnum) oAllom.mp_iWhatSaplingHDFunction.GetValue().elementAt(2);
      assertEquals(oEnum.GetValue(), 1);
      oEnum = (ModelEnum) oAllom.mp_iWhatSaplingHDFunction.GetValue().elementAt(
          1);
      assertEquals(oEnum.GetValue(), 1);

      oEnum = (ModelEnum) oAllom.mp_iWhatAdultHDFunction.GetValue().elementAt(2);
      assertEquals(oEnum.GetValue(), 0);
      oEnum = (ModelEnum) oAllom.mp_iWhatAdultHDFunction.GetValue().elementAt(1);
      assertEquals(oEnum.GetValue(), 2);

      oEnum = (ModelEnum) oAllom.mp_iWhatAdultCRDFunction.GetValue().elementAt(2);
      assertEquals(oEnum.GetValue(), 0);
      oEnum = (ModelEnum) oAllom.mp_iWhatAdultCRDFunction.GetValue().elementAt(
          1);
      assertEquals(oEnum.GetValue(), 1);

      oEnum = (ModelEnum) oAllom.mp_iWhatSaplingCRDFunction.GetValue().
          elementAt(2);
      assertEquals(oEnum.GetValue(), 1);
      oEnum = (ModelEnum) oAllom.mp_iWhatSaplingCRDFunction.GetValue().
          elementAt(1);
      assertEquals(oEnum.GetValue(), 0);

      oEnum = (ModelEnum) oAllom.mp_iWhatAdultCHHFunction.GetValue().elementAt(2);
      assertEquals(oEnum.GetValue(), 1);
      oEnum = (ModelEnum) oAllom.mp_iWhatAdultCHHFunction.GetValue().elementAt(
          1);
      assertEquals(oEnum.GetValue(), 0);

      oEnum = (ModelEnum) oAllom.mp_iWhatSaplingCHHFunction.GetValue().
          elementAt(2);
      assertEquals(oEnum.GetValue(), 1);
      oEnum = (ModelEnum) oAllom.mp_iWhatSaplingCHHFunction.GetValue().
          elementAt(1);
      assertEquals(oEnum.GetValue(), 1);

      assertEquals( ( (Float) oAllom.mp_fAdultLinearSlope.GetValue().
                     elementAt(2)).floatValue(), 0.96, 0.001);
      assertEquals( ( (Float) oAllom.mp_fAdultLinearSlope.GetValue().
                     elementAt(1)).floatValue(), 1.3, 0.001);

      assertEquals( ( (Float) oAllom.mp_fAdultLinearIntercept.GetValue().
                     elementAt(2)).floatValue(), 0, 0.001);
      assertEquals( ( (Float) oAllom.mp_fAdultLinearIntercept.GetValue().
                     elementAt(1)).floatValue(), -0.9, 0.001);

      assertEquals( ( (Float) oAllom.mp_fSaplingLinearSlope.GetValue().
                     elementAt(2)).floatValue(), 0.492, 0.001);
      assertEquals( ( (Float) oAllom.mp_fSaplingLinearSlope.GetValue().
                     elementAt(1)).floatValue(), 0.0549, 0.001);

      assertEquals( ( (Float) oAllom.mp_fSaplingLinearIntercept.GetValue().
                     elementAt(2)).floatValue(), 1.2, 0.001);
      assertEquals( ( (Float) oAllom.mp_fSaplingLinearIntercept.GetValue().
                     elementAt(1)).floatValue(), 0, 0.001);

      assertEquals( ( (Float) oAllom.mp_fSeedlingLinearSlope.GetValue().
                     elementAt(2)).floatValue(), 0.9629, 0.001);
      assertEquals( ( (Float) oAllom.mp_fSeedlingLinearSlope.GetValue().
                     elementAt(1)).floatValue(), 1.228, 0.001);

      assertEquals( ( (Float) oAllom.mp_fSeedlingLinearIntercept.GetValue().
                     elementAt(2)).floatValue(), 0, 0.001);
      assertEquals( ( (Float) oAllom.mp_fSeedlingLinearIntercept.GetValue().
                     elementAt(1)).floatValue(), -0.9, 0.001);

      assertEquals( ( (Float) oAllom.mp_fAdultReverseLinearSlope.GetValue().
                     elementAt(2)).floatValue(), 2.1, 0.001);
     assertEquals( ( (Float) oAllom.mp_fAdultReverseLinearSlope.GetValue().
                     elementAt(1)).floatValue(), 1.9, 0.001);

      assertEquals( ( (Float) oAllom.mp_fAdultReverseLinearIntercept.GetValue().
                     elementAt(2)).floatValue(), 0.02418, 0.001);
     assertEquals( ( (Float) oAllom.mp_fAdultReverseLinearIntercept.GetValue().
                     elementAt(1)).floatValue(), 0.034, 0.001);

      assertEquals( ( (Float) oAllom.mp_fSaplingReverseLinearSlope.GetValue().
                     elementAt(2)).floatValue(), 1.5, 0.001);
      assertEquals( ( (Float) oAllom.mp_fSaplingReverseLinearSlope.GetValue().
                     elementAt(1)).floatValue(), 1.1, 0.001);

      assertEquals( ( (Float) oAllom.mp_fSaplingReverseLinearIntercept.GetValue().
                     elementAt(2)).floatValue(), 0.761, 0.001);
      assertEquals( ( (Float) oAllom.mp_fSaplingReverseLinearIntercept.GetValue().
                     elementAt(1)).floatValue(), -0.847, 0.001);

      assertEquals( ( (Float) oAllom.mp_fSeedlingReverseLinearSlope.GetValue().
                     elementAt(2)).floatValue(), 1.5, 0.001);
      assertEquals( ( (Float) oAllom.mp_fSeedlingReverseLinearSlope.GetValue().
                     elementAt(1)).floatValue(), 0.02, 0.001);

      assertEquals( ( (Float) oAllom.mp_fSeedlingReverseLinearIntercept.
                     GetValue().elementAt(2)).floatValue(), 0.761, 0.001);
      assertEquals( ( (Float) oAllom.mp_fSeedlingReverseLinearIntercept.
                     GetValue().elementAt(1)).floatValue(), 0.758, 0.001);

      assertEquals( ( (Float) oAllom.mp_fCRCrownRadIntercept.GetValue().
                     elementAt(2)).floatValue(), 0.3, 0.001);
      assertEquals( ( (Float) oAllom.mp_fCRCrownRadIntercept.GetValue().
                     elementAt(1)).floatValue(), 0.4, 0.001);

      assertEquals( ( (Float) oAllom.mp_fCRAsympCrownRad.GetValue().
                     elementAt(2)).floatValue(), 8, 0.001);
      assertEquals( ( (Float) oAllom.mp_fCRAsympCrownRad.GetValue().
                     elementAt(1)).floatValue(), 7, 0.001);

      assertEquals( ( (Float) oAllom.mp_fCRCrownRadShape1.GetValue().
                     elementAt(2)).floatValue(), 0.09, 0.001);
      assertEquals( ( (Float) oAllom.mp_fCRCrownRadShape1.GetValue().
                     elementAt(1)).floatValue(), 0.1, 0.001);

      assertEquals( ( (Float) oAllom.mp_fCRCrownRadShape2.GetValue().
                     elementAt(2)).floatValue(), 1.5, 0.001);
      assertEquals( ( (Float) oAllom.mp_fCRCrownRadShape2.GetValue().
                     elementAt(1)).floatValue(), 2.4, 0.001);

      assertEquals( ( (Float) oAllom.mp_fCRCrownHtIntercept.GetValue().
                     elementAt(2)).floatValue(), 0.6, 0.001);
      assertEquals( ( (Float) oAllom.mp_fCRCrownHtIntercept.GetValue().
                     elementAt(1)).floatValue(), 0.5, 0.001);

      assertEquals( ( (Float) oAllom.mp_fCRAsympCrownHt.GetValue().
                     elementAt(2)).floatValue(), 60, 0.001);
      assertEquals( ( (Float) oAllom.mp_fCRAsympCrownHt.GetValue().
                     elementAt(1)).floatValue(), 12, 0.001);

      assertEquals( ( (Float) oAllom.mp_fCRCrownHtShape1.GetValue().
                     elementAt(2)).floatValue(), 0.3, 0.001);
      assertEquals( ( (Float) oAllom.mp_fCRCrownHtShape1.GetValue().
                     elementAt(1)).floatValue(), 0.1, 0.001);

      assertEquals( ( (Float) oAllom.mp_fCRCrownHtShape2.GetValue().
                     elementAt(2)).floatValue(), 3.2, 0.001);
      assertEquals( ( (Float) oAllom.mp_fCRCrownHtShape2.GetValue().
                     elementAt(1)).floatValue(), 4.1, 0.001);
    }
    catch (ModelException oErr) {
      fail("Allometry setup testing failed with message " +
           oErr.getMessage());
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }
  }


  /**
   * Makes sure all parameters read correctly.
   */
  public void testBackwardsCompatibility() {
    GUIManager oManager = null;
    String sFileName = null;
    try {

      oManager = new GUIManager(null);
      oManager.ClearCurrentData();
      sFileName = Write6_04XMLFile();
      oManager.InputXMLParameterFile(sFileName);
      Allometry oAllom = oManager.GetAllometry();

      assertEquals( ( (Float) oAllom.mp_fMaxCanopyHeight.GetValue().elementAt(0)).
                   floatValue(), 39.48, 0.001);
      assertEquals( ( (Float) oAllom.mp_fMaxCanopyHeight.GetValue().elementAt(1)).
                   floatValue(), 39.54, 0.001);

      assertEquals( ( (Float) oAllom.mp_fSlopeOfAsympCrownRad.GetValue().
                     elementAt(
          0)).
                   floatValue(), 0.0549, 0.001);
      assertEquals( ( (Float) oAllom.mp_fSlopeOfAsympCrownRad.GetValue().
                     elementAt(
          1)).
                   floatValue(), 0.0614, 0.001);

      assertEquals( ( (Float) oAllom.mp_fCrownRadExp.GetValue().elementAt(0)).
                   floatValue(), 0.9, 0.001);
      assertEquals( ( (Float) oAllom.mp_fCrownRadExp.GetValue().elementAt(1)).
                   floatValue(), 1.0, 0.001);

      assertEquals( ( (Float) oAllom.mp_fDiam10ToDbhSlope.GetValue().
                     elementAt(0)).
                   floatValue(), 0.8008, 0.001);
      assertEquals( ( (Float) oAllom.mp_fDiam10ToDbhSlope.GetValue().
                     elementAt(1)).
                   floatValue(), 0.5944, 0.001);

     assertEquals( ( (Float) oAllom.mp_fDiam10ToDbhIntercept.GetValue().
                     elementAt(0)).floatValue(), 0.0, 0.001);
      assertEquals( ( (Float) oAllom.mp_fDiam10ToDbhIntercept.GetValue().
                     elementAt(1)).floatValue(), 0.0, 0.001);

      assertEquals( ( (Float) oAllom.mp_fSlopeOfAsympCrownHt.GetValue().
                     elementAt(0)).
                   floatValue(), 0.389, 0.001);
      assertEquals( ( (Float) oAllom.mp_fSlopeOfAsympCrownHt.GetValue().
                     elementAt(1)).
                   floatValue(), 0.368, 0.001);

      assertEquals( ( (Float) oAllom.mp_fCrownHtExp.GetValue().elementAt(0)).
                   floatValue(), 1.0, 0.001);
      assertEquals( ( (Float) oAllom.mp_fCrownHtExp.GetValue().elementAt(1)).
                   floatValue(), 0.5, 0.001);

      assertEquals( ( (Float) oAllom.mp_fSlopeOfHeightDiam10.GetValue().
                     elementAt(0)).
                   floatValue(), 0.03418, 0.001);
      assertEquals( ( (Float) oAllom.mp_fSlopeOfHeightDiam10.GetValue().
                     elementAt(1)).
                   floatValue(), 0.2871, 0.001);

      assertEquals( ( (Float) oAllom.mp_fSlopeOfAsymptoticHeight.GetValue().
                     elementAt(0)).floatValue(), 0.0299, 0.001);
      assertEquals( ( (Float) oAllom.mp_fSlopeOfAsymptoticHeight.GetValue().
                     elementAt(1)).floatValue(), 0.0241, 0.001);

      ModelEnum oEnum = (ModelEnum) oAllom.mp_iWhatSeedlingHDFunction.GetValue().
          elementAt(0);
      assertEquals(oEnum.GetValue(), 0);
      oEnum = (ModelEnum) oAllom.mp_iWhatSeedlingHDFunction.GetValue().
          elementAt(1);
      assertEquals(oEnum.GetValue(), 2);

      oEnum = (ModelEnum) oAllom.mp_iWhatSaplingHDFunction.GetValue().elementAt(
          0);
      assertEquals(oEnum.GetValue(), 0);
      oEnum = (ModelEnum) oAllom.mp_iWhatSaplingHDFunction.GetValue().elementAt(
          1);
      assertEquals(oEnum.GetValue(), 1);

      oEnum = (ModelEnum) oAllom.mp_iWhatAdultHDFunction.GetValue().elementAt(0);
      assertEquals(oEnum.GetValue(), 0);
      oEnum = (ModelEnum) oAllom.mp_iWhatAdultHDFunction.GetValue().elementAt(1);
      assertEquals(oEnum.GetValue(), 0);

      oEnum = (ModelEnum) oAllom.mp_iWhatAdultCRDFunction.GetValue().elementAt(
          0);
      assertEquals(oEnum.GetValue(), 0);
      oEnum = (ModelEnum) oAllom.mp_iWhatAdultCRDFunction.GetValue().elementAt(
          1);
      assertEquals(oEnum.GetValue(), 0);

      oEnum = (ModelEnum) oAllom.mp_iWhatSaplingCRDFunction.GetValue().
          elementAt(0);
      assertEquals(oEnum.GetValue(), 0);
      oEnum = (ModelEnum) oAllom.mp_iWhatSaplingCRDFunction.GetValue().
          elementAt(1);
      assertEquals(oEnum.GetValue(), 0);

      oEnum = (ModelEnum) oAllom.mp_iWhatAdultCHHFunction.GetValue().elementAt(
          0);
      assertEquals(oEnum.GetValue(), 0);
      oEnum = (ModelEnum) oAllom.mp_iWhatAdultCHHFunction.GetValue().elementAt(
          1);
      assertEquals(oEnum.GetValue(), 0);

      oEnum = (ModelEnum) oAllom.mp_iWhatSaplingCHHFunction.GetValue().
          elementAt(0);
      assertEquals(oEnum.GetValue(), 0);
      oEnum = (ModelEnum) oAllom.mp_iWhatSaplingCHHFunction.GetValue().
          elementAt(1);
      assertEquals(oEnum.GetValue(), 0);

      assertEquals( ( (Float) oAllom.mp_fAdultLinearSlope.GetValue().
                     elementAt(0)).floatValue(), 0.96, 0.001);
      assertEquals( ( (Float) oAllom.mp_fAdultLinearSlope.GetValue().
                     elementAt(1)).floatValue(), 1.3, 0.001);

      assertEquals( ( (Float) oAllom.mp_fAdultLinearIntercept.GetValue().
                     elementAt(0)).floatValue(), 0, 0.001);
      assertEquals( ( (Float) oAllom.mp_fAdultLinearIntercept.GetValue().
                     elementAt(1)).floatValue(), -0.9, 0.001);

      assertEquals( ( (Float) oAllom.mp_fSaplingLinearSlope.GetValue().
                     elementAt(0)).floatValue(), 0.492, 0.001);
      assertEquals( ( (Float) oAllom.mp_fSaplingLinearSlope.GetValue().
                     elementAt(1)).floatValue(), 0.0549, 0.001);

      assertEquals( ( (Float) oAllom.mp_fSaplingLinearIntercept.GetValue().
                     elementAt(0)).floatValue(), 1.2, 0.001);
      assertEquals( ( (Float) oAllom.mp_fSaplingLinearIntercept.GetValue().
                     elementAt(1)).floatValue(), 0, 0.001);

      assertEquals( ( (Float) oAllom.mp_fSeedlingLinearSlope.GetValue().
                     elementAt(0)).floatValue(), 0.9629, 0.001);
      assertEquals( ( (Float) oAllom.mp_fSeedlingLinearSlope.GetValue().
                     elementAt(1)).floatValue(), 1.228, 0.001);

      assertEquals( ( (Float) oAllom.mp_fSeedlingLinearIntercept.GetValue().
                     elementAt(0)).floatValue(), 0, 0.001);
      assertEquals( ( (Float) oAllom.mp_fSeedlingLinearIntercept.GetValue().
                     elementAt(1)).floatValue(), -0.9, 0.001);

      assertEquals( ( (Float) oAllom.mp_fAdultReverseLinearSlope.GetValue().
                     elementAt(0)).floatValue(), 2.1, 0.001);

      assertEquals( ( (Float) oAllom.mp_fAdultReverseLinearIntercept.GetValue().
                     elementAt(0)).floatValue(), 0.02418, 0.001);

      assertEquals( ( (Float) oAllom.mp_fSaplingReverseLinearSlope.GetValue().
                     elementAt(0)).floatValue(), 1.5, 0.001);
      assertEquals( ( (Float) oAllom.mp_fSaplingReverseLinearSlope.GetValue().
                     elementAt(1)).floatValue(), 1.1, 0.001);

      assertEquals( ( (Float) oAllom.mp_fSaplingReverseLinearIntercept.GetValue().
                     elementAt(0)).floatValue(), 0.761, 0.001);
      assertEquals( ( (Float) oAllom.mp_fSaplingReverseLinearIntercept.GetValue().
                     elementAt(1)).floatValue(), -0.847, 0.001);

      assertEquals( ( (Float) oAllom.mp_fSeedlingReverseLinearSlope.GetValue().
                     elementAt(0)).floatValue(), 1.5, 0.001);
      assertEquals( ( (Float) oAllom.mp_fSeedlingReverseLinearSlope.GetValue().
                     elementAt(1)).floatValue(), 0.02, 0.001);

      assertEquals( ( (Float) oAllom.mp_fSeedlingReverseLinearIntercept.
                     GetValue().
                     elementAt(0)).floatValue(), 0.761, 0.001);
      assertEquals( ( (Float) oAllom.mp_fSeedlingReverseLinearIntercept.
                     GetValue().
                     elementAt(1)).floatValue(), 0.758, 0.001);

      assertEquals(oAllom.mp_fCRCrownRadIntercept.GetValue().size(), 0);
      assertEquals(oAllom.mp_fCRAsympCrownRad.GetValue().size(), 0);
      assertEquals(oAllom.mp_fCRCrownRadShape1.GetValue().size(), 0);
      assertEquals(oAllom.mp_fCRCrownRadShape2.GetValue().size(), 0);
      assertEquals(oAllom.mp_fCRCrownHtIntercept.GetValue().size(), 0);
      assertEquals(oAllom.mp_fCRAsympCrownHt.GetValue().size(), 0);
      assertEquals(oAllom.mp_fCRCrownHtShape1.GetValue().size(), 0);
      assertEquals(oAllom.mp_fCRCrownHtShape2.GetValue().size(), 0);
   }
    catch (ModelException oErr) {
      fail("Allometry parameter file reading failed with message " +
           oErr.getMessage());
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
  }

  /**
   * Tests data validation
   */
  public void testValidateData() {
    GUIManager oManager = null;
    String sFileName = null;
    try {

      oManager = new GUIManager(null);
      oManager.ClearCurrentData();
      sFileName = WriteXMLFile1();
      oManager.InputXMLParameterFile(sFileName);
      Allometry oAllom = oManager.GetAllometry();
      TreePopulation oPop = oManager.GetTreePopulation();

      //Case:  Everything's OK
      oAllom.ValidateData(oPop);

      //Case:  adult linear slope is 0 but linear isn't used - not an error
      ModelEnum oEnum = (ModelEnum) oAllom.mp_iWhatAdultHDFunction.GetValue().
          elementAt(0);
      oEnum.SetValue(0);
      oEnum = (ModelEnum) oAllom.mp_iWhatAdultHDFunction.GetValue().elementAt(1);
      oEnum.SetValue(1);
      oAllom.SetVectorValues(oAllom.mp_fAdultLinearSlope,
                             new Float[] {new Float(0), new Float(1.3)});
      oAllom.ValidateData(oPop);

      //Case:  adult linear slope is 0 and linear is used - error
      oEnum = (ModelEnum) oAllom.mp_iWhatAdultHDFunction.GetValue().elementAt(0);
      oEnum.SetValue(1);
      oEnum = (ModelEnum) oAllom.mp_iWhatAdultHDFunction.GetValue().elementAt(1);
      oEnum.SetValue(0);
      try {
        oAllom.ValidateData(oPop);
        fail("ValidateData didn't catch adult linear slope of 0");
      }
      catch (ModelException oErr) {
        ;
      }
      oAllom.SetVectorValues(oAllom.mp_fAdultLinearSlope,
                             new Float[] {new Float(0.1), new Float(1.3)});

      //Case:  sapling linear slope is 0 but linear isn't used - not an error
      oEnum = (ModelEnum) oAllom.mp_iWhatSaplingHDFunction.GetValue().elementAt(
          0);
      oEnum.SetValue(0);
      oEnum = (ModelEnum) oAllom.mp_iWhatSaplingHDFunction.GetValue().elementAt(
          1);
      oEnum.SetValue(1);
      oAllom.SetVectorValues(oAllom.mp_fSaplingLinearSlope,
                             new Float[] {new Float(0), new Float(1.3)});
      oAllom.ValidateData(oPop);

      //Case:  sapling linear slope is 0 and linear is used - error
      oEnum = (ModelEnum) oAllom.mp_iWhatSaplingHDFunction.GetValue().elementAt(
          0);
      oEnum.SetValue(1);
      oEnum = (ModelEnum) oAllom.mp_iWhatSaplingHDFunction.GetValue().elementAt(
          1);
      oEnum.SetValue(0);
      try {
        oAllom.ValidateData(oPop);
        fail("ValidateData didn't catch sapling linear slope of 0");
      }
      catch (ModelException oErr) {
        ;
      }
      oAllom.SetVectorValues(oAllom.mp_fSaplingLinearSlope,
                             new Float[] {new Float(10), new Float(1.3)});

      //Case:  seedling linear slope is 0 but linear isn't used - not an error
      oEnum = (ModelEnum) oAllom.mp_iWhatSeedlingHDFunction.GetValue().
          elementAt(0);
      oEnum.SetValue(0);
      oEnum = (ModelEnum) oAllom.mp_iWhatSeedlingHDFunction.GetValue().
          elementAt(1);
      oEnum.SetValue(1);
      oAllom.SetVectorValues(oAllom.mp_fSeedlingLinearSlope,
                             new Float[] {new Float(0), new Float(1.3)});
      oAllom.ValidateData(oPop);

      //Case:  seedling linear slope is 0 and linear is used - error
      oEnum = (ModelEnum) oAllom.mp_iWhatSeedlingHDFunction.GetValue().
          elementAt(0);
      oEnum.SetValue(1);
      oEnum = (ModelEnum) oAllom.mp_iWhatSeedlingHDFunction.GetValue().
          elementAt(1);
      oEnum.SetValue(0);
      try {
        oAllom.ValidateData(oPop);
        fail("ValidateData didn't catch seedling linear slope of 0");
      }
      catch (ModelException oErr) {
        ;
      }
      oAllom.SetVectorValues(oAllom.mp_fSeedlingLinearSlope,
                             new Float[] {new Float(10), new Float(1.3)});

      //Case:  adult reverse linear slope is 0 but reverse linear isn't used - not an error
      oEnum = (ModelEnum) oAllom.mp_iWhatAdultHDFunction.GetValue().elementAt(0);
      oEnum.SetValue(0);
      oEnum = (ModelEnum) oAllom.mp_iWhatAdultHDFunction.GetValue().elementAt(1);
      oEnum.SetValue(2);
      oAllom.SetVectorValues(oAllom.mp_fAdultReverseLinearSlope,
                             new Float[] {new Float(0), new Float(1.3)});
      oAllom.ValidateData(oPop);

      //Case:  adult reverse linear slope is 0 and reverse linear is used - error
      oEnum = (ModelEnum) oAllom.mp_iWhatAdultHDFunction.GetValue().elementAt(0);
      oEnum.SetValue(2);
      oEnum = (ModelEnum) oAllom.mp_iWhatAdultHDFunction.GetValue().elementAt(1);
      oEnum.SetValue(0);
      try {
        oAllom.ValidateData(oPop);
        fail("ValidateData didn't catch adult reverse linear slope of 0");
      }
      catch (ModelException oErr) {
        ;
      }
      oAllom.SetVectorValues(oAllom.mp_fAdultReverseLinearSlope,
                             new Float[] {new Float(10), new Float(1.3)});

      //Case:  sapling reverse linear slope is 0 but reverse linear isn't used - not an error
      oEnum = (ModelEnum) oAllom.mp_iWhatSaplingHDFunction.GetValue().elementAt(
          0);
      oEnum.SetValue(0);
      oEnum = (ModelEnum) oAllom.mp_iWhatSaplingHDFunction.GetValue().elementAt(
          1);
      oEnum.SetValue(2);
      oAllom.SetVectorValues(oAllom.mp_fSaplingReverseLinearSlope,
                             new Float[] {new Float(0), new Float(1.3)});
      oAllom.ValidateData(oPop);

      //Case:  sapling reverse linear slope is 0 and reverse linear is used - error
      oEnum = (ModelEnum) oAllom.mp_iWhatSaplingHDFunction.GetValue().elementAt(
          0);
      oEnum.SetValue(2);
      oEnum = (ModelEnum) oAllom.mp_iWhatSaplingHDFunction.GetValue().elementAt(
          1);
      oEnum.SetValue(0);
      try {
        oAllom.ValidateData(oPop);
        fail("ValidateData didn't catch sapling reverse linear slope of 0");
      }
      catch (ModelException oErr) {
        ;
      }
      oAllom.SetVectorValues(oAllom.mp_fSaplingReverseLinearSlope,
                             new Float[] {new Float(10), new Float(1.3)});

      //Case:  seedling reverse linear slope is 0 but reverse linear isn't used - not an error
      oEnum = (ModelEnum) oAllom.mp_iWhatSeedlingHDFunction.GetValue().
          elementAt(0);
      oEnum.SetValue(0);
      oEnum = (ModelEnum) oAllom.mp_iWhatSeedlingHDFunction.GetValue().
          elementAt(1);
      oEnum.SetValue(2);
      oAllom.SetVectorValues(oAllom.mp_fSeedlingReverseLinearSlope,
                             new Float[] {new Float(0), new Float(1.3)});
      oAllom.ValidateData(oPop);

      //Case:  seedling reverse linear slope is 0 and reverse linear is used - error
      oEnum = (ModelEnum) oAllom.mp_iWhatSeedlingHDFunction.GetValue().
          elementAt(0);
      oEnum.SetValue(2);
      oEnum = (ModelEnum) oAllom.mp_iWhatSeedlingHDFunction.GetValue().
          elementAt(1);
      oEnum.SetValue(0);
      try {
        oAllom.ValidateData(oPop);
        fail("ValidateData didn't catch seedling reverse linear slope of 0");
      }
      catch (ModelException oErr) {
        ;
      }
      oAllom.SetVectorValues(oAllom.mp_fSeedlingReverseLinearSlope,
                             new Float[] {new Float(10), new Float(1.3)});

      //Case:  Max tree height less than 0
      oAllom.SetVectorValues(oAllom.mp_fMaxCanopyHeight,
                             new Float[] {new Float( -10), new Float(1.3)});
      try {
        oAllom.ValidateData(oPop);
        fail("ValidateData didn't negative max canopy height");
      }
      catch (ModelException oErr) {
        ;
      }
      oAllom.SetVectorValues(oAllom.mp_fMaxCanopyHeight,
                             new Float[] {new Float(10), new Float(10)});
      oAllom.ValidateData(oPop);

      //Case:  slope of asymptotic height is 0 but standard isn't used - not an error
      oEnum = (ModelEnum) oAllom.mp_iWhatAdultHDFunction.GetValue().elementAt(0);
      oEnum.SetValue(2);
      oEnum = (ModelEnum) oAllom.mp_iWhatAdultHDFunction.GetValue().elementAt(1);
      oEnum.SetValue(2);
      oEnum = (ModelEnum) oAllom.mp_iWhatSaplingHDFunction.GetValue().elementAt(
          0);
      oEnum.SetValue(2);
      oEnum = (ModelEnum) oAllom.mp_iWhatSaplingHDFunction.GetValue().elementAt(
          1);
      oEnum.SetValue(2);
      oAllom.SetVectorValues(oAllom.mp_fSlopeOfAsymptoticHeight,
                             new Float[] {new Float( -0.1), new Float(1.3)});
      oAllom.ValidateData(oPop);

      //Case:  slope of asymptotic height is 0 and adult standard is used - error
      oEnum = (ModelEnum) oAllom.mp_iWhatAdultHDFunction.GetValue().elementAt(0);
      oEnum.SetValue(0);
      oEnum = (ModelEnum) oAllom.mp_iWhatAdultHDFunction.GetValue().elementAt(1);
      oEnum.SetValue(2);
      try {
        oAllom.ValidateData(oPop);
        fail("ValidateData didn't catch negative slope of asymptotic height");
      }
      catch (ModelException oErr) {
        ;
      }

      //Case:  slope of asymptotic height is 0 and sapling standard is used - error
      oEnum = (ModelEnum) oAllom.mp_iWhatAdultHDFunction.GetValue().elementAt(0);
      oEnum.SetValue(2);
      oEnum = (ModelEnum) oAllom.mp_iWhatAdultHDFunction.GetValue().elementAt(1);
      oEnum.SetValue(2);
      oEnum = (ModelEnum) oAllom.mp_iWhatSaplingHDFunction.GetValue().elementAt(
          0);
      oEnum.SetValue(0);
      oEnum = (ModelEnum) oAllom.mp_iWhatSaplingHDFunction.GetValue().elementAt(
          1);
      oEnum.SetValue(2);
      try {
        oAllom.ValidateData(oPop);
        fail("ValidateData didn't catch negative slope of asymptotic height");
      }
      catch (ModelException oErr) {
        ;
      }
      oAllom.SetVectorValues(oAllom.mp_fSlopeOfAsymptoticHeight,
                             new Float[] {new Float(0.1), new Float(1.3)});
      oAllom.ValidateData(oPop);

      //Case:  Canrad to dbh ratio less than 0
      oAllom.SetVectorValues(oAllom.mp_fSlopeOfAsympCrownRad,
                             new Float[] {new Float( -10), new Float(1.3)});
      try {
        oAllom.ValidateData(oPop);
        fail("ValidateData didn't negative canrad to dbh ratio");
      }
      catch (ModelException oErr) {
        ;
      }
      oAllom.SetVectorValues(oAllom.mp_fSlopeOfAsympCrownRad,
                             new Float[] {new Float(10), new Float(10)});
      oAllom.ValidateData(oPop);

      //Case:  diam 10 to dbh ratio less than 0
      oAllom.SetVectorValues(oAllom.mp_fDiam10ToDbhSlope,
                             new Float[] {new Float( -10), new Float(1.3)});
      try {
        oAllom.ValidateData(oPop);
        fail("ValidateData didn't negative diam10 to dbh ratio");
      }
      catch (ModelException oErr) {
        ;
      }
      oAllom.SetVectorValues(oAllom.mp_fDiam10ToDbhSlope,
                             new Float[] {new Float(10), new Float(10)});
      oAllom.ValidateData(oPop);

      //Case:  asymptotic slope of canopy depth less than 0
      oAllom.SetVectorValues(oAllom.mp_fSlopeOfAsympCrownHt,
                             new Float[] {new Float( -10), new Float(1.3)});
      oEnum = new ModelEnum(new int[] {1, 0},
                            new String[] {"Chapman-Richards", "Standard"},
                            "Function used", "");
      oEnum.SetValue("Standard");
      oAllom.mp_iWhatAdultCHHFunction.GetValue().add(0, oEnum);
      try {
        oAllom.ValidateData(oPop);
        fail("ValidateData didn't catch negative asymptotic slope of canopy depth");
      }
      catch (ModelException oErr) {
        ;
      }
      oAllom.SetVectorValues(oAllom.mp_fSlopeOfAsympCrownHt,
                             new Float[] {new Float(10), new Float(10)});
      oAllom.ValidateData(oPop);
    }
    catch (ModelException oErr) {
      fail("Allometry validation failed with message " + oErr.getMessage());
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }

  }

  /**
   * Writes a version 6.04 file for testing parameter file reading backwards
   * compatibility.
   * @throws IOException if the file can't be written.
   * @return String Filename.
   */
  private String Write6_04XMLFile() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    java.io.FileWriter oOut = new java.io.FileWriter(sFileName);

    oOut.write("<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>");
    oOut.write("<paramFile fileCode=\"06040101\">");
    oOut.write("<plot>");
    oOut.write("<timesteps>3</timesteps>");
    oOut.write("<yearsPerTimestep>1</yearsPerTimestep>");
    oOut.write("<randomSeed>1</randomSeed>");
    oOut.write("<plot_lenX>200.0</plot_lenX>");
    oOut.write("<plot_lenY>200.0</plot_lenY>");
    oOut.write("<plot_latitude>55.37</plot_latitude>");
    oOut.write("</plot>");
    oOut.write("<trees>");
    oOut.write("<tr_speciesList>");
    oOut.write("<tr_species speciesName=\"Species_1\"/>");
    oOut.write("<tr_species speciesName=\"Species_2\"/>");
    oOut.write("</tr_speciesList>");
    oOut.write("<tr_seedDiam10Cm>0.1</tr_seedDiam10Cm>");
    oOut.write("<tr_minAdultDBH>");
    oOut.write("<tr_madVal species=\"Species_1\">10</tr_madVal>");
    oOut.write("<tr_madVal species=\"Species_2\">10</tr_madVal>");
    oOut.write("</tr_minAdultDBH>");
    oOut.write("</trees>");
    oOut.write("<allometry>");
    oOut.write("<tr_canopyHeight>");
    oOut.write("<tr_chVal species=\"Species_1\">39.48</tr_chVal>");
    oOut.write("<tr_chVal species=\"Species_2\">39.54</tr_chVal>");
    oOut.write("</tr_canopyHeight>");
    oOut.write("<tr_canopyRadiusToDBH>");
    oOut.write("<tr_crtdVal species=\"Species_1\">0.0549</tr_crtdVal>");
    oOut.write("<tr_crtdVal species=\"Species_2\">0.0614</tr_crtdVal>");
    oOut.write("</tr_canopyRadiusToDBH>");
    oOut.write("<tr_canopyRadiusExponent>");
    oOut.write("<tr_creVal species=\"Species_1\">0.9</tr_creVal>");
    oOut.write("<tr_creVal species=\"Species_2\">1.0</tr_creVal>");
    oOut.write("</tr_canopyRadiusExponent>");
    oOut.write("<tr_conversionDiam10ToDBH>");
    oOut.write("<tr_cdtdVal species=\"Species_1\">0.8008</tr_cdtdVal>");
    oOut.write("<tr_cdtdVal species=\"Species_2\">0.5944</tr_cdtdVal>");
    oOut.write("</tr_conversionDiam10ToDBH>");
    oOut.write("<tr_fractOfHeightInCanopy>");
    oOut.write("<tr_fohicVal species=\"Species_1\">0.389</tr_fohicVal>");
    oOut.write("<tr_fohicVal species=\"Species_2\">0.368</tr_fohicVal>");
    oOut.write("</tr_fractOfHeightInCanopy>");
    oOut.write("<tr_canopyDepthExponent>");
    oOut.write("<tr_cdeVal species=\"Species_1\">1.0</tr_cdeVal>");
    oOut.write("<tr_cdeVal species=\"Species_2\">0.5</tr_cdeVal>");
    oOut.write("</tr_canopyDepthExponent>");
    oOut.write("<tr_slopeOfHeight-Diam10>");
    oOut.write("<tr_sohdVal species=\"Species_1\">0.03418</tr_sohdVal>");
    oOut.write("<tr_sohdVal species=\"Species_2\">0.2871</tr_sohdVal>");
    oOut.write("</tr_slopeOfHeight-Diam10>");
    oOut.write("<tr_slopeOfAsymHeight>");
    oOut.write("<tr_soahVal species=\"Species_1\">0.0299</tr_soahVal>");
    oOut.write("<tr_soahVal species=\"Species_2\">0.0241</tr_soahVal>");
    oOut.write("</tr_slopeOfAsymHeight>");
    oOut.write("<tr_whatSeedlingHeightDiam>");
    oOut.write("<tr_wsehdVal species=\"Species_1\">0</tr_wsehdVal>");
    oOut.write("<tr_wsehdVal species=\"Species_2\">2</tr_wsehdVal>");
    oOut.write("</tr_whatSeedlingHeightDiam>");
    oOut.write("<tr_whatSaplingHeightDiam>");
    oOut.write("<tr_wsahdVal species=\"Species_1\">0</tr_wsahdVal>");
    oOut.write("<tr_wsahdVal species=\"Species_2\">1</tr_wsahdVal>");
    oOut.write("</tr_whatSaplingHeightDiam>");
    oOut.write("<tr_whatAdultHeightDiam>");
    oOut.write("<tr_wahdVal species=\"Species_1\">0</tr_wahdVal>");
    oOut.write("<tr_wahdVal species=\"Species_2\">0</tr_wahdVal>");
    oOut.write("</tr_whatAdultHeightDiam>");
    oOut.write("<tr_adultLinearSlope>");
    oOut.write("<tr_alsVal species=\"Species_1\">0.96</tr_alsVal>");
    oOut.write("<tr_alsVal species=\"Species_2\">1.3</tr_alsVal>");
    oOut.write("</tr_adultLinearSlope>");
    oOut.write("<tr_adultLinearIntercept>");
    oOut.write("<tr_aliVal species=\"Species_1\">0</tr_aliVal>");
    oOut.write("<tr_aliVal species=\"Species_2\">-0.9</tr_aliVal>");
    oOut.write("</tr_adultLinearIntercept>");
    oOut.write("<tr_saplingLinearSlope>");
    oOut.write("<tr_salsVal species=\"Species_1\">0.492</tr_salsVal>");
    oOut.write("<tr_salsVal species=\"Species_2\">0.0549</tr_salsVal>");
    oOut.write("</tr_saplingLinearSlope>");
    oOut.write("<tr_saplingLinearIntercept>");
    oOut.write("<tr_saliVal species=\"Species_1\">1.2</tr_saliVal>");
    oOut.write("<tr_saliVal species=\"Species_2\">0</tr_saliVal>");
    oOut.write("</tr_saplingLinearIntercept>");
    oOut.write("<tr_seedlingLinearSlope>");
    oOut.write("<tr_selsVal species=\"Species_1\">0.9629</tr_selsVal>");
    oOut.write("<tr_selsVal species=\"Species_2\">1.228</tr_selsVal>");
    oOut.write("</tr_seedlingLinearSlope>");
    oOut.write("<tr_seedlingLinearIntercept>");
    oOut.write("<tr_seliVal species=\"Species_1\">0</tr_seliVal>");
    oOut.write("<tr_seliVal species=\"Species_2\">-0.9</tr_seliVal>");
    oOut.write("</tr_seedlingLinearIntercept>");
    oOut.write("<tr_adultReverseLinearSlope>");
    oOut.write("<tr_arlsVal species=\"Species_1\">2.1</tr_arlsVal>");
    oOut.write("</tr_adultReverseLinearSlope>");
    oOut.write("<tr_adultReverseLinearIntercept>");
    oOut.write("<tr_arliVal species=\"Species_1\">0.02418</tr_arliVal>");
    oOut.write("</tr_adultReverseLinearIntercept>");
    oOut.write("<tr_saplingReverseLinearSlope>");
    oOut.write("<tr_sarlsVal species=\"Species_1\">1.5</tr_sarlsVal>");
    oOut.write("<tr_sarlsVal species=\"Species_2\">1.1</tr_sarlsVal>");
    oOut.write("</tr_saplingReverseLinearSlope>");
    oOut.write("<tr_saplingReverseLinearIntercept>");
    oOut.write("<tr_sarliVal species=\"Species_1\">0.761</tr_sarliVal>");
    oOut.write("<tr_sarliVal species=\"Species_2\">-0.847</tr_sarliVal>");
    oOut.write("</tr_saplingReverseLinearIntercept>");
    oOut.write("<tr_seedlingReverseLinearSlope>");
    oOut.write("<tr_serlsVal species=\"Species_1\">1.5</tr_serlsVal>");
    oOut.write("<tr_serlsVal species=\"Species_2\">0.02</tr_serlsVal>");
    oOut.write("</tr_seedlingReverseLinearSlope>");
    oOut.write("<tr_seedlingReverseLinearIntercept>");
    oOut.write("<tr_serliVal species=\"Species_1\">0.761</tr_serliVal>");
    oOut.write("<tr_serliVal species=\"Species_2\">0.758</tr_serliVal>");
    oOut.write("</tr_seedlingReverseLinearIntercept>");
    oOut.write("</allometry>");
    oOut.write("<behaviorList>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>radiallight</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Adult\" />");
    oOut.write("</behavior>");
    oOut.write("</behaviorList>");
    oOut.write("</paramFile>");

    oOut.close();

    return sFileName;
  }

  /**
   * Writes a valid file for testing parameter file reading.
   * @throws IOException if the file can't be written.
   * @return String Filename.
   */
  private String WriteXMLFile1() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    java.io.FileWriter oOut = new java.io.FileWriter(sFileName);

    oOut.write("<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>");
    oOut.write("<paramFile fileCode=\"06040101\">");
    oOut.write("<plot>");
    oOut.write("<timesteps>3</timesteps>");
    oOut.write("<yearsPerTimestep>1</yearsPerTimestep>");
    oOut.write("<randomSeed>1</randomSeed>");
    oOut.write("<plot_lenX>200.0</plot_lenX>");
    oOut.write("<plot_lenY>200.0</plot_lenY>");
    oOut.write("<plot_latitude>55.37</plot_latitude>");
    oOut.write("</plot>");
    oOut.write("<trees>");
    oOut.write("<tr_speciesList>");
    oOut.write("<tr_species speciesName=\"Species_1\"/>");
    oOut.write("<tr_species speciesName=\"Species_2\"/>");
    oOut.write("</tr_speciesList>");
    oOut.write("<tr_seedDiam10Cm>0.1</tr_seedDiam10Cm>");
    oOut.write("<tr_minAdultDBH>");
    oOut.write("<tr_madVal species=\"Species_1\">10</tr_madVal>");
    oOut.write("<tr_madVal species=\"Species_2\">10</tr_madVal>");
    oOut.write("</tr_minAdultDBH>");
    oOut.write("<tr_maxSeedlingHeight>");
    oOut.write("<tr_mshVal species=\"Species_1\">1.35</tr_mshVal>");
    oOut.write("<tr_mshVal species=\"Species_2\">1.35</tr_mshVal>");
    oOut.write("</tr_maxSeedlingHeight>");
    oOut.write("</trees>");
    oOut.write("<allometry>");
    oOut.write("<tr_canopyHeight>");
    oOut.write("<tr_chVal species=\"Species_1\">45</tr_chVal>");
    oOut.write("<tr_chVal species=\"Species_2\">39.48</tr_chVal>");
    oOut.write("</tr_canopyHeight>");
    oOut.write("<tr_stdAsympCrownRad>");
    oOut.write("<tr_sacrVal species=\"Species_1\">0.02418</tr_sacrVal>");
    oOut.write("<tr_sacrVal species=\"Species_2\">0.0549</tr_sacrVal>");
    oOut.write("</tr_stdAsympCrownRad>");
    oOut.write("<tr_stdCrownRadExp>");
    oOut.write("<tr_screVal species=\"Species_1\">1.1</tr_screVal>");
    oOut.write("<tr_screVal species=\"Species_2\">0.9</tr_screVal>");
    oOut.write("</tr_stdCrownRadExp>");
    oOut.write("<tr_conversionDiam10ToDBH>");
    oOut.write("<tr_cdtdVal species=\"Species_1\">0.7059</tr_cdtdVal>");
    oOut.write("<tr_cdtdVal species=\"Species_2\">0.8008</tr_cdtdVal>");
    oOut.write("</tr_conversionDiam10ToDBH>");
    oOut.write("<tr_interceptDiam10ToDBH>");
    oOut.write("<tr_idtdVal species=\"Species_1\">0.0</tr_idtdVal>");
    oOut.write("<tr_idtdVal species=\"Species_2\">1.3</tr_idtdVal>");
    oOut.write("</tr_interceptDiam10ToDBH>");
    oOut.write("<tr_stdAsympCrownHt>");
    oOut.write("<tr_sachVal species=\"Species_1\">0.34</tr_sachVal>");
    oOut.write("<tr_sachVal species=\"Species_2\">0.389</tr_sachVal>");
    oOut.write("</tr_stdAsympCrownHt>");
    oOut.write("<tr_stdCrownHtExp>");
    oOut.write("<tr_scheVal species=\"Species_1\">0.9</tr_scheVal>");
    oOut.write("<tr_scheVal species=\"Species_2\">1.0</tr_scheVal>");
    oOut.write("</tr_stdCrownHtExp>");
    oOut.write("<tr_slopeOfHeight-Diam10>");
    oOut.write("<tr_sohdVal species=\"Species_1\">0.03418</tr_sohdVal>");
    oOut.write("<tr_sohdVal species=\"Species_2\">0.2871</tr_sohdVal>");
    oOut.write("</tr_slopeOfHeight-Diam10>");
    oOut.write("<tr_slopeOfAsymHeight>");
    oOut.write("<tr_soahVal species=\"Species_1\">0.0263</tr_soahVal>");
    oOut.write("<tr_soahVal species=\"Species_2\">0.0163</tr_soahVal>");
    oOut.write("</tr_slopeOfAsymHeight>");
    oOut.write("<tr_whatSeedlingHeightDiam>");
    oOut.write("<tr_wsehdVal species=\"Species_1\">2</tr_wsehdVal>");
    oOut.write("<tr_wsehdVal species=\"Species_2\">1</tr_wsehdVal>");
    oOut.write("</tr_whatSeedlingHeightDiam>");
    oOut.write("<tr_whatSaplingHeightDiam>");
    oOut.write("<tr_wsahdVal species=\"Species_1\">1</tr_wsahdVal>");
    oOut.write("<tr_wsahdVal species=\"Species_2\">1</tr_wsahdVal>");
    oOut.write("</tr_whatSaplingHeightDiam>");
    oOut.write("<tr_whatAdultHeightDiam>");
    oOut.write("<tr_wahdVal species=\"Species_1\">0</tr_wahdVal>");
    oOut.write("<tr_wahdVal species=\"Species_2\">2</tr_wahdVal>");
    oOut.write("</tr_whatAdultHeightDiam>");
    oOut.write("<tr_whatAdultCrownRadDiam>");
    oOut.write("<tr_wacrdVal species=\"Species_1\">0</tr_wacrdVal>");
    oOut.write("<tr_wacrdVal species=\"Species_2\">1</tr_wacrdVal>");
    oOut.write("</tr_whatAdultCrownRadDiam>");
    oOut.write("<tr_whatAdultCrownHeightHeight>");
    oOut.write("<tr_wachhVal species=\"Species_1\">1</tr_wachhVal>");
    oOut.write("<tr_wachhVal species=\"Species_2\">0</tr_wachhVal>");
    oOut.write("</tr_whatAdultCrownHeightHeight>");
    oOut.write("<tr_whatSaplingCrownRadDiam>");
    oOut.write("<tr_wscrdVal species=\"Species_1\">1</tr_wscrdVal>");
    oOut.write("<tr_wscrdVal species=\"Species_2\">0</tr_wscrdVal>");
    oOut.write("</tr_whatSaplingCrownRadDiam>");
    oOut.write("<tr_whatSaplingCrownHeightHeight>");
    oOut.write("<tr_wschhVal species=\"Species_1\">1</tr_wschhVal>");
    oOut.write("<tr_wschhVal species=\"Species_2\">1</tr_wschhVal>");
    oOut.write("</tr_whatSaplingCrownHeightHeight>");
    oOut.write("<tr_adultLinearSlope>");
    oOut.write("<tr_alsVal species=\"Species_1\">0.96</tr_alsVal>");
    oOut.write("<tr_alsVal species=\"Species_2\">1.3</tr_alsVal>");
    oOut.write("</tr_adultLinearSlope>");
    oOut.write("<tr_adultLinearIntercept>");
    oOut.write("<tr_aliVal species=\"Species_1\">0</tr_aliVal>");
    oOut.write("<tr_aliVal species=\"Species_2\">-0.9</tr_aliVal>");
    oOut.write("</tr_adultLinearIntercept>");
    oOut.write("<tr_saplingLinearSlope>");
    oOut.write("<tr_salsVal species=\"Species_1\">0.492</tr_salsVal>");
    oOut.write("<tr_salsVal species=\"Species_2\">0.0549</tr_salsVal>");
    oOut.write("</tr_saplingLinearSlope>");
    oOut.write("<tr_saplingLinearIntercept>");
    oOut.write("<tr_saliVal species=\"Species_1\">1.2</tr_saliVal>");
    oOut.write("<tr_saliVal species=\"Species_2\">0</tr_saliVal>");
    oOut.write("</tr_saplingLinearIntercept>");
    oOut.write("<tr_seedlingLinearSlope>");
    oOut.write("<tr_selsVal species=\"Species_1\">0.9629</tr_selsVal>");
    oOut.write("<tr_selsVal species=\"Species_2\">1.228</tr_selsVal>");
    oOut.write("</tr_seedlingLinearSlope>");
    oOut.write("<tr_seedlingLinearIntercept>");
    oOut.write("<tr_seliVal species=\"Species_1\">0</tr_seliVal>");
    oOut.write("<tr_seliVal species=\"Species_2\">-0.9</tr_seliVal>");
    oOut.write("</tr_seedlingLinearIntercept>");
    oOut.write("<tr_adultReverseLinearSlope>");
    oOut.write("<tr_arlsVal species=\"Species_1\">2.1</tr_arlsVal>");
    oOut.write("<tr_arlsVal species=\"Species_2\">1.9</tr_arlsVal>");
    oOut.write("</tr_adultReverseLinearSlope>");
    oOut.write("<tr_adultReverseLinearIntercept>");
    oOut.write("<tr_arliVal species=\"Species_1\">0.02418</tr_arliVal>");
    oOut.write("<tr_arliVal species=\"Species_2\">0.034</tr_arliVal>");
    oOut.write("</tr_adultReverseLinearIntercept>");
    oOut.write("<tr_saplingReverseLinearSlope>");
    oOut.write("<tr_sarlsVal species=\"Species_1\">1.5</tr_sarlsVal>");
    oOut.write("<tr_sarlsVal species=\"Species_2\">1.1</tr_sarlsVal>");
    oOut.write("</tr_saplingReverseLinearSlope>");
    oOut.write("<tr_saplingReverseLinearIntercept>");
    oOut.write("<tr_sarliVal species=\"Species_1\">0.761</tr_sarliVal>");
    oOut.write("<tr_sarliVal species=\"Species_2\">-0.847</tr_sarliVal>");
    oOut.write("</tr_saplingReverseLinearIntercept>");
    oOut.write("<tr_seedlingReverseLinearSlope>");
    oOut.write("<tr_serlsVal species=\"Species_1\">1.5</tr_serlsVal>");
    oOut.write("<tr_serlsVal species=\"Species_2\">0.02</tr_serlsVal>");
    oOut.write("</tr_seedlingReverseLinearSlope>");
    oOut.write("<tr_seedlingReverseLinearIntercept>");
    oOut.write("<tr_serliVal species=\"Species_1\">0.761</tr_serliVal>");
    oOut.write("<tr_serliVal species=\"Species_2\">0.758</tr_serliVal>");
    oOut.write("</tr_seedlingReverseLinearIntercept>");
    oOut.write("<tr_chRichCrownRadIntercept>");
    oOut.write("<tr_crcriVal species=\"Species_1\">0.3</tr_crcriVal>");
    oOut.write("<tr_crcriVal species=\"Species_2\">0.4</tr_crcriVal>");
    oOut.write("</tr_chRichCrownRadIntercept>");
    oOut.write("<tr_chRichCrownRadAsymp>");
    oOut.write("<tr_crcraVal species=\"Species_1\">8</tr_crcraVal>");
    oOut.write("<tr_crcraVal species=\"Species_2\">7</tr_crcraVal>");
    oOut.write("</tr_chRichCrownRadAsymp>");
    oOut.write("<tr_chRichCrownRadShape1b>");
    oOut.write("<tr_crcrs1bVal species=\"Species_1\">0.09</tr_crcrs1bVal>");
    oOut.write("<tr_crcrs1bVal species=\"Species_2\">0.1</tr_crcrs1bVal>");
    oOut.write("</tr_chRichCrownRadShape1b>");
    oOut.write("<tr_chRichCrownRadShape2c>");
    oOut.write("<tr_crcrs2cVal species=\"Species_1\">1.5</tr_crcrs2cVal>");
    oOut.write("<tr_crcrs2cVal species=\"Species_2\">2.4</tr_crcrs2cVal>");
    oOut.write("</tr_chRichCrownRadShape2c>");
    oOut.write("<tr_chRichCrownHtIntercept>");
    oOut.write("<tr_crchiVal species=\"Species_1\">0.6</tr_crchiVal>");
    oOut.write("<tr_crchiVal species=\"Species_2\">0.5</tr_crchiVal>");
    oOut.write("</tr_chRichCrownHtIntercept>");
    oOut.write("<tr_chRichCrownHtAsymp>");
    oOut.write("<tr_crchaVal species=\"Species_1\">60</tr_crchaVal>");
    oOut.write("<tr_crchaVal species=\"Species_2\">12</tr_crchaVal>");
    oOut.write("</tr_chRichCrownHtAsymp>");
    oOut.write("<tr_chRichCrownHtShape1b>");
    oOut.write("<tr_crchs1bVal species=\"Species_1\">0.3</tr_crchs1bVal>");
    oOut.write("<tr_crchs1bVal species=\"Species_2\">0.1</tr_crchs1bVal>");
    oOut.write("</tr_chRichCrownHtShape1b>");
    oOut.write("<tr_chRichCrownHtShape2c>");
    oOut.write("<tr_crchs2cVal species=\"Species_1\">3.2</tr_crchs2cVal>");
    oOut.write("<tr_crchs2cVal species=\"Species_2\">4.1</tr_crchs2cVal>");
    oOut.write("</tr_chRichCrownHtShape2c>");
    oOut.write("</allometry>");
    oOut.write("<behaviorList>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>radiallight</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Adult\" />");
    oOut.write("</behavior>");
    oOut.write("</behaviorList>");
    oOut.write("</paramFile>");

    oOut.close();

    return sFileName;
  }
}
