package javawrapper;

import java.io.FileWriter;
import java.io.File;
import java.awt.event.ActionEvent;
import java.io.IOException;

/**
 * Tests the AnalysisBehaviors class.
 * Copyright: Copyright (c) Charles D. Canham 2003
 * Company: Institute of Ecosystem Studies
 * @author Lora E. Murphy
 * @version 1.0
 */

public class TestAnalysisBehaviors
    extends ModelTestCase {
	
	/**
	 * Makes sure that changes in user parameters are correctly reflected in the
	 * Ripley's K grid and any output settings that may have been made from 
	 * that grid.
	 */
	public void testRipleysKGridChange() {
		
		//Allow the grid to be set up as default
		MainWindow oMainWindow = new MainWindow();
		GUIManager oManager = null;
    String sFileName = null;
    try {
      oManager = oMainWindow.m_oDataManager;
      oManager.ClearCurrentData();
      sFileName = WriteNeutralFile();
      oManager.InputXMLParameterFile(sFileName);
      AnalysisBehaviors oAnalysis = oManager.GetAnalysisBehaviors();
      OutputBehaviors oOutput = oManager.GetOutputBehaviors();
      TreePopulation oPop = oManager.GetTreePopulation();
      Behavior oBeh = oAnalysis.GetBehaviorByKey("Ripley's K");
      oBeh.m_bIsEnabled = true;
      oAnalysis.ValidateData(oPop); //this causes grid setup
      
      //Set the grid settings to use all data members for Ripley's K
      DetailedOutputGridSetup oWindow = new DetailedOutputGridSetup(null, oOutput);
      oWindow.m_jGridListCombo.setSelectedIndex(1);
      oWindow.actionPerformed(new ActionEvent(oWindow.m_jGridListCombo, 0, "OK"));
      int iNumToSelect = oWindow.jDataMemberListModel.getSize(),
          i;
      int[] p_iIndexes = new int[iNumToSelect];
      for (i = 0; i < iNumToSelect; i++)
        p_iIndexes[i] = i;  
      oWindow.jDataMemberList.setSelectedIndices(p_iIndexes);	
      oWindow.jTimestepsEdit.setText("1");
      oWindow.jAddButton.doClick();
      oWindow.actionPerformed(new ActionEvent(this, 0, "OK"));
      //Verify the result
      DetailedGridSettings oSettings = (DetailedGridSettings)oOutput.mp_oDetailedGridSaveSettings.elementAt(0);
      assertEquals(202, oSettings.GetNumberOfFloats());
      
      //Now change the number of increments to 5
      oAnalysis.m_fRipleysKMaxDistance.SetValue(25);
      oAnalysis.m_fRipleysKDistanceInc.SetValue(5);
      oAnalysis.ValidateData(oPop);
      oSettings = (DetailedGridSettings)oOutput.mp_oDetailedGridSaveSettings.elementAt(0);
      assertEquals(22, oSettings.GetNumberOfFloats());
      assertEquals("0_3", oSettings.GetFloat(2).GetCodeName());
      assertEquals("1_3", oSettings.GetFloat(3).GetCodeName());
      assertEquals("2_3", oSettings.GetFloat(4).GetCodeName());
      assertEquals("3_3", oSettings.GetFloat(5).GetCodeName());
      assertEquals("4_3", oSettings.GetFloat(6).GetCodeName());
      
      assertEquals("0_0", oSettings.GetFloat(7).GetCodeName());
      assertEquals("1_0", oSettings.GetFloat(8).GetCodeName());
      assertEquals("2_0", oSettings.GetFloat(9).GetCodeName());
      assertEquals("3_0", oSettings.GetFloat(10).GetCodeName());
      assertEquals("4_0", oSettings.GetFloat(11).GetCodeName());
      
      assertEquals("0_1", oSettings.GetFloat(12).GetCodeName());
      assertEquals("1_1", oSettings.GetFloat(13).GetCodeName());
      assertEquals("2_1", oSettings.GetFloat(14).GetCodeName());
      assertEquals("3_1", oSettings.GetFloat(15).GetCodeName());
      assertEquals("4_1", oSettings.GetFloat(16).GetCodeName());
      
      assertEquals("0_2", oSettings.GetFloat(17).GetCodeName());
      assertEquals("1_2", oSettings.GetFloat(18).GetCodeName());
      assertEquals("2_2", oSettings.GetFloat(19).GetCodeName());
      assertEquals("3_2", oSettings.GetFloat(20).GetCodeName());
      assertEquals("4_2", oSettings.GetFloat(21).GetCodeName());
		
		System.out.println("Ripley's K grid change test succeeded.");
		}
    catch (ModelException oErr) {
      fail("XML volume reading test failed with message " + oErr.getMessage());
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }
	}
	
	/**
	 * Makes sure that changes in user parameters are correctly reflected in the
	 * Relative Neighborhood Density grid and any output settings that may have 
	 * been made from that grid.
	 */
	public void testRelativeNeighborhoodDensityGridChange() {
		
		//Allow the grid to be set up as default
		MainWindow oMainWindow = new MainWindow();
		GUIManager oManager = null;
    String sFileName = null;
    try {
      oManager = oMainWindow.m_oDataManager;
      oManager.ClearCurrentData();
      sFileName = WriteNeutralFile();
      oManager.InputXMLParameterFile(sFileName);
      AnalysisBehaviors oAnalysis = oManager.GetAnalysisBehaviors();
      OutputBehaviors oOutput = oManager.GetOutputBehaviors();
      TreePopulation oPop = oManager.GetTreePopulation();
      Behavior oBeh = oAnalysis.GetBehaviorByKey("Condit's Omega");
      oBeh.m_bIsEnabled = true;
      oAnalysis.ValidateData(oPop); //this causes grid setup
      
      //Set the grid settings to use all data members for Relative Neighborhood
      //Density
      DetailedOutputGridSetup oWindow = new DetailedOutputGridSetup(null, oOutput);
      oWindow.m_jGridListCombo.setSelectedIndex(1);
      oWindow.actionPerformed(new ActionEvent(oWindow.m_jGridListCombo, 0, "OK"));
      int iNumToSelect = oWindow.jDataMemberListModel.getSize(),
          i;
      int[] p_iIndexes = new int[iNumToSelect];
      for (i = 0; i < iNumToSelect; i++)
        p_iIndexes[i] = i;  
      oWindow.jDataMemberList.setSelectedIndices(p_iIndexes);	
      oWindow.jTimestepsEdit.setText("1");
      oWindow.jAddButton.doClick();
      oWindow.actionPerformed(new ActionEvent(this, 0, "OK"));
      //Verify the result
      DetailedGridSettings oSettings = (DetailedGridSettings)oOutput.mp_oDetailedGridSaveSettings.elementAt(0);
      assertEquals(152, oSettings.GetNumberOfFloats());
      
      //Now change the number of increments to 5
      oAnalysis.m_fConditsOmegaMaxDistance.SetValue(25);
      oAnalysis.m_fConditsOmegaDistanceInc.SetValue(5);
      oAnalysis.ValidateData(oPop);
      oSettings = (DetailedGridSettings)oOutput.mp_oDetailedGridSaveSettings.elementAt(0);
      assertEquals("0_0", oSettings.GetFloat(2).GetCodeName());
      assertEquals("1_0", oSettings.GetFloat(3).GetCodeName());
      assertEquals("2_0", oSettings.GetFloat(4).GetCodeName());
      assertEquals("3_0", oSettings.GetFloat(5).GetCodeName());
      assertEquals("4_0", oSettings.GetFloat(6).GetCodeName());
      
      assertEquals("0_1", oSettings.GetFloat(7).GetCodeName());
      assertEquals("1_1", oSettings.GetFloat(8).GetCodeName());
      assertEquals("2_1", oSettings.GetFloat(9).GetCodeName());
      assertEquals("3_1", oSettings.GetFloat(10).GetCodeName());
      assertEquals("4_1", oSettings.GetFloat(11).GetCodeName());
      
      assertEquals("0_2", oSettings.GetFloat(12).GetCodeName());
      assertEquals("1_2", oSettings.GetFloat(13).GetCodeName());
      assertEquals("2_2", oSettings.GetFloat(14).GetCodeName());
      assertEquals("3_2", oSettings.GetFloat(15).GetCodeName());
      assertEquals("4_2", oSettings.GetFloat(16).GetCodeName());
		
		System.out.println("Condit's Omega grid change test succeeded.");
		}
    catch (ModelException oErr) {
      fail("XML volume reading test failed with message " + oErr.getMessage());
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }
	}

  /**
   * Tests reading of parameter file XML volume calculator settings.
   */
  public void testReadXMLVolumeSettings() {
    GUIManager oManager = null;
    String sFileName = null;
    try {
      oManager = new GUIManager(null);
      oManager.ClearCurrentData();
      sFileName = WriteVolumeXMLFile1();
      oManager.InputXMLParameterFile(sFileName);

      AnalysisBehaviors oAnalysis = oManager.GetAnalysisBehaviors();
      assertEquals( ( (Float) oAnalysis.mp_fTaperA.GetValue().elementAt(1)).
                   floatValue(), 0.843142, 0.001);
      assertEquals( ( (Float) oAnalysis.mp_fTaperA.GetValue().elementAt(2)).
                   floatValue(), 0.8, 0.001);

      assertEquals( ( (Float) oAnalysis.mp_fTaperB.GetValue().elementAt(1)).
                   floatValue(), 1.00813, 0.001);
      assertEquals( ( (Float) oAnalysis.mp_fTaperB.GetValue().elementAt(2)).
                   floatValue(), 0.9, 0.001);

      assertEquals( ( (Float) oAnalysis.mp_fTaperC.GetValue().elementAt(1)).
                   floatValue(), 0.031402, 0.001);
      assertEquals( ( (Float) oAnalysis.mp_fTaperC.GetValue().elementAt(2)).
                   floatValue(), 0.032, 0.001);

      assertEquals( ( (Float) oAnalysis.mp_fTaperD.GetValue().elementAt(1)).
                   floatValue(), 0.319773, 0.001);
      assertEquals( ( (Float) oAnalysis.mp_fTaperD.GetValue().elementAt(2)).
                   floatValue(), 0.321, 0.001);

      assertEquals( ( (Float) oAnalysis.mp_fTaperF.GetValue().elementAt(1)).
                   floatValue(), -0.532732, 0.001);
      assertEquals( ( (Float) oAnalysis.mp_fTaperF.GetValue().elementAt(2)).
                   floatValue(), -0.51, 0.001);

      assertEquals( ( (Float) oAnalysis.mp_fTaperG.GetValue().elementAt(1)).
                   floatValue(), 0.585361, 0.001);
      assertEquals( ( (Float) oAnalysis.mp_fTaperG.GetValue().elementAt(2)).
                   floatValue(), 0.59, 0.001);

      assertEquals( ( (Float) oAnalysis.mp_fTaperI.GetValue().elementAt(1)).
                   floatValue(), 1.0315, 0.001);
      assertEquals( ( (Float) oAnalysis.mp_fTaperI.GetValue().elementAt(2)).
                   floatValue(), 1.04, 0.001);

      assertEquals( ( (Float) oAnalysis.mp_fTaperJ.GetValue().elementAt(1)).
                   floatValue(), 0.018622, 0.001);
      assertEquals( ( (Float) oAnalysis.mp_fTaperJ.GetValue().elementAt(2)).
                   floatValue(), 0.02, 0.001);

      assertEquals( ( (Float) oAnalysis.mp_fTaperK.GetValue().elementAt(1)).
                   floatValue(), -0.225146, 0.001);
      assertEquals( ( (Float) oAnalysis.mp_fTaperK.GetValue().elementAt(2)).
                   floatValue(), -0.23, 0.001);

      assertEquals( ( (Float) oAnalysis.mp_fBarkA.GetValue().elementAt(1)).
                   floatValue(), 0.2, 0.001);
      assertEquals( ( (Float) oAnalysis.mp_fBarkA.GetValue().elementAt(2)).
                   floatValue(), 0.01, 0.001);

      assertEquals( ( (Float) oAnalysis.mp_fBarkB.GetValue().elementAt(1)).
                   floatValue(), 0.8, 0.001);
      assertEquals( ( (Float) oAnalysis.mp_fBarkB.GetValue().elementAt(2)).
                   floatValue(), 1, 0.001);

      assertEquals( ( (Float) oAnalysis.mp_fBarkC.GetValue().elementAt(1)).
                   floatValue(), 0.01, 0.001);
      assertEquals( ( (Float) oAnalysis.mp_fBarkC.GetValue().elementAt(2)).
                   floatValue(), 0.001, 0.001);

      assertEquals(oAnalysis.m_fMinUsableDiam.GetValue(), 3, 0.001);
      assertEquals(oAnalysis.m_fSegmentLength.GetValue(), 2, 0.001);
      assertEquals(oAnalysis.m_fStumpHeight.GetValue(), 13, 0.001);

      System.out.println("XML volume reading test succeeded.");
    }
    catch (ModelException oErr) {
      fail("XML volume reading test failed with message " + oErr.getMessage());
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }
  }

  /**
   * Tests reading of parameter file XML bole volume calculator settings.
   */
  public void testReadXMLBoleVolumeSettings() {
    GUIManager oManager = null;
    String sFileName = null;
    try {
      oManager = new GUIManager(null);
      oManager.ClearCurrentData();
      sFileName = WriteBoleVolumeXMLFile1();
      oManager.InputXMLParameterFile(sFileName);

      AnalysisBehaviors oAnalysis = oManager.GetAnalysisBehaviors();
      assertEquals( ( (Float) oAnalysis.mp_fBoleVolumeB0.GetValue().elementAt(1)).
                   floatValue(), 0.11, 0.001);
      assertEquals( ( (Float) oAnalysis.mp_fBoleVolumeB0.GetValue().elementAt(2)).
                   floatValue(), 0.24, 0.001);

      assertEquals( ( (Float) oAnalysis.mp_fBoleVolumeB1.GetValue().elementAt(1)).
                   floatValue(), -0.05977, 0.001);
      assertEquals( ( (Float) oAnalysis.mp_fBoleVolumeB1.GetValue().elementAt(2)).
                   floatValue(), -0.05895, 0.001);

      assertEquals( ( (Float) oAnalysis.mp_fBoleVolumeB2.GetValue().elementAt(1)).
                   floatValue(), 2.0498, 0.001);
      assertEquals( ( (Float) oAnalysis.mp_fBoleVolumeB2.GetValue().elementAt(2)).
                   floatValue(), 2.0362, 0.001);

      assertEquals( ( (Float) oAnalysis.mp_fBoleVolumeB3.GetValue().elementAt(1)).
                   floatValue(), 0.4965, 0.001);
      assertEquals( ( (Float) oAnalysis.mp_fBoleVolumeB3.GetValue().elementAt(2)).
                   floatValue(), 0.4947, 0.001);

      assertEquals( ( (Float) oAnalysis.mp_fBoleVolumeB4.GetValue().elementAt(1)).
                   floatValue(), 2.0198, 0.001);
      assertEquals( ( (Float) oAnalysis.mp_fBoleVolumeB4.GetValue().elementAt(2)).
                   floatValue(), 2.0172, 0.001);

      assertEquals( ( (Float) oAnalysis.mp_fBoleVolumeB5.GetValue().elementAt(1)).
                   floatValue(), 0.3468, 0.001);
      assertEquals( ( (Float) oAnalysis.mp_fBoleVolumeB5.GetValue().elementAt(2)).
                   floatValue(), 0.3366, 0.001);

      assertEquals( ( (Float) oAnalysis.mp_fBoleVolumeFormClasses.GetValue().elementAt(1)).
                   floatValue(), 80, 0.001);
      assertEquals( ( (Float) oAnalysis.mp_fBoleVolumeFormClasses.GetValue().elementAt(2)).
                   floatValue(), 79, 0.001);

      System.out.println("XML bole volume reading test succeeded.");
    }
    catch (ModelException oErr) {
      fail("XML volume reading test failed with message " + oErr.getMessage());
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }
  }

  /**
   * Tests reading of parameter file XML dimension analysis calculator settings.
   */
  public void testReadXMLDimensionAnalysisSettings() {
    GUIManager oManager = null;
    String sFileName = null;
    try {
      oManager = new GUIManager(null);
      ModelEnum oEnum;
      oManager.ClearCurrentData();
      sFileName = WriteDimensionAnalysisXMLFile1();
      oManager.InputXMLParameterFile(sFileName);

      AnalysisBehaviors oAnalysis = oManager.GetAnalysisBehaviors();
      assertEquals( ( (Float) oAnalysis.mp_fBiomassA.GetValue().elementAt(0)).
                   floatValue(), -0.99, 0.001);
      assertEquals( ( (Float) oAnalysis.mp_fBiomassA.GetValue().elementAt(1)).
                   floatValue(), -1.01, 0.001);
      assertEquals( ( (Float) oAnalysis.mp_fBiomassA.GetValue().elementAt(2)).
                   floatValue(), -1.09, 0.001);

      assertEquals( ( (Float) oAnalysis.mp_fBiomassB.GetValue().elementAt(0)).
                   floatValue(), 2.2, 0.001);
      assertEquals( ( (Float) oAnalysis.mp_fBiomassB.GetValue().elementAt(1)).
                   floatValue(), 2.41, 0.001);
      assertEquals( ( (Float) oAnalysis.mp_fBiomassB.GetValue().elementAt(2)).
                   floatValue(), 2.35, 0.001);

      assertEquals( ( (Float) oAnalysis.mp_fBiomassC.GetValue().elementAt(0)).
                   floatValue(), 0.9, 0.001);
      assertEquals( ( (Float) oAnalysis.mp_fBiomassC.GetValue().elementAt(1)).
                   floatValue(), 1, 0.001);
      assertEquals( ( (Float) oAnalysis.mp_fBiomassC.GetValue().elementAt(2)).
                   floatValue(), 1.12, 0.001);

      assertEquals( ( (Float) oAnalysis.mp_fBiomassD.GetValue().elementAt(0)).
                   floatValue(), 1.1, 0.001);
      assertEquals( ( (Float) oAnalysis.mp_fBiomassD.GetValue().elementAt(1)).
                   floatValue(), 1.2, 0.001);
      assertEquals( ( (Float) oAnalysis.mp_fBiomassD.GetValue().elementAt(2)).
                   floatValue(), 1.3, 0.001);

      assertEquals( ( (Float) oAnalysis.mp_fBiomassE.GetValue().elementAt(0)).
                  floatValue(), 0.1, 0.001);
      assertEquals( ( (Float) oAnalysis.mp_fBiomassE.GetValue().elementAt(1)).
                  floatValue(), 0.2, 0.001);
      assertEquals( ( (Float) oAnalysis.mp_fBiomassE.GetValue().elementAt(2)).
                  floatValue(), 0.3, 0.001);

      oEnum = (ModelEnum) oAnalysis.mp_iEquationID.GetValue().elementAt(0);
      assertEquals(oEnum.GetValue(), 4);
      oEnum = (ModelEnum) oAnalysis.mp_iEquationID.GetValue().elementAt(1);
      assertEquals(oEnum.GetValue(), 5);
      oEnum = (ModelEnum) oAnalysis.mp_iEquationID.GetValue().elementAt(2);
      assertEquals(oEnum.GetValue(), 6);

      oEnum = (ModelEnum) oAnalysis.mp_iDbhUnits.GetValue().elementAt(0);
      assertEquals(oEnum.GetValue(), 0);
      oEnum = (ModelEnum) oAnalysis.mp_iDbhUnits.GetValue().elementAt(1);
      assertEquals(oEnum.GetValue(), 1);
      oEnum = (ModelEnum) oAnalysis.mp_iDbhUnits.GetValue().elementAt(2);
      assertEquals(oEnum.GetValue(), 2);

      oEnum = (ModelEnum) oAnalysis.mp_iBiomassUnits.GetValue().elementAt(0);
      assertEquals(oEnum.GetValue(), 2);
      oEnum = (ModelEnum) oAnalysis.mp_iBiomassUnits.GetValue().elementAt(1);
      assertEquals(oEnum.GetValue(), 1);
      oEnum = (ModelEnum) oAnalysis.mp_iBiomassUnits.GetValue().elementAt(2);
      assertEquals(oEnum.GetValue(), 0);

      oEnum = (ModelEnum) oAnalysis.mp_iUseCorrectionFactor.GetValue().elementAt(0);
      assertEquals(oEnum.GetValue(), 0);
      oEnum = (ModelEnum) oAnalysis.mp_iUseCorrectionFactor.GetValue().elementAt(1);
      assertEquals(oEnum.GetValue(), 1);
      oEnum = (ModelEnum) oAnalysis.mp_iUseCorrectionFactor.GetValue().elementAt(2);
      assertEquals(oEnum.GetValue(), 1);

      assertEquals( ( (Float) oAnalysis.mp_fCorrectionFactor.GetValue().elementAt(0)).
                  floatValue(), 1.11, 0.001);
      assertEquals( ( (Float) oAnalysis.mp_fCorrectionFactor.GetValue().elementAt(1)).
                  floatValue(), 1.12, 0.001);
      assertEquals( ( (Float) oAnalysis.mp_fCorrectionFactor.GetValue().elementAt(2)).
                  floatValue(), 1.13, 0.001);

      System.out.println("XML dimension analysis reading test succeeded.");
    }
    catch (ModelException oErr) {
      fail("XML volume reading test failed with message " + oErr.getMessage());
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }
  }
  
  /**
   * Tests reading of parameter file XML Ripley's K calculator settings.
   */
  public void testReadRipleysKSettings() {
    GUIManager oManager = null;
    String sFileName = null;
    try {
      oManager = new GUIManager(null);
      oManager.ClearCurrentData();
      sFileName = WriteRipleysKXMLFile1();
      oManager.InputXMLParameterFile(sFileName);

      AnalysisBehaviors oAnalysis = oManager.GetAnalysisBehaviors();
      assertEquals( 15, oAnalysis.m_fRipleysKMaxDistance.GetValue(), 0.001);
      assertEquals( 5, oAnalysis.m_fRipleysKDistanceInc.GetValue(), 0.001);
      
      System.out.println("Ripley's K parameter reading test succeeded.");
    }
    catch (ModelException oErr) {
      fail("Ripley's K reading test failed with message " + oErr.getMessage());
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }
  }
  
  /**
   * Tests reading of parameter file XML Condit's Omega (Relative Neighborhood
   * Density) calculator settings.
   */
  public void testReadConditsOmegaSettings() {
    GUIManager oManager = null;
    String sFileName = null;
    try {
      oManager = new GUIManager(null);
      oManager.ClearCurrentData();
      sFileName = WriteConditsOmegaXMLFile1();
      oManager.InputXMLParameterFile(sFileName);

      AnalysisBehaviors oAnalysis = oManager.GetAnalysisBehaviors();
      assertEquals( 15, oAnalysis.m_fConditsOmegaMaxDistance.GetValue(), 0.001);
      assertEquals( 5, oAnalysis.m_fConditsOmegaDistanceInc.GetValue(), 0.001);
      
      System.out.println("Relative Neighborhood Density calculator parameter " +
      		               "reading test succeeded.");
    }
    catch (ModelException oErr) {
      fail("Relative Neighborhood Density calculator parameter reading test " +
      		 "failed with message " + oErr.getMessage());
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }
  }

  /**
   * Tests species changes. Even though AnalysisBehaviors doesn't explicitly
   * have code for changing species, this makes sure that it is treated
   * correctly.
   */
  public void testChangeOfSpecies() {
    GUIManager oManager = null;
    String sFileName = null;
    try {
      oManager = new GUIManager(null);
      ModelEnum oEnum;
      oManager.ClearCurrentData();
      sFileName = WriteComboXMLFile1();
      oManager.InputXMLParameterFile(sFileName);

      //dimension analysis settings
      AnalysisBehaviors oAnalysis = oManager.GetAnalysisBehaviors();
      assertEquals( ( (Float) oAnalysis.mp_fBiomassA.GetValue().elementAt(0)).
                   floatValue(), -0.99, 0.001);
      assertEquals( ( (Float) oAnalysis.mp_fBiomassA.GetValue().elementAt(1)).
                   floatValue(), -1.01, 0.001);

      assertEquals( ( (Float) oAnalysis.mp_fBiomassB.GetValue().elementAt(0)).
                   floatValue(), 2.2, 0.001);
      assertEquals( ( (Float) oAnalysis.mp_fBiomassB.GetValue().elementAt(1)).
                   floatValue(), 2.41, 0.001);

      assertEquals( ( (Float) oAnalysis.mp_fBiomassC.GetValue().elementAt(0)).
                   floatValue(), 0.9, 0.001);
      assertEquals( ( (Float) oAnalysis.mp_fBiomassC.GetValue().elementAt(1)).
                   floatValue(), 1, 0.001);

      assertEquals( ( (Float) oAnalysis.mp_fBiomassD.GetValue().elementAt(0)).
                   floatValue(), 1.1, 0.001);
      assertEquals( ( (Float) oAnalysis.mp_fBiomassD.GetValue().elementAt(1)).
                   floatValue(), 1.2, 0.001);

      assertEquals( ( (Float) oAnalysis.mp_fBiomassE.GetValue().elementAt(0)).
                  floatValue(), 0.1, 0.001);
      assertEquals( ( (Float) oAnalysis.mp_fBiomassE.GetValue().elementAt(1)).
                  floatValue(), 0.2, 0.001);

      oEnum = (ModelEnum) oAnalysis.mp_iEquationID.GetValue().elementAt(0);
      assertEquals(oEnum.GetValue(), 4);
      oEnum = (ModelEnum) oAnalysis.mp_iEquationID.GetValue().elementAt(1);
      assertEquals(oEnum.GetValue(), 5);

      oEnum = (ModelEnum) oAnalysis.mp_iDbhUnits.GetValue().elementAt(0);
      assertEquals(oEnum.GetValue(), 0);
      oEnum = (ModelEnum) oAnalysis.mp_iDbhUnits.GetValue().elementAt(1);
      assertEquals(oEnum.GetValue(), 1);

      oEnum = (ModelEnum) oAnalysis.mp_iBiomassUnits.GetValue().elementAt(0);
      assertEquals(oEnum.GetValue(), 2);
      oEnum = (ModelEnum) oAnalysis.mp_iBiomassUnits.GetValue().elementAt(1);
      assertEquals(oEnum.GetValue(), 1);

      oEnum = (ModelEnum) oAnalysis.mp_iUseCorrectionFactor.GetValue().elementAt(0);
      assertEquals(oEnum.GetValue(), 0);
      oEnum = (ModelEnum) oAnalysis.mp_iUseCorrectionFactor.GetValue().elementAt(1);
      assertEquals(oEnum.GetValue(), 1);

      assertEquals( ( (Float) oAnalysis.mp_fCorrectionFactor.GetValue().elementAt(0)).
                  floatValue(), 1.11, 0.001);
      assertEquals( ( (Float) oAnalysis.mp_fCorrectionFactor.GetValue().elementAt(1)).
                  floatValue(), 1.12, 0.001);

      //Volume
      assertEquals( ( (Float) oAnalysis.mp_fTaperA.GetValue().elementAt(0)).
                   floatValue(), 0.843142, 0.001);
      assertEquals( ( (Float) oAnalysis.mp_fTaperA.GetValue().elementAt(1)).
                   floatValue(), 0.8, 0.001);

      assertEquals( ( (Float) oAnalysis.mp_fTaperB.GetValue().elementAt(0)).
                   floatValue(), 1.00813, 0.001);
      assertEquals( ( (Float) oAnalysis.mp_fTaperB.GetValue().elementAt(1)).
                   floatValue(), 0.9, 0.001);

      assertEquals( ( (Float) oAnalysis.mp_fTaperC.GetValue().elementAt(0)).
                   floatValue(), 0.031402, 0.001);
      assertEquals( ( (Float) oAnalysis.mp_fTaperC.GetValue().elementAt(1)).
                   floatValue(), 0.032, 0.001);

      assertEquals( ( (Float) oAnalysis.mp_fTaperD.GetValue().elementAt(0)).
                   floatValue(), 0.319773, 0.001);
      assertEquals( ( (Float) oAnalysis.mp_fTaperD.GetValue().elementAt(1)).
                   floatValue(), 0.321, 0.001);

      assertEquals( ( (Float) oAnalysis.mp_fTaperF.GetValue().elementAt(0)).
                   floatValue(), -0.532732, 0.001);
      assertEquals( ( (Float) oAnalysis.mp_fTaperF.GetValue().elementAt(1)).
                   floatValue(), -0.51, 0.001);

      assertEquals( ( (Float) oAnalysis.mp_fTaperG.GetValue().elementAt(0)).
                   floatValue(), 0.585361, 0.001);
      assertEquals( ( (Float) oAnalysis.mp_fTaperG.GetValue().elementAt(1)).
                   floatValue(), 0.59, 0.001);

      assertEquals( ( (Float) oAnalysis.mp_fTaperI.GetValue().elementAt(0)).
                   floatValue(), 1.0315, 0.001);
      assertEquals( ( (Float) oAnalysis.mp_fTaperI.GetValue().elementAt(1)).
                   floatValue(), 1.04, 0.001);

      assertEquals( ( (Float) oAnalysis.mp_fTaperJ.GetValue().elementAt(0)).
                   floatValue(), 0.018622, 0.001);
      assertEquals( ( (Float) oAnalysis.mp_fTaperJ.GetValue().elementAt(1)).
                   floatValue(), 0.02, 0.001);

      assertEquals( ( (Float) oAnalysis.mp_fTaperK.GetValue().elementAt(0)).
                   floatValue(), -0.225146, 0.001);
      assertEquals( ( (Float) oAnalysis.mp_fTaperK.GetValue().elementAt(1)).
                   floatValue(), -0.23, 0.001);

      assertEquals( ( (Float) oAnalysis.mp_fBarkA.GetValue().elementAt(0)).
                   floatValue(), 0.2, 0.001);
      assertEquals( ( (Float) oAnalysis.mp_fBarkA.GetValue().elementAt(1)).
                   floatValue(), 0.01, 0.001);

      assertEquals( ( (Float) oAnalysis.mp_fBarkB.GetValue().elementAt(0)).
                   floatValue(), 0.8, 0.001);
      assertEquals( ( (Float) oAnalysis.mp_fBarkB.GetValue().elementAt(1)).
                   floatValue(), 1, 0.001);

      assertEquals( ( (Float) oAnalysis.mp_fBarkC.GetValue().elementAt(0)).
                   floatValue(), 0.01, 0.001);
      assertEquals( ( (Float) oAnalysis.mp_fBarkC.GetValue().elementAt(1)).
                   floatValue(), 0.001, 0.001);

      //Bole volume
      assertEquals( ( (Float) oAnalysis.mp_fBoleVolumeB0.GetValue().elementAt(0)).
                   floatValue(), 0.11, 0.001);
      assertEquals( ( (Float) oAnalysis.mp_fBoleVolumeB0.GetValue().elementAt(1)).
                   floatValue(), 0.24, 0.001);

      assertEquals( ( (Float) oAnalysis.mp_fBoleVolumeB1.GetValue().elementAt(0)).
                   floatValue(), -0.05977, 0.001);
      assertEquals( ( (Float) oAnalysis.mp_fBoleVolumeB1.GetValue().elementAt(1)).
                   floatValue(), -0.05895, 0.001);

      assertEquals( ( (Float) oAnalysis.mp_fBoleVolumeB2.GetValue().elementAt(0)).
                   floatValue(), 2.0498, 0.001);
      assertEquals( ( (Float) oAnalysis.mp_fBoleVolumeB2.GetValue().elementAt(1)).
                   floatValue(), 2.0362, 0.001);

      assertEquals( ( (Float) oAnalysis.mp_fBoleVolumeB3.GetValue().elementAt(0)).
                   floatValue(), 0.4965, 0.001);
      assertEquals( ( (Float) oAnalysis.mp_fBoleVolumeB3.GetValue().elementAt(1)).
                   floatValue(), 0.4947, 0.001);

      assertEquals( ( (Float) oAnalysis.mp_fBoleVolumeB4.GetValue().elementAt(0)).
                   floatValue(), 2.0198, 0.001);
      assertEquals( ( (Float) oAnalysis.mp_fBoleVolumeB4.GetValue().elementAt(1)).
                   floatValue(), 2.0172, 0.001);

      assertEquals( ( (Float) oAnalysis.mp_fBoleVolumeB5.GetValue().elementAt(0)).
                   floatValue(), 0.3468, 0.001);
      assertEquals( ( (Float) oAnalysis.mp_fBoleVolumeB5.GetValue().elementAt(1)).
                   floatValue(), 0.3366, 0.001);

      assertEquals( ( (Float) oAnalysis.mp_fBoleVolumeFormClasses.GetValue().elementAt(0)).
                   floatValue(), 80, 0.001);
      assertEquals( ( (Float) oAnalysis.mp_fBoleVolumeFormClasses.GetValue().elementAt(1)).
                   floatValue(), 79, 0.001);

      //Timber volume values
      oEnum = (ModelEnum) oAnalysis.mp_fMerchValueFormClasses.GetValue().elementAt(0);
      assertEquals(oEnum.GetValue(), 78);
      oEnum = (ModelEnum) oAnalysis.mp_fMerchValueFormClasses.GetValue().elementAt(1);
      assertEquals(oEnum.GetValue(), 79);

      assertEquals( ( (Float) oAnalysis.mp_fMerchValuePricePer1K.GetValue().elementAt(0)).
                   floatValue(), 1.23, 0.001);
      assertEquals( ( (Float) oAnalysis.mp_fMerchValuePricePer1K.GetValue().elementAt(1)).
                   floatValue(), 0.89, 0.001);

      //Carbon value values
      assertEquals( ( (Float) oAnalysis.mp_fCarbonValuePercentBiomassCarbon.GetValue().elementAt(0)).
                   floatValue(), 0, 0.001);
      assertEquals( ( (Float) oAnalysis.mp_fCarbonValuePercentBiomassCarbon.GetValue().elementAt(1)).
                   floatValue(), 100, 0.001);

      //Partitioned biomass values
      assertEquals( ( (Float) oAnalysis.mp_fPartBioDbhLeafA.GetValue().elementAt(0)).
                   floatValue(), 1.523, 0.001);
      assertEquals( ( (Float) oAnalysis.mp_fPartBioDbhLeafA.GetValue().elementAt(1)).
                   floatValue(), 1.792, 0.001);

      assertEquals( ( (Float) oAnalysis.mp_fPartBioDbhLeafB.GetValue().elementAt(0)).
                   floatValue(), -3.49, 0.001);
      assertEquals( ( (Float) oAnalysis.mp_fPartBioDbhLeafB.GetValue().elementAt(1)).
                   floatValue(), 3.758, 0.001);

      assertEquals( ( (Float) oAnalysis.mp_fPartBioDbhBranchA.GetValue().elementAt(0)).
                   floatValue(), 2.062, 0.001);
      assertEquals( ( (Float) oAnalysis.mp_fPartBioDbhBranchA.GetValue().elementAt(1)).
                   floatValue(), 1.982, 0.001);

      assertEquals( ( (Float) oAnalysis.mp_fPartBioDbhBranchB.GetValue().elementAt(0)).
                   floatValue(), -5.963, 0.001);
      assertEquals( ( (Float) oAnalysis.mp_fPartBioDbhBranchB.GetValue().elementAt(1)).
                   floatValue(), 3.69, 0.001);

      assertEquals( ( (Float) oAnalysis.mp_fPartBioDbhBoleA.GetValue().elementAt(0)).
                   floatValue(), 2.543, 0.001);
      assertEquals( ( (Float) oAnalysis.mp_fPartBioDbhBoleA.GetValue().elementAt(1)).
                   floatValue(), 2.559, 0.001);

      assertEquals( ( (Float) oAnalysis.mp_fPartBioDbhBoleB.GetValue().elementAt(0)).
                   floatValue(), -3.139, 0.001);
      assertEquals( ( (Float) oAnalysis.mp_fPartBioDbhBoleB.GetValue().elementAt(1)).
                   floatValue(), 2.832, 0.001);

      assertEquals( ( (Float) oAnalysis.mp_fPartBioHeightLeafA.GetValue().elementAt(0)).
                    floatValue(), 1.7, 0.001);
      assertEquals( ( (Float) oAnalysis.mp_fPartBioHeightLeafA.GetValue().elementAt(1)).
                    floatValue(), 1.3, 0.001);

      assertEquals( ( (Float) oAnalysis.mp_fPartBioHeightLeafB.GetValue().elementAt(0)).
                    floatValue(), 6.8, 0.001);
      assertEquals( ( (Float) oAnalysis.mp_fPartBioHeightLeafB.GetValue().elementAt(1)).
                    floatValue(), -2.3, 0.001);

      assertEquals( ( (Float) oAnalysis.mp_fPartBioHeightBoleA.GetValue().elementAt(0)).
                    floatValue(), 5.9, 0.001);
      assertEquals( ( (Float) oAnalysis.mp_fPartBioHeightBoleA.GetValue().elementAt(1)).
                    floatValue(), 6.9, 0.001);

      assertEquals( ( (Float) oAnalysis.mp_fPartBioHeightBoleB.GetValue().elementAt(0)).
                    floatValue(), -1.6, 0.001);
      assertEquals( ( (Float) oAnalysis.mp_fPartBioHeightBoleB.GetValue().elementAt(1)).
                    floatValue(), -1.1, 0.001);


      //Now change the species
      String[] sNewSpecies = new String[] {
            "Species 3",
            "Species 2",
            "Species 1"};
      oManager.GetTreePopulation().SetSpeciesNames(sNewSpecies);
      oAnalysis = oManager.GetAnalysisBehaviors();
      assertEquals( ( (Float) oAnalysis.mp_fBiomassA.GetValue().elementAt(2)).
                   floatValue(), -0.99, 0.001);
      assertEquals( ( (Float) oAnalysis.mp_fBiomassA.GetValue().elementAt(1)).
                   floatValue(), -1.01, 0.001);

      assertEquals( ( (Float) oAnalysis.mp_fBiomassB.GetValue().elementAt(2)).
                   floatValue(), 2.2, 0.001);
      assertEquals( ( (Float) oAnalysis.mp_fBiomassB.GetValue().elementAt(1)).
                   floatValue(), 2.41, 0.001);

      assertEquals( ( (Float) oAnalysis.mp_fBiomassC.GetValue().elementAt(2)).
                   floatValue(), 0.9, 0.001);
      assertEquals( ( (Float) oAnalysis.mp_fBiomassC.GetValue().elementAt(1)).
                   floatValue(), 1, 0.001);

      assertEquals( ( (Float) oAnalysis.mp_fBiomassD.GetValue().elementAt(2)).
                   floatValue(), 1.1, 0.001);
      assertEquals( ( (Float) oAnalysis.mp_fBiomassD.GetValue().elementAt(1)).
                   floatValue(), 1.2, 0.001);

      assertEquals( ( (Float) oAnalysis.mp_fBiomassE.GetValue().elementAt(2)).
                  floatValue(), 0.1, 0.001);
      assertEquals( ( (Float) oAnalysis.mp_fBiomassE.GetValue().elementAt(1)).
                  floatValue(), 0.2, 0.001);

      int i;
      assertEquals(3, oAnalysis.mp_iEquationID.GetValue().size());
      for (i = 0; i < 3; i++) {
        assertTrue(null != oAnalysis.mp_iEquationID.GetValue().elementAt(i));
      }
      oEnum = (ModelEnum) oAnalysis.mp_iEquationID.GetValue().elementAt(2);
      assertEquals(oEnum.GetValue(), 4);
      oEnum = (ModelEnum) oAnalysis.mp_iEquationID.GetValue().elementAt(1);
      assertEquals(oEnum.GetValue(), 5);

      assertEquals(3, oAnalysis.mp_iDbhUnits.GetValue().size());
      for (i = 0; i < 3; i++) {
        assertTrue(null != oAnalysis.mp_iDbhUnits.GetValue().elementAt(i));
      }
      oEnum = (ModelEnum) oAnalysis.mp_iDbhUnits.GetValue().elementAt(2);
      assertEquals(oEnum.GetValue(), 0);
      oEnum = (ModelEnum) oAnalysis.mp_iDbhUnits.GetValue().elementAt(1);
      assertEquals(oEnum.GetValue(), 1);

      assertEquals(3, oAnalysis.mp_iBiomassUnits.GetValue().size());
      for (i = 0; i < 3; i++) {
        assertTrue(null != oAnalysis.mp_iBiomassUnits.GetValue().elementAt(i));
      }
      oEnum = (ModelEnum) oAnalysis.mp_iBiomassUnits.GetValue().elementAt(2);
      assertEquals(oEnum.GetValue(), 2);
      oEnum = (ModelEnum) oAnalysis.mp_iBiomassUnits.GetValue().elementAt(1);
      assertEquals(oEnum.GetValue(), 1);

      assertEquals(3, oAnalysis.mp_iUseCorrectionFactor.GetValue().size());
      for (i = 0; i < 3; i++) {
        assertTrue(null != oAnalysis.mp_iUseCorrectionFactor.GetValue().elementAt(i));
      }
      oEnum = (ModelEnum) oAnalysis.mp_iUseCorrectionFactor.GetValue().elementAt(2);
      assertEquals(oEnum.GetValue(), 0);
      oEnum = (ModelEnum) oAnalysis.mp_iUseCorrectionFactor.GetValue().elementAt(1);
      assertEquals(oEnum.GetValue(), 1);

      assertEquals( ( (Float) oAnalysis.mp_fCorrectionFactor.GetValue().elementAt(2)).
                  floatValue(), 1.11, 0.001);
      assertEquals( ( (Float) oAnalysis.mp_fCorrectionFactor.GetValue().elementAt(1)).
                  floatValue(), 1.12, 0.001);

      //Volume
      assertEquals( ( (Float) oAnalysis.mp_fTaperA.GetValue().elementAt(2)).
                   floatValue(), 0.843142, 0.001);
      assertEquals( ( (Float) oAnalysis.mp_fTaperA.GetValue().elementAt(1)).
                   floatValue(), 0.8, 0.001);

      assertEquals( ( (Float) oAnalysis.mp_fTaperB.GetValue().elementAt(2)).
                   floatValue(), 1.00813, 0.001);
      assertEquals( ( (Float) oAnalysis.mp_fTaperB.GetValue().elementAt(1)).
                   floatValue(), 0.9, 0.001);

      assertEquals( ( (Float) oAnalysis.mp_fTaperC.GetValue().elementAt(2)).
                   floatValue(), 0.031402, 0.001);
      assertEquals( ( (Float) oAnalysis.mp_fTaperC.GetValue().elementAt(1)).
                   floatValue(), 0.032, 0.001);

      assertEquals( ( (Float) oAnalysis.mp_fTaperD.GetValue().elementAt(2)).
                   floatValue(), 0.319773, 0.001);
      assertEquals( ( (Float) oAnalysis.mp_fTaperD.GetValue().elementAt(1)).
                   floatValue(), 0.321, 0.001);

      assertEquals( ( (Float) oAnalysis.mp_fTaperF.GetValue().elementAt(2)).
                   floatValue(), -0.532732, 0.001);
      assertEquals( ( (Float) oAnalysis.mp_fTaperF.GetValue().elementAt(1)).
                   floatValue(), -0.51, 0.001);

      assertEquals( ( (Float) oAnalysis.mp_fTaperG.GetValue().elementAt(2)).
                   floatValue(), 0.585361, 0.001);
      assertEquals( ( (Float) oAnalysis.mp_fTaperG.GetValue().elementAt(1)).
                   floatValue(), 0.59, 0.001);

      assertEquals( ( (Float) oAnalysis.mp_fTaperI.GetValue().elementAt(2)).
                   floatValue(), 1.0315, 0.001);
      assertEquals( ( (Float) oAnalysis.mp_fTaperI.GetValue().elementAt(1)).
                   floatValue(), 1.04, 0.001);

      assertEquals( ( (Float) oAnalysis.mp_fTaperJ.GetValue().elementAt(2)).
                   floatValue(), 0.018622, 0.001);
      assertEquals( ( (Float) oAnalysis.mp_fTaperJ.GetValue().elementAt(1)).
                   floatValue(), 0.02, 0.001);

      assertEquals( ( (Float) oAnalysis.mp_fTaperK.GetValue().elementAt(2)).
                   floatValue(), -0.225146, 0.001);
      assertEquals( ( (Float) oAnalysis.mp_fTaperK.GetValue().elementAt(1)).
                   floatValue(), -0.23, 0.001);

      assertEquals( ( (Float) oAnalysis.mp_fBarkA.GetValue().elementAt(2)).
                   floatValue(), 0.2, 0.001);
      assertEquals( ( (Float) oAnalysis.mp_fBarkA.GetValue().elementAt(1)).
                   floatValue(), 0.01, 0.001);

      assertEquals( ( (Float) oAnalysis.mp_fBarkB.GetValue().elementAt(2)).
                   floatValue(), 0.8, 0.001);
      assertEquals( ( (Float) oAnalysis.mp_fBarkB.GetValue().elementAt(1)).
                   floatValue(), 1, 0.001);

      assertEquals( ( (Float) oAnalysis.mp_fBarkC.GetValue().elementAt(2)).
                   floatValue(), 0.01, 0.001);
      assertEquals( ( (Float) oAnalysis.mp_fBarkC.GetValue().elementAt(1)).
                   floatValue(), 0.001, 0.001);

      //Bole volume
      assertEquals( ( (Float) oAnalysis.mp_fBoleVolumeB0.GetValue().elementAt(2)).
                   floatValue(), 0.11, 0.001);
      assertEquals( ( (Float) oAnalysis.mp_fBoleVolumeB0.GetValue().elementAt(1)).
                   floatValue(), 0.24, 0.001);

      assertEquals( ( (Float) oAnalysis.mp_fBoleVolumeB1.GetValue().elementAt(2)).
                   floatValue(), -0.05977, 0.001);
      assertEquals( ( (Float) oAnalysis.mp_fBoleVolumeB1.GetValue().elementAt(1)).
                   floatValue(), -0.05895, 0.001);

      assertEquals( ( (Float) oAnalysis.mp_fBoleVolumeB2.GetValue().elementAt(2)).
                   floatValue(), 2.0498, 0.001);
      assertEquals( ( (Float) oAnalysis.mp_fBoleVolumeB2.GetValue().elementAt(1)).
                   floatValue(), 2.0362, 0.001);

      assertEquals( ( (Float) oAnalysis.mp_fBoleVolumeB3.GetValue().elementAt(2)).
                   floatValue(), 0.4965, 0.001);
      assertEquals( ( (Float) oAnalysis.mp_fBoleVolumeB3.GetValue().elementAt(1)).
                   floatValue(), 0.4947, 0.001);

      assertEquals( ( (Float) oAnalysis.mp_fBoleVolumeB4.GetValue().elementAt(2)).
                   floatValue(), 2.0198, 0.001);
      assertEquals( ( (Float) oAnalysis.mp_fBoleVolumeB4.GetValue().elementAt(1)).
                   floatValue(), 2.0172, 0.001);

      assertEquals( ( (Float) oAnalysis.mp_fBoleVolumeB5.GetValue().elementAt(2)).
                   floatValue(), 0.3468, 0.001);
      assertEquals( ( (Float) oAnalysis.mp_fBoleVolumeB5.GetValue().elementAt(1)).
                   floatValue(), 0.3366, 0.001);

      assertEquals( ( (Float) oAnalysis.mp_fBoleVolumeFormClasses.GetValue().elementAt(2)).
                   floatValue(), 80, 0.001);
      assertEquals( ( (Float) oAnalysis.mp_fBoleVolumeFormClasses.GetValue().elementAt(1)).
                   floatValue(), 79, 0.001);

      //Timber volume values
      assertEquals(3, oAnalysis.mp_fMerchValueFormClasses.GetValue().size());
      for (i = 0; i < 3; i++) {
        assertTrue(null != oAnalysis.mp_fMerchValueFormClasses.GetValue().elementAt(i));
      }
      oEnum = (ModelEnum) oAnalysis.mp_fMerchValueFormClasses.GetValue().elementAt(2);
      assertEquals(oEnum.GetValue(), 78);
      oEnum = (ModelEnum) oAnalysis.mp_fMerchValueFormClasses.GetValue().elementAt(1);
      assertEquals(oEnum.GetValue(), 79);

      assertEquals( ( (Float) oAnalysis.mp_fMerchValuePricePer1K.GetValue().elementAt(2)).
                   floatValue(), 1.23, 0.001);
      assertEquals( ( (Float) oAnalysis.mp_fMerchValuePricePer1K.GetValue().elementAt(1)).
                   floatValue(), 0.89, 0.001);

      //Carbon value values
      assertEquals( ( (Float) oAnalysis.mp_fCarbonValuePercentBiomassCarbon.GetValue().elementAt(2)).
                   floatValue(), 0, 0.001);
      assertEquals( ( (Float) oAnalysis.mp_fCarbonValuePercentBiomassCarbon.GetValue().elementAt(1)).
                   floatValue(), 100, 0.001);

      //Partitioned biomass values
      assertEquals( ( (Float) oAnalysis.mp_fPartBioDbhLeafA.GetValue().elementAt(2)).
                   floatValue(), 1.523, 0.001);
      assertEquals( ( (Float) oAnalysis.mp_fPartBioDbhLeafA.GetValue().elementAt(1)).
                   floatValue(), 1.792, 0.001);

      assertEquals( ( (Float) oAnalysis.mp_fPartBioDbhLeafB.GetValue().elementAt(2)).
                   floatValue(), -3.49, 0.001);
      assertEquals( ( (Float) oAnalysis.mp_fPartBioDbhLeafB.GetValue().elementAt(1)).
                   floatValue(), 3.758, 0.001);

      assertEquals( ( (Float) oAnalysis.mp_fPartBioDbhBranchA.GetValue().elementAt(2)).
                   floatValue(), 2.062, 0.001);
      assertEquals( ( (Float) oAnalysis.mp_fPartBioDbhBranchA.GetValue().elementAt(1)).
                   floatValue(), 1.982, 0.001);

      assertEquals( ( (Float) oAnalysis.mp_fPartBioDbhBranchB.GetValue().elementAt(2)).
                   floatValue(), -5.963, 0.001);
      assertEquals( ( (Float) oAnalysis.mp_fPartBioDbhBranchB.GetValue().elementAt(1)).
                   floatValue(), 3.69, 0.001);

      assertEquals( ( (Float) oAnalysis.mp_fPartBioDbhBoleA.GetValue().elementAt(2)).
                   floatValue(), 2.543, 0.001);
      assertEquals( ( (Float) oAnalysis.mp_fPartBioDbhBoleA.GetValue().elementAt(1)).
                   floatValue(), 2.559, 0.001);

      assertEquals( ( (Float) oAnalysis.mp_fPartBioDbhBoleB.GetValue().elementAt(2)).
                   floatValue(), -3.139, 0.001);
      assertEquals( ( (Float) oAnalysis.mp_fPartBioDbhBoleB.GetValue().elementAt(1)).
                   floatValue(), 2.832, 0.001);

      assertEquals( ( (Float) oAnalysis.mp_fPartBioHeightLeafA.GetValue().elementAt(2)).
                    floatValue(), 1.7, 0.001);
      assertEquals( ( (Float) oAnalysis.mp_fPartBioHeightLeafA.GetValue().elementAt(1)).
                    floatValue(), 1.3, 0.001);

      assertEquals( ( (Float) oAnalysis.mp_fPartBioHeightLeafB.GetValue().elementAt(2)).
                    floatValue(), 6.8, 0.001);
      assertEquals( ( (Float) oAnalysis.mp_fPartBioHeightLeafB.GetValue().elementAt(1)).
                    floatValue(), -2.3, 0.001);

      assertEquals( ( (Float) oAnalysis.mp_fPartBioHeightBoleA.GetValue().elementAt(2)).
                    floatValue(), 5.9, 0.001);
      assertEquals( ( (Float) oAnalysis.mp_fPartBioHeightBoleA.GetValue().elementAt(1)).
                    floatValue(), 6.9, 0.001);

      assertEquals( ( (Float) oAnalysis.mp_fPartBioHeightBoleB.GetValue().elementAt(2)).
                    floatValue(), -1.6, 0.001);
      assertEquals( ( (Float) oAnalysis.mp_fPartBioHeightBoleB.GetValue().elementAt(1)).
                    floatValue(), -1.1, 0.001);


      System.out.println("Change of species test succeeded.");
    }
    catch (ModelException oErr) {
      fail("Change of species test failed with message " + oErr.getMessage());
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }
  }
  
  /**
   * Tests reading parameter file values for partitioned biomass, including
   * 6.8 backwards compatibility.
   */
  public void testReadPartitionedBiomassSettings() {
    GUIManager oManager = null;
    String sFileName = null;
    try {
      oManager = new GUIManager(null);
      
      //Backwards compatibility
      sFileName = Write68PartitionedBiomassXMLFile();
      oManager.InputXMLParameterFile(sFileName);
      AnalysisBehaviors oAnalysis = oManager.GetAnalysisBehaviors();
      
      assertEquals( ( (Float) oAnalysis.mp_fPartBioDbhLeafA.GetValue().elementAt(0)).
                   floatValue(), 1.523, 0.001);
      assertEquals( ( (Float) oAnalysis.mp_fPartBioDbhLeafA.GetValue().elementAt(1)).
                   floatValue(), 1.792, 0.001);

      assertEquals( ( (Float) oAnalysis.mp_fPartBioDbhLeafB.GetValue().elementAt(0)).
                   floatValue(), -3.49, 0.001);
      assertEquals( ( (Float) oAnalysis.mp_fPartBioDbhLeafB.GetValue().elementAt(1)).
                   floatValue(), 3.758, 0.001);

      assertEquals( ( (Float) oAnalysis.mp_fPartBioDbhBranchA.GetValue().elementAt(0)).
                   floatValue(), 2.062, 0.001);
      assertEquals( ( (Float) oAnalysis.mp_fPartBioDbhBranchA.GetValue().elementAt(1)).
                   floatValue(), 1.982, 0.001);

      assertEquals( ( (Float) oAnalysis.mp_fPartBioDbhBranchB.GetValue().elementAt(0)).
                   floatValue(), -5.963, 0.001);
      assertEquals( ( (Float) oAnalysis.mp_fPartBioDbhBranchB.GetValue().elementAt(1)).
                   floatValue(), 3.69, 0.001);

      assertEquals( ( (Float) oAnalysis.mp_fPartBioDbhBoleA.GetValue().elementAt(0)).
                   floatValue(), 2.543, 0.001);
      assertEquals( ( (Float) oAnalysis.mp_fPartBioDbhBoleA.GetValue().elementAt(1)).
                   floatValue(), 2.559, 0.001);

      assertEquals( ( (Float) oAnalysis.mp_fPartBioDbhBoleB.GetValue().elementAt(0)).
                   floatValue(), -3.139, 0.001);
      assertEquals( ( (Float) oAnalysis.mp_fPartBioDbhBoleB.GetValue().elementAt(1)).
                   floatValue(), 2.832, 0.001);

      assertEquals( ( (Float) oAnalysis.mp_fPartBioHeightLeafA.GetValue().elementAt(0)).
                    floatValue(), 1.7, 0.001);
      assertEquals( ( (Float) oAnalysis.mp_fPartBioHeightLeafA.GetValue().elementAt(1)).
                    floatValue(), 1.3, 0.001);

      assertEquals( ( (Float) oAnalysis.mp_fPartBioHeightLeafB.GetValue().elementAt(0)).
                    floatValue(), 6.8, 0.001);
      assertEquals( ( (Float) oAnalysis.mp_fPartBioHeightLeafB.GetValue().elementAt(1)).
                    floatValue(), -2.3, 0.001);

      assertEquals( ( (Float) oAnalysis.mp_fPartBioHeightBoleA.GetValue().elementAt(0)).
                    floatValue(), 5.9, 0.001);
      assertEquals( ( (Float) oAnalysis.mp_fPartBioHeightBoleA.GetValue().elementAt(1)).
                    floatValue(), 6.9, 0.001);

      assertEquals( ( (Float) oAnalysis.mp_fPartBioHeightBoleB.GetValue().elementAt(0)).
                    floatValue(), -1.6, 0.001);
      assertEquals( ( (Float) oAnalysis.mp_fPartBioHeightBoleB.GetValue().elementAt(1)).
                    floatValue(), -1.1, 0.001);
      
      assertEquals(3, oAnalysis.mp_fPartBioDBHNA.GetValue().size());
      assertEquals(((Float) oAnalysis.mp_fPartBioDBHNA.GetValue().elementAt(0)).
          floatValue(), 0, 0.001);
      assertEquals(((Float) oAnalysis.mp_fPartBioDBHNA.GetValue().elementAt(1)).
          floatValue(), 0, 0.001);
      
      assertEquals(3, oAnalysis.mp_fPartBioDBHNB.GetValue().size());
      assertEquals(((Float) oAnalysis.mp_fPartBioDBHNB.GetValue().elementAt(0)).
          floatValue(), 0, 0.001);
      assertEquals(((Float) oAnalysis.mp_fPartBioDBHNB.GetValue().elementAt(1)).
          floatValue(), 0, 0.001);
      
      assertEquals(3, oAnalysis.mp_fPartBioDBHPA.GetValue().size());
      assertEquals(((Float) oAnalysis.mp_fPartBioDBHPA.GetValue().elementAt(0)).
          floatValue(), 0, 0.001);
      assertEquals(((Float) oAnalysis.mp_fPartBioDBHPA.GetValue().elementAt(1)).
          floatValue(), 0, 0.001);
      
      assertEquals(3, oAnalysis.mp_fPartBioDBHPB.GetValue().size());
      assertEquals(((Float) oAnalysis.mp_fPartBioDBHPB.GetValue().elementAt(0)).
          floatValue(), 0, 0.001);
      assertEquals(((Float) oAnalysis.mp_fPartBioDBHPB.GetValue().elementAt(1)).
          floatValue(), 0, 0.001);
      
      assertEquals(3, oAnalysis.mp_fPartBioDBHLigninA.GetValue().size());
      assertEquals(((Float) oAnalysis.mp_fPartBioDBHLigninA.GetValue().elementAt(0)).
          floatValue(), 0, 0.001);
      assertEquals(((Float) oAnalysis.mp_fPartBioDBHLigninA.GetValue().elementAt(1)).
          floatValue(), 0, 0.001);
      
      assertEquals(3, oAnalysis.mp_fPartBioDBHLigninB.GetValue().size());
      assertEquals(((Float) oAnalysis.mp_fPartBioDBHLigninB.GetValue().elementAt(0)).
          floatValue(), 0, 0.001);
      assertEquals(((Float) oAnalysis.mp_fPartBioDBHLigninB.GetValue().elementAt(1)).
          floatValue(), 0, 0.001);
      
      assertEquals(3, oAnalysis.mp_fPartBioDBHFiberA.GetValue().size());
      assertEquals(((Float) oAnalysis.mp_fPartBioDBHFiberA.GetValue().elementAt(0)).
          floatValue(), 0, 0.001);
      assertEquals(((Float) oAnalysis.mp_fPartBioDBHFiberA.GetValue().elementAt(1)).
          floatValue(), 0, 0.001);
      
      assertEquals(3, oAnalysis.mp_fPartBioDBHFiberB.GetValue().size());
      assertEquals(((Float) oAnalysis.mp_fPartBioDBHFiberB.GetValue().elementAt(0)).
          floatValue(), 0, 0.001);
      assertEquals(((Float) oAnalysis.mp_fPartBioDBHFiberB.GetValue().elementAt(1)).
          floatValue(), 0, 0.001);
      
      assertEquals(3, oAnalysis.mp_fPartBioDBHCelluloseA.GetValue().size());
      assertEquals(((Float) oAnalysis.mp_fPartBioDBHCelluloseA.GetValue().elementAt(0)).
          floatValue(), 0, 0.001);
      assertEquals(((Float) oAnalysis.mp_fPartBioDBHCelluloseA.GetValue().elementAt(1)).
          floatValue(), 0, 0.001);
      
      assertEquals(3, oAnalysis.mp_fPartBioDBHCelluloseB.GetValue().size());
      assertEquals(((Float) oAnalysis.mp_fPartBioDBHCelluloseB.GetValue().elementAt(0)).
          floatValue(), 0, 0.001);
      assertEquals(((Float) oAnalysis.mp_fPartBioDBHCelluloseB.GetValue().elementAt(1)).
          floatValue(), 0, 0.001);
      
      assertEquals(3, oAnalysis.mp_fPartBioDBHTanninsA.GetValue().size());
      assertEquals(((Float) oAnalysis.mp_fPartBioDBHTanninsA.GetValue().elementAt(0)).
          floatValue(), 0, 0.001);
      assertEquals(((Float) oAnalysis.mp_fPartBioDBHTanninsA.GetValue().elementAt(1)).
          floatValue(), 0, 0.001);
      
      assertEquals(3, oAnalysis.mp_fPartBioDBHTanninsB.GetValue().size());
      assertEquals(((Float) oAnalysis.mp_fPartBioDBHTanninsB.GetValue().elementAt(0)).
          floatValue(), 0, 0.001);
      assertEquals(((Float) oAnalysis.mp_fPartBioDBHTanninsB.GetValue().elementAt(1)).
          floatValue(), 0, 0.001);
      
      assertEquals(3, oAnalysis.mp_fPartBioDBHPhenolicsA.GetValue().size());
      assertEquals(((Float) oAnalysis.mp_fPartBioDBHPhenolicsA.GetValue().elementAt(0)).
          floatValue(), 0, 0.001);
      assertEquals(((Float) oAnalysis.mp_fPartBioDBHPhenolicsA.GetValue().elementAt(1)).
          floatValue(), 0, 0.001);
      
      assertEquals(3, oAnalysis.mp_fPartBioDBHPhenolicsB.GetValue().size());
      assertEquals(((Float) oAnalysis.mp_fPartBioDBHPhenolicsB.GetValue().elementAt(0)).
          floatValue(), 0, 0.001);
      assertEquals(((Float) oAnalysis.mp_fPartBioDBHPhenolicsB.GetValue().elementAt(1)).
          floatValue(), 0, 0.001);
      
      assertEquals(3, oAnalysis.mp_fPartBioDBHSLAA.GetValue().size());
      assertEquals(((Float) oAnalysis.mp_fPartBioDBHSLAA.GetValue().elementAt(0)).
          floatValue(), 0, 0.001);
      assertEquals(((Float) oAnalysis.mp_fPartBioDBHSLAA.GetValue().elementAt(1)).
          floatValue(), 0, 0.001); 
      
      assertEquals(3, oAnalysis.mp_fPartBioDBHSLAB.GetValue().size());
      assertEquals(((Float) oAnalysis.mp_fPartBioDBHSLAB.GetValue().elementAt(0)).
          floatValue(), 0, 0.001);
      assertEquals(((Float) oAnalysis.mp_fPartBioDBHSLAB.GetValue().elementAt(1)).
          floatValue(), 0, 0.001);
      
      //Regular file
      sFileName = WritePartitionedBiomassXMLFile1();
      oManager.InputXMLParameterFile(sFileName);
      oAnalysis = oManager.GetAnalysisBehaviors();
            
      assertEquals( ( (Float) oAnalysis.mp_fPartBioDbhLeafA.GetValue().elementAt(0)).
                   floatValue(), 1.523, 0.001);
      assertEquals( ( (Float) oAnalysis.mp_fPartBioDbhLeafA.GetValue().elementAt(1)).
                   floatValue(), 1.792, 0.001);

      assertEquals( ( (Float) oAnalysis.mp_fPartBioDbhLeafB.GetValue().elementAt(0)).
                   floatValue(), -3.49, 0.001);
      assertEquals( ( (Float) oAnalysis.mp_fPartBioDbhLeafB.GetValue().elementAt(1)).
                   floatValue(), 3.758, 0.001);

      assertEquals( ( (Float) oAnalysis.mp_fPartBioDbhBranchA.GetValue().elementAt(0)).
                   floatValue(), 2.062, 0.001);
      assertEquals( ( (Float) oAnalysis.mp_fPartBioDbhBranchA.GetValue().elementAt(1)).
                   floatValue(), 1.982, 0.001);

      assertEquals( ( (Float) oAnalysis.mp_fPartBioDbhBranchB.GetValue().elementAt(0)).
                   floatValue(), -5.963, 0.001);
      assertEquals( ( (Float) oAnalysis.mp_fPartBioDbhBranchB.GetValue().elementAt(1)).
                   floatValue(), 3.69, 0.001);

      assertEquals( ( (Float) oAnalysis.mp_fPartBioDbhBoleA.GetValue().elementAt(0)).
                   floatValue(), 2.543, 0.001);
      assertEquals( ( (Float) oAnalysis.mp_fPartBioDbhBoleA.GetValue().elementAt(1)).
                   floatValue(), 2.559, 0.001);

      assertEquals( ( (Float) oAnalysis.mp_fPartBioDbhBoleB.GetValue().elementAt(0)).
                   floatValue(), -3.139, 0.001);
      assertEquals( ( (Float) oAnalysis.mp_fPartBioDbhBoleB.GetValue().elementAt(1)).
                   floatValue(), 2.832, 0.001);

      assertEquals( ( (Float) oAnalysis.mp_fPartBioHeightLeafA.GetValue().elementAt(0)).
                    floatValue(), 1.7, 0.001);
      assertEquals( ( (Float) oAnalysis.mp_fPartBioHeightLeafA.GetValue().elementAt(1)).
                    floatValue(), 1.3, 0.001);

      assertEquals( ( (Float) oAnalysis.mp_fPartBioHeightLeafB.GetValue().elementAt(0)).
                    floatValue(), 6.8, 0.001);
      assertEquals( ( (Float) oAnalysis.mp_fPartBioHeightLeafB.GetValue().elementAt(1)).
                    floatValue(), -2.3, 0.001);

      assertEquals( ( (Float) oAnalysis.mp_fPartBioHeightBoleA.GetValue().elementAt(0)).
                    floatValue(), 5.9, 0.001);
      assertEquals( ( (Float) oAnalysis.mp_fPartBioHeightBoleA.GetValue().elementAt(1)).
                    floatValue(), 6.9, 0.001);

      assertEquals( ( (Float) oAnalysis.mp_fPartBioHeightBoleB.GetValue().elementAt(0)).
                    floatValue(), -1.6, 0.001);
      assertEquals( ( (Float) oAnalysis.mp_fPartBioHeightBoleB.GetValue().elementAt(1)).
                    floatValue(), -1.1, 0.001);
      
      assertEquals(((Float) oAnalysis.mp_fPartBioDBHNA.GetValue().elementAt(0)).
          floatValue(), 0.12, 0.001);
      assertEquals(((Float) oAnalysis.mp_fPartBioDBHNA.GetValue().elementAt(1)).
          floatValue(), 1.12, 0.001);
      
      assertEquals(((Float) oAnalysis.mp_fPartBioDBHNB.GetValue().elementAt(0)).
          floatValue(), 0.13, 0.001);
      assertEquals(((Float) oAnalysis.mp_fPartBioDBHNB.GetValue().elementAt(1)).
          floatValue(), 1.13, 0.001);
      
      assertEquals(((Float) oAnalysis.mp_fPartBioDBHPA.GetValue().elementAt(0)).
          floatValue(), 0.14, 0.001);
      assertEquals(((Float) oAnalysis.mp_fPartBioDBHPA.GetValue().elementAt(1)).
          floatValue(), 1.14, 0.001);
      
      assertEquals(((Float) oAnalysis.mp_fPartBioDBHPB.GetValue().elementAt(0)).
          floatValue(), 0.15, 0.001);
      assertEquals(((Float) oAnalysis.mp_fPartBioDBHPB.GetValue().elementAt(1)).
          floatValue(), 1.15, 0.001);
      
      assertEquals(((Float) oAnalysis.mp_fPartBioDBHLigninA.GetValue().elementAt(0)).
          floatValue(), 0.18, 0.001);
      assertEquals(((Float) oAnalysis.mp_fPartBioDBHLigninA.GetValue().elementAt(1)).
          floatValue(), 1.18, 0.001);
      
      assertEquals(((Float) oAnalysis.mp_fPartBioDBHLigninB.GetValue().elementAt(0)).
          floatValue(), 0.19, 0.001);
      assertEquals(((Float) oAnalysis.mp_fPartBioDBHLigninB.GetValue().elementAt(1)).
          floatValue(), 1.19, 0.001);
      
      assertEquals(((Float) oAnalysis.mp_fPartBioDBHFiberA.GetValue().elementAt(0)).
          floatValue(), 0.20, 0.001);
      assertEquals(((Float) oAnalysis.mp_fPartBioDBHFiberA.GetValue().elementAt(1)).
          floatValue(), 1.2, 0.001);
      
      assertEquals(((Float) oAnalysis.mp_fPartBioDBHFiberB.GetValue().elementAt(0)).
          floatValue(), 0.21, 0.001);
      assertEquals(((Float) oAnalysis.mp_fPartBioDBHFiberB.GetValue().elementAt(1)).
          floatValue(), 1.21, 0.001);
      
      assertEquals(((Float) oAnalysis.mp_fPartBioDBHCelluloseA.GetValue().elementAt(0)).
          floatValue(), 0.22, 0.001);
      assertEquals(((Float) oAnalysis.mp_fPartBioDBHCelluloseA.GetValue().elementAt(1)).
          floatValue(), 1.22, 0.001);
      
      assertEquals(((Float) oAnalysis.mp_fPartBioDBHCelluloseB.GetValue().elementAt(0)).
          floatValue(), 0.23, 0.001);
      assertEquals(((Float) oAnalysis.mp_fPartBioDBHCelluloseB.GetValue().elementAt(1)).
          floatValue(), 1.23, 0.001);
      
      assertEquals(((Float) oAnalysis.mp_fPartBioDBHTanninsA.GetValue().elementAt(0)).
          floatValue(), 0.24, 0.001);
      assertEquals(((Float) oAnalysis.mp_fPartBioDBHTanninsA.GetValue().elementAt(1)).
          floatValue(), 1.24, 0.001);
      
      assertEquals(((Float) oAnalysis.mp_fPartBioDBHTanninsB.GetValue().elementAt(0)).
          floatValue(), 0.25, 0.001);
      assertEquals(((Float) oAnalysis.mp_fPartBioDBHTanninsB.GetValue().elementAt(1)).
          floatValue(), 1.25, 0.001);
      
      assertEquals(((Float) oAnalysis.mp_fPartBioDBHPhenolicsA.GetValue().elementAt(0)).
          floatValue(), 0.26, 0.001);
      assertEquals(((Float) oAnalysis.mp_fPartBioDBHPhenolicsA.GetValue().elementAt(1)).
          floatValue(), 1.26, 0.001);
      
      assertEquals(((Float) oAnalysis.mp_fPartBioDBHPhenolicsB.GetValue().elementAt(0)).
          floatValue(), 0.27, 0.001);
      assertEquals(((Float) oAnalysis.mp_fPartBioDBHPhenolicsB.GetValue().elementAt(1)).
          floatValue(), 1.27, 0.001);
      
      assertEquals(((Float) oAnalysis.mp_fPartBioDBHSLAA.GetValue().elementAt(0)).
          floatValue(), 0.16, 0.001);
      assertEquals(((Float) oAnalysis.mp_fPartBioDBHSLAA.GetValue().elementAt(1)).
          floatValue(), 1.16, 0.001); 
      
      assertEquals(((Float) oAnalysis.mp_fPartBioDBHSLAB.GetValue().elementAt(0)).
          floatValue(), 0.17, 0.001);
      assertEquals(((Float) oAnalysis.mp_fPartBioDBHSLAB.GetValue().elementAt(1)).
          floatValue(), 1.17, 0.001);

      System.out.println("Read partitioned biomass parameters test succeeded.");
    }
    catch (ModelException oErr) {
      fail("Read partitioned biomass parameters test failed with message " + 
      		oErr.getMessage());
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException during read partitioned biomass parameters " +
      		"test.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }
  }

  /**
   * Tests to make sure the appropriate parameters are displayed for each
   * behavior.
   */
  public void testFormatDataForDisplay() {
    GUIManager oManager = null;
    String sFileName = null;
    try {
      oManager = new GUIManager(null);
      oManager.ClearCurrentData();
      sFileName = WriteNeutralFile();
      oManager.InputXMLParameterFile(sFileName);
      TreePopulation oPop = oManager.GetTreePopulation();
      AnalysisBehaviors oAnalysis = oManager.GetAnalysisBehaviors();
      String[] p_sExpected;
      Behavior oBeh;

      //Case:  no Analysis behaviors are enabled.
      assertEquals(null, oAnalysis.FormatDataForDisplay(oPop));

      //Case:  only tree volume is enabled.
      oManager.ClearCurrentData();
      sFileName = WriteNeutralFile();
      oManager.InputXMLParameterFile(sFileName);
      oPop = oManager.GetTreePopulation();
      oAnalysis = oManager.GetAnalysisBehaviors();
      oBeh = oAnalysis.GetBehaviorByKey("tree volume calculator");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      p_sExpected = new String[] {
          "Taper Equation Initial Multiplier (a0)",
          "Taper Equation DBH Exponent (a1)",
          "Taper Equation Height Exponent (a2)",
          "Taper Equation X Exponent 1 (b1)",
          "Taper Equation X Exponent 2 (b2)",
          "Taper Equation X Exponent 3 (b3)",
          "Taper Equation X Exponent 4 (b4)",
          "Taper Equation X Exponent 5 (b5)",
          "Taper Equation X Exponent 6 (b6)",
          "Diameter-Outside-Bark Constant (a1)",
          "Diameter-Outside-Bark First Degree Parameter (a2)",
          "Diameter-Outside-Bark Second Degree Parameter (a3)",
          "Height to Begin Calculating Trunk Volume, in cm",
          "Minimum Trunk Diameter for Volume Calculations, in cm",
          "Trunk Segment Length for Volume Calculations, in m"};
      TestTable(oAnalysis.FormatDataForDisplay(oPop), p_sExpected);

      //Case:  only tree dimension analysis is enabled.
      oManager.ClearCurrentData();
      sFileName = WriteNeutralFile();
      oManager.InputXMLParameterFile(sFileName);
      oPop = oManager.GetTreePopulation();
      oAnalysis = oManager.GetAnalysisBehaviors();
      oBeh = oAnalysis.GetBehaviorByKey("tree biomass calculator");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      p_sExpected = new String[] {
          "Dimension Analysis Equation ID",
          "Dimension Analysis Biomass Units",
          "Dimension Analysis: Use Correction Factor?",
          "Dimension Analysis Correction Factor",
          "Dimension Analysis Parameter (a)",
          "Dimension Analysis Parameter (b)",
          "Dimension Analysis Parameter (c)",
          "Dimension Analysis Parameter (d)",
          "Dimension Analysis Parameter (e)",
          "Dimension Analysis DBH Units"};
      TestTable(oAnalysis.FormatDataForDisplay(oPop), p_sExpected);

      //Case:  only tree dimension analysis is enabled.
      oManager.ClearCurrentData();
      sFileName = WriteNeutralFile();
      oManager.InputXMLParameterFile(sFileName);
      oPop = oManager.GetTreePopulation();
      oAnalysis = oManager.GetAnalysisBehaviors();
      oBeh = oAnalysis.GetBehaviorByKey("bole volume");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      p_sExpected = new String[] {
          "Bole Volume Parameter (b0)",
          "Bole Volume Parameter (b1)",
          "Bole Volume Parameter (b2)",
          "Bole Volume Parameter (b3)",
          "Bole Volume Parameter (b4)",
          "Bole Volume Parameter (b5)",
          "Bole Volume Form Class, 60% - 100%"};
      TestTable(oAnalysis.FormatDataForDisplay(oPop), p_sExpected);

      //Case:  only merchantable timber value is enabled.
      oManager.ClearCurrentData();
      sFileName = WriteNeutralFile();
      oManager.InputXMLParameterFile(sFileName);
      oPop = oManager.GetTreePopulation();
      oAnalysis = oManager.GetAnalysisBehaviors();
      oBeh = oAnalysis.GetBehaviorByKey("Merchantable Timber Value");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      p_sExpected = new String[] {
          "Merchantable Timber Value Form Class",
          "Merchantable Timber Value Price / 1000 Board Feet"};
      TestTable(oAnalysis.FormatDataForDisplay(oPop), p_sExpected);

      //Case:  only carbon value is enabled.
      oManager.ClearCurrentData();
      sFileName = WriteNeutralFile();
      oManager.InputXMLParameterFile(sFileName);
      oPop = oManager.GetTreePopulation();
      oAnalysis = oManager.GetAnalysisBehaviors();
      oBeh = oAnalysis.GetBehaviorByKey("Carbon Value");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      p_sExpected = new String[] {
          "Carbon Value - Carbon Amount of Biomass (0-100%)",
          "Carbon Value - Price Per Metric Ton Carbon"};
      TestTable(oAnalysis.FormatDataForDisplay(oPop), p_sExpected);

      //Case:  only partitioned DBH biomass is enabled.
      oManager.ClearCurrentData();
      sFileName = WriteNeutralFile();
      oManager.InputXMLParameterFile(sFileName);
      oPop = oManager.GetTreePopulation();
      oAnalysis = oManager.GetAnalysisBehaviors();
      oBeh = oAnalysis.GetBehaviorByKey("Partitioned DBH Biomass");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      p_sExpected = new String[] {
          "Partitioned DBH Biomass - Leaf Slope (a)",
          "Partitioned DBH Biomass - Leaf Intercept (b)",
          "Partitioned DBH Biomass - Branch Slope (a)",
          "Partitioned DBH Biomass - Branch Intercept (b)",
          "Partitioned DBH Biomass - Bole Slope (a)",
          "Partitioned DBH Biomass - Bole Intercept (b)",
          "Partitioned DBH Biomass - N (a)",
          "Partitioned DBH Biomass - N (b)",
          "Partitioned DBH Biomass - P (a)",
          "Partitioned DBH Biomass - P (b)",
          "Partitioned DBH Biomass - Lignin (a)",
          "Partitioned DBH Biomass - Lignin (b)",
          "Partitioned DBH Biomass - Fiber (a)",
          "Partitioned DBH Biomass - Fiber (b)",
          "Partitioned DBH Biomass - Cellulose (a)",
          "Partitioned DBH Biomass - Cellulose (b)",
          "Partitioned DBH Biomass - Tannins (a)",
          "Partitioned DBH Biomass - Tannins (b)",
          "Partitioned DBH Biomass - Phenolics (a)",
          "Partitioned DBH Biomass - Phenolics (b)",
          "Partitioned DBH Biomass - SLA (a)",
          "Partitioned DBH Biomass - SLA (b)"    
      };
      TestTable(oAnalysis.FormatDataForDisplay(oPop), p_sExpected);
      
      //Case:  only storm-killed partitioned DBH biomass is enabled.
      oManager.ClearCurrentData();
      sFileName = WriteNeutralFile();
      oManager.InputXMLParameterFile(sFileName);
      oPop = oManager.GetTreePopulation();
      oAnalysis = oManager.GetAnalysisBehaviors();
      oBeh = oAnalysis.GetBehaviorByKey("Storm Killed Partitioned DBH Biomass");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, TreePopulation.SNAG, oPop));
      p_sExpected = new String[] {
          "Partitioned DBH Biomass - Leaf Slope (a)",
          "Partitioned DBH Biomass - Leaf Intercept (b)",
          "Partitioned DBH Biomass - Branch Slope (a)",
          "Partitioned DBH Biomass - Branch Intercept (b)",
          "Partitioned DBH Biomass - Bole Slope (a)",
          "Partitioned DBH Biomass - Bole Intercept (b)"};
      TestTable(oAnalysis.FormatDataForDisplay(oPop), p_sExpected);

      //Case:  only partitioned height (palm) biomass is enabled.
      oManager.ClearCurrentData();
      sFileName = WriteNeutralFile();
      oManager.InputXMLParameterFile(sFileName);
      oPop = oManager.GetTreePopulation();
      oAnalysis = oManager.GetAnalysisBehaviors();
      oBeh = oAnalysis.GetBehaviorByKey("Partitioned Height Biomass");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      p_sExpected = new String[] {
          "Partitioned Palm Biomass - Leaf Slope (a)",
          "Partitioned Palm Biomass - Leaf Intercept (b)",
          "Partitioned Palm Biomass - Bole Slope (a)",
          "Partitioned Palm Biomass - Bole Intercept (b)"};
      TestTable(oAnalysis.FormatDataForDisplay(oPop), p_sExpected);
      
      //Case: only storm-killed partitioned height (palm) biomass is enabled.
      oManager.ClearCurrentData();
      sFileName = WriteNeutralFile();
      oManager.InputXMLParameterFile(sFileName);
      oPop = oManager.GetTreePopulation();
      oAnalysis = oManager.GetAnalysisBehaviors();
      oBeh = oAnalysis.GetBehaviorByKey("Storm Killed Partitioned Height Biomass");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, TreePopulation.SNAG, oPop));
      p_sExpected = new String[] {
          "Partitioned Palm Biomass - Leaf Slope (a)",
          "Partitioned Palm Biomass - Leaf Intercept (b)",
          "Partitioned Palm Biomass - Bole Slope (a)",
          "Partitioned Palm Biomass - Bole Intercept (b)"};
      TestTable(oAnalysis.FormatDataForDisplay(oPop), p_sExpected);
      
      //Case: only Ripley's K calculator is enabled.
      oManager.ClearCurrentData();
      sFileName = WriteNeutralFile();
      oManager.InputXMLParameterFile(sFileName);
      oPop = oManager.GetTreePopulation();
      oAnalysis = oManager.GetAnalysisBehaviors();
      oBeh = oAnalysis.GetBehaviorByKey("Ripley's K");
      oBeh.m_bIsEnabled = true;
      p_sExpected = new String[] {
          "Ripley's K - Maximum Distance (m)",
          "Ripley's K - Distance Increment (m)"};
      TestTable(oAnalysis.FormatDataForDisplay(oPop), p_sExpected);
      
      //Case: only relative neighborhood density (Condit's Omega) calculator 
      //is enabled.
      oManager.ClearCurrentData();
      sFileName = WriteNeutralFile();
      oManager.InputXMLParameterFile(sFileName);
      oPop = oManager.GetTreePopulation();
      oAnalysis = oManager.GetAnalysisBehaviors();
      oBeh = oAnalysis.GetBehaviorByKey("Condit's Omega");
      oBeh.m_bIsEnabled = true;
      p_sExpected = new String[] {
          "Relative Neighborhood Density - Maximum Distance (m)",
          "Relative Neighborhood Density - Distance Increment (m)"};
      TestTable(oAnalysis.FormatDataForDisplay(oPop), p_sExpected);

      //Case:  all are enabled.
      oManager.ClearCurrentData();
      sFileName = WriteNeutralFile();
      oManager.InputXMLParameterFile(sFileName);
      oPop = oManager.GetTreePopulation();
      oAnalysis = oManager.GetAnalysisBehaviors();
      int i;
      for (i = 0; i < oAnalysis.mp_oChildBehaviors.length; i++) {
        oAnalysis.mp_oChildBehaviors[i].m_bIsEnabled = true;
        if (oAnalysis.mp_oChildBehaviors[i].GetKey().indexOf("Storm Killed") == -1)
          oAnalysis.mp_oChildBehaviors[i].AddSpeciesTypeCombo(new
              SpeciesTypeCombo(0, 3, oPop));
        else
        	oAnalysis.mp_oChildBehaviors[i].AddSpeciesTypeCombo(new
              SpeciesTypeCombo(0, TreePopulation.SNAG, oPop));
      }
      p_sExpected = new String[] {
          "Taper Equation Initial Multiplier (a0)",
          "Taper Equation DBH Exponent (a1)",
          "Taper Equation Height Exponent (a2)",
          "Taper Equation X Exponent 1 (b1)",
          "Taper Equation X Exponent 2 (b2)",
          "Taper Equation X Exponent 3 (b3)",
          "Taper Equation X Exponent 4 (b4)",
          "Taper Equation X Exponent 5 (b5)",
          "Taper Equation X Exponent 6 (b6)",
          "Diameter-Outside-Bark Constant (a1)",
          "Diameter-Outside-Bark First Degree Parameter (a2)",
          "Diameter-Outside-Bark Second Degree Parameter (a3)",
          "Height to Begin Calculating Trunk Volume, in cm",
          "Minimum Trunk Diameter for Volume Calculations, in cm",
          "Trunk Segment Length for Volume Calculations, in m",
          "Dimension Analysis Equation ID",
          "Dimension Analysis Biomass Units",
          "Dimension Analysis: Use Correction Factor?",
          "Dimension Analysis Correction Factor",
          "Dimension Analysis Parameter (a)",
          "Dimension Analysis Parameter (b)",
          "Dimension Analysis Parameter (c)",
          "Dimension Analysis Parameter (d)",
          "Dimension Analysis Parameter (e)",
          "Dimension Analysis DBH Units",
          "Bole Volume Parameter (b0)",
          "Bole Volume Parameter (b1)",
          "Bole Volume Parameter (b2)",
          "Bole Volume Parameter (b3)",
          "Bole Volume Parameter (b4)",
          "Bole Volume Parameter (b5)",
          "Bole Volume Form Class, 60% - 100%",
          "Merchantable Timber Value Form Class",
          "Merchantable Timber Value Price / 1000 Board Feet",
          "Carbon Value - Carbon Amount of Biomass (0-100%)",
          "Carbon Value - Price Per Metric Ton Carbon",
          "Partitioned DBH Biomass - Leaf Slope (a)",
          "Partitioned DBH Biomass - Leaf Intercept (b)",
          "Partitioned DBH Biomass - Branch Slope (a)",
          "Partitioned DBH Biomass - Branch Intercept (b)",
          "Partitioned DBH Biomass - Bole Slope (a)",
          "Partitioned DBH Biomass - Bole Intercept (b)",
          "Partitioned DBH Biomass - N (a)",
          "Partitioned DBH Biomass - N (b)",
          "Partitioned DBH Biomass - P (a)",
          "Partitioned DBH Biomass - P (b)",
          "Partitioned DBH Biomass - Lignin (a)",
          "Partitioned DBH Biomass - Lignin (b)",
          "Partitioned DBH Biomass - Fiber (a)",
          "Partitioned DBH Biomass - Fiber (b)",
          "Partitioned DBH Biomass - Cellulose (a)",
          "Partitioned DBH Biomass - Cellulose (b)",
          "Partitioned DBH Biomass - Tannins (a)",
          "Partitioned DBH Biomass - Tannins (b)",
          "Partitioned DBH Biomass - Phenolics (a)",
          "Partitioned DBH Biomass - Phenolics (b)",
          "Partitioned DBH Biomass - SLA (a)",
          "Partitioned DBH Biomass - SLA (b)",
          "Partitioned Palm Biomass - Leaf Slope (a)",
          "Partitioned Palm Biomass - Leaf Intercept (b)",
          "Partitioned Palm Biomass - Bole Slope (a)",
          "Partitioned Palm Biomass - Bole Intercept (b)",
          "Ripley's K - Maximum Distance (m)",
          "Ripley's K - Distance Increment (m)",
          "Relative Neighborhood Density - Maximum Distance (m)",
          "Relative Neighborhood Density - Distance Increment (m)"};
      
      TestTable(oAnalysis.FormatDataForDisplay(oPop), p_sExpected);

      System.out.println("FormatDataForDisplay succeeded for Analysis.");
    }
    catch (ModelException oErr) {
      fail("Caught error with message " + oErr.getMessage());
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
  }

  /**
   * Tests ValidateData() for data analysis behaviors.
   * @throws ModelException if the validation happens incorrectly.
   */
  public void testValidateData() throws ModelException {
    GUIManager oManager = null;
    String sFileName = null;
    try {

      oManager = new GUIManager(null);

      //Valid volume file
      oManager.ClearCurrentData();
      sFileName = WriteVolumeXMLFile1();
      oManager.InputXMLParameterFile(sFileName);
      oManager.GetAnalysisBehaviors().ValidateData(oManager.GetTreePopulation());
      new File(sFileName).delete();

      //Valid dimension analysis file
      oManager.ClearCurrentData();
      sFileName = WriteDimensionAnalysisXMLFile1();
      oManager.InputXMLParameterFile(sFileName);
      oManager.GetAnalysisBehaviors().ValidateData(oManager.GetTreePopulation());
      new File(sFileName).delete();

      //Valid bole volume file
      oManager.ClearCurrentData();
      sFileName = WriteBoleVolumeXMLFile1();
      oManager.InputXMLParameterFile(sFileName);
      oManager.GetAnalysisBehaviors().ValidateData(oManager.GetTreePopulation());
      new File(sFileName).delete();
      
      //Valid Ripley's K file
      oManager.ClearCurrentData();
      sFileName = WriteRipleysKXMLFile1();
      oManager.InputXMLParameterFile(sFileName);
      oManager.GetAnalysisBehaviors().ValidateData(oManager.GetTreePopulation());
      new File(sFileName).delete();

    }
    catch (ModelException oErr) {
      fail("Analysis behavior validation failed with message " + oErr.getMessage());
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }

    //Exception processing - volume error file 1
    try {
      oManager.ClearCurrentData();
      sFileName = WriteVolumeXMLErrorFile1();
      oManager.InputXMLParameterFile(sFileName);
      oManager.GetAnalysisBehaviors().ValidateData(oManager.GetTreePopulation());
      fail("Didn't catch error in volume error file 1.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //Exception processing - volume error file 2
    try {
      oManager.ClearCurrentData();
      sFileName = WriteVolumeXMLErrorFile2();
      oManager.InputXMLParameterFile(sFileName);
      oManager.GetAnalysisBehaviors().ValidateData(oManager.GetTreePopulation());
      fail("Didn't catch error in volume error file 2.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //Exception processing - volume error file 3
    try {
      oManager.ClearCurrentData();
      sFileName = WriteVolumeXMLErrorFile3();
      oManager.InputXMLParameterFile(sFileName);
      oManager.GetAnalysisBehaviors().ValidateData(oManager.GetTreePopulation());
      fail("Didn't catch error in volume error file 3.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //Exception processing - bole volume error file 1
    try {
      oManager.ClearCurrentData();
      sFileName = WriteBoleVolumeErrorXMLFile1();
      oManager.InputXMLParameterFile(sFileName);
      oManager.GetAnalysisBehaviors().ValidateData(oManager.GetTreePopulation());
      fail("Didn't catch error in volume error file 1.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //Exception processing - bole volume error file 2
    try {
      oManager.ClearCurrentData();
      sFileName = WriteBoleVolumeErrorXMLFile2();
      oManager.InputXMLParameterFile(sFileName);
      oManager.GetAnalysisBehaviors().ValidateData(oManager.GetTreePopulation());
      fail("Didn't catch error in volume error file 2.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //Normal processing of carbon value
    try {
      oManager.ClearCurrentData();
      sFileName = WriteDimensionAnalysisXMLFile1();
      oManager.InputXMLParameterFile(sFileName);
      AnalysisBehaviors oAnalysis = oManager.GetAnalysisBehaviors();
      TreePopulation oPop = oManager.GetTreePopulation();
      Behavior oBeh = oAnalysis.GetBehaviorByKey("Carbon Value");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      oAnalysis.ValidateData(oPop);
      new File(sFileName).delete();
    } catch (ModelException oErr) {
      fail("Analysis behavior validation failed with message " + oErr.getMessage());
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }

    //Exception - percent of biomass that is carbon is not between 0 and 100
    try {
      sFileName = WriteDimensionAnalysisXMLFile1();
      oManager.InputXMLParameterFile(sFileName);
      AnalysisBehaviors oAnalysis = oManager.GetAnalysisBehaviors();
      TreePopulation oPop = oManager.GetTreePopulation();
      Behavior oBeh = oAnalysis.GetBehaviorByKey("Carbon Value");
      oBeh.m_bIsEnabled = true;
      oAnalysis.mp_fCarbonValuePercentBiomassCarbon.GetValue().add(new Float(-3));
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      oAnalysis.ValidateData(oPop);
      fail("Didn't catch carbon value carbon percent of biomass not between 0 and 100.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //Exception - dimension analysis not enabled for all trees that get
    //carbon value
    try {
      sFileName = WriteDimensionAnalysisXMLFile1();
      oManager.InputXMLParameterFile(sFileName);
      AnalysisBehaviors oAnalysis = oManager.GetAnalysisBehaviors();
      TreePopulation oPop = oManager.GetTreePopulation();
      Behavior oBeh = oAnalysis.GetBehaviorByKey("tree biomass calculator");
      oBeh.ClearSpeciesTypeCombos();
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(1, 3, oPop));
      oBeh = oAnalysis.GetBehaviorByKey("Carbon Value");
      oBeh.m_bIsEnabled = true;
      oAnalysis.mp_fCarbonValuePercentBiomassCarbon.GetValue().add(new Float(-3));
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 2, oPop));
      oAnalysis.ValidateData(oPop);
      fail("Didn't catch dimension analysis not applied to all the same trees as carbon value.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }
    
    //Exception - Ripley's K max distance not positive number
    try {
      sFileName = WriteNeutralFile();
      oManager.InputXMLParameterFile(sFileName);
      AnalysisBehaviors oAnalysis = oManager.GetAnalysisBehaviors();
      TreePopulation oPop = oManager.GetTreePopulation();
      Behavior oBeh = oAnalysis.GetBehaviorByKey("Ripley's K");
      oBeh.m_bIsEnabled = true;
      oAnalysis.m_fRipleysKMaxDistance.SetValue(-2);
      oAnalysis.ValidateData(oPop);
      fail("Didn't catch non-positive Ripley's K maximum distance.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }
    
    //Exception - Ripley's K distance increment not positive number
    try {
      sFileName = WriteNeutralFile();
      oManager.InputXMLParameterFile(sFileName);
      AnalysisBehaviors oAnalysis = oManager.GetAnalysisBehaviors();
      TreePopulation oPop = oManager.GetTreePopulation();
      Behavior oBeh = oAnalysis.GetBehaviorByKey("Ripley's K");
      oBeh.m_bIsEnabled = true;
      oAnalysis.m_fRipleysKDistanceInc.SetValue(-2);
      oAnalysis.ValidateData(oPop);
      fail("Didn't catch non-positive Ripley's K distance increment.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }
    
    //Exception - Ripley's K max distance not greater than distance increment
    try {
      sFileName = WriteNeutralFile();
      oManager.InputXMLParameterFile(sFileName);
      AnalysisBehaviors oAnalysis = oManager.GetAnalysisBehaviors();
      TreePopulation oPop = oManager.GetTreePopulation();
      Behavior oBeh = oAnalysis.GetBehaviorByKey("Ripley's K");
      oBeh.m_bIsEnabled = true;
      oAnalysis.m_fRipleysKMaxDistance.SetValue(5);
      oAnalysis.m_fRipleysKDistanceInc.SetValue(10);
      oAnalysis.ValidateData(oPop);
      fail("Didn't catch Ripley's K maximum distance less than distance increment.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    System.out.println("Analysis behaviors ValidateData test was successful.");

  }

  /**
   * Writes a valid file for testing parameter file reading of volume
   * calculator behavior parameters.
   * @return String Filename written.
   * @throws IOException if there is a problem writing the file.
   */
  protected String WriteVolumeXMLFile1() throws IOException {
    String sFileName = "\\loratestxml1.xml";
    FileWriter oOut = new FileWriter(sFileName);

    WriteVolumeCommonStuff(oOut);

    oOut.write("<vo_stumpHeight>13</vo_stumpHeight>");
    oOut.write("<vo_minUsableDiam>3</vo_minUsableDiam>");
    oOut.write("<vo_segmentLength>2</vo_segmentLength>");
    oOut.write("</volume>");
    oOut.write("</paramFile>");

    oOut.close();
    return sFileName;
  }
  
  /**
   * Writes a valid file for testing parameter file reading of partitioned
   * biomass behavior parameters.
   * @return String Filename written.
   * @throws IOException if there is a problem writing the file.
   */
  protected String WritePartitionedBiomassXMLFile1() throws IOException {
    String sFileName = "\\loratestxml1.xml";
    FileWriter oOut = new FileWriter(sFileName);

    WritePlotAndTrees(oOut);

    oOut.write("<behaviorList>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>Partitioned DBH Biomass</behaviorName>");
    oOut.write("<version>2</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Adult\"/>");
    oOut.write("<applyTo species=\"Species_2\" type=\"Adult\"/>");
    oOut.write("</behavior>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>Partitioned Height Biomass</behaviorName>");
    oOut.write("<version>2</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Adult\"/>");
    oOut.write("<applyTo species=\"Species_2\" type=\"Adult\"/>");
    oOut.write("</behavior>");
    oOut.write("</behaviorList>");
    oOut.write("<analysis>");
    oOut.write("<an_partBioDbhLeafA>");
    oOut.write("<an_pbdlaVal species=\"Species_1\">1.523</an_pbdlaVal>");
    oOut.write("<an_pbdlaVal species=\"Species_2\">1.792</an_pbdlaVal>");
    oOut.write("</an_partBioDbhLeafA>");
    oOut.write("<an_partBioDbhLeafB>");
    oOut.write("<an_pbdlbVal species=\"Species_1\">-3.49</an_pbdlbVal>");
    oOut.write("<an_pbdlbVal species=\"Species_2\">3.758</an_pbdlbVal>");
    oOut.write("</an_partBioDbhLeafB>");
    oOut.write("<an_partBioDbhBranchA>");
    oOut.write("<an_pbdbraVal species=\"Species_1\">2.062</an_pbdbraVal>");
    oOut.write("<an_pbdbraVal species=\"Species_2\">1.982</an_pbdbraVal>");
    oOut.write("</an_partBioDbhBranchA>");
    oOut.write("<an_partBioDbhBranchB>");
    oOut.write("<an_pbdbrbVal species=\"Species_1\">-5.963</an_pbdbrbVal>");
    oOut.write("<an_pbdbrbVal species=\"Species_2\">3.69</an_pbdbrbVal>");
    oOut.write("</an_partBioDbhBranchB>");
    oOut.write("<an_partBioDbhBoleA>");
    oOut.write("<an_pbdboaVal species=\"Species_1\">2.543</an_pbdboaVal>");
    oOut.write("<an_pbdboaVal species=\"Species_2\">2.559</an_pbdboaVal>");
    oOut.write("</an_partBioDbhBoleA>");
    oOut.write("<an_partBioDbhBoleB>");
    oOut.write("<an_pbdbobVal species=\"Species_1\">-3.139</an_pbdbobVal>");
    oOut.write("<an_pbdbobVal species=\"Species_2\">2.832</an_pbdbobVal>");
    oOut.write("</an_partBioDbhBoleB>");
    oOut.write("<an_partBioHeightLeafA>");
    oOut.write("<an_pbhlaVal species=\"Species_1\">1.7</an_pbhlaVal>");
    oOut.write("<an_pbhlaVal species=\"Species_2\">1.3</an_pbhlaVal>");
    oOut.write("</an_partBioHeightLeafA>");
    oOut.write("<an_partBioHeightLeafB>");
    oOut.write("<an_pbhlbVal species=\"Species_1\">6.8</an_pbhlbVal>");
    oOut.write("<an_pbhlbVal species=\"Species_2\">-2.3</an_pbhlbVal>");
    oOut.write("</an_partBioHeightLeafB>");
    oOut.write("<an_partBioHeightBoleA>");
    oOut.write("<an_pbhboaVal species=\"Species_1\">5.9</an_pbhboaVal>");
    oOut.write("<an_pbhboaVal species=\"Species_2\">6.9</an_pbhboaVal>");
    oOut.write("</an_partBioHeightBoleA>");
    oOut.write("<an_partBioHeightBoleB>");
    oOut.write("<an_pbhbobVal species=\"Species_1\">-1.6</an_pbhbobVal>");
    oOut.write("<an_pbhbobVal species=\"Species_2\">-1.1</an_pbhbobVal>");
    oOut.write("</an_partBioHeightBoleB>");
    oOut.write("<an_partBioDbhNA>");
    oOut.write("<an_pbdnaVal species=\"Species_1\">0.12</an_pbdnaVal>");
    oOut.write("<an_pbdnaVal species=\"Species_2\">1.12</an_pbdnaVal>");
    oOut.write("</an_partBioDbhNA>");
    oOut.write("<an_partBioDbhNB>");
    oOut.write("<an_pbdnbVal species=\"Species_1\">0.13</an_pbdnbVal>");
    oOut.write("<an_pbdnbVal species=\"Species_2\">1.13</an_pbdnbVal>");
    oOut.write("</an_partBioDbhNB>");
    oOut.write("<an_partBioDbhPA>");
    oOut.write("<an_pbdpaVal species=\"Species_1\">0.14</an_pbdpaVal>");
    oOut.write("<an_pbdpaVal species=\"Species_2\">1.14</an_pbdpaVal>");
    oOut.write("</an_partBioDbhPA>");
    oOut.write("<an_partBioDbhPB>");
    oOut.write("<an_pbdpbVal species=\"Species_1\">0.15</an_pbdpbVal>");
    oOut.write("<an_pbdpbVal species=\"Species_2\">1.15</an_pbdpbVal>");
    oOut.write("</an_partBioDbhPB>");
    oOut.write("<an_partBioDbhSLAA>");
    oOut.write("<an_pbdsaVal species=\"Species_1\">0.16</an_pbdsaVal>");
    oOut.write("<an_pbdsaVal species=\"Species_2\">1.16</an_pbdsaVal>");
    oOut.write("</an_partBioDbhSLAA>");
    oOut.write("<an_partBioDbhSLAB>");
    oOut.write("<an_pbdsbVal species=\"Species_1\">0.17</an_pbdsbVal>");
    oOut.write("<an_pbdsbVal species=\"Species_2\">1.17</an_pbdsbVal>");
    oOut.write("</an_partBioDbhSLAB>");
    oOut.write("<an_partBioDbhLigninA>");
    oOut.write("<an_pbdlaVal species=\"Species_1\">0.18</an_pbdlaVal>");
    oOut.write("<an_pbdlaVal species=\"Species_2\">1.18</an_pbdlaVal>");
    oOut.write("</an_partBioDbhLigninA>");
    oOut.write("<an_partBioDbhLigninB>");
    oOut.write("<an_pbdlbVal species=\"Species_1\">0.19</an_pbdlbVal>");
    oOut.write("<an_pbdlbVal species=\"Species_2\">1.19</an_pbdlbVal>");
    oOut.write("</an_partBioDbhLigninB>");
    oOut.write("<an_partBioDbhTanninsA>");
    oOut.write("<an_pbdtaVal species=\"Species_1\">0.24</an_pbdtaVal>");
    oOut.write("<an_pbdtaVal species=\"Species_2\">1.24</an_pbdtaVal>");
    oOut.write("</an_partBioDbhTanninsA>");
    oOut.write("<an_partBioDbhTanninsB>");
    oOut.write("<an_pbdtbVal species=\"Species_1\">0.25</an_pbdtbVal>");
    oOut.write("<an_pbdtbVal species=\"Species_2\">1.25</an_pbdtbVal>");
    oOut.write("</an_partBioDbhTanninsB>");
    oOut.write("<an_partBioDbhPhenolicsA>");
    oOut.write("<an_pbdphaVal species=\"Species_1\">0.26</an_pbdphaVal>");
    oOut.write("<an_pbdphaVal species=\"Species_2\">1.26</an_pbdphaVal>");
    oOut.write("</an_partBioDbhPhenolicsA>");
    oOut.write("<an_partBioDbhPhenolicsB>");
    oOut.write("<an_pbdphbVal species=\"Species_1\">0.27</an_pbdphbVal>");
    oOut.write("<an_pbdphbVal species=\"Species_2\">1.27</an_pbdphbVal>");
    oOut.write("</an_partBioDbhPhenolicsB>");
    oOut.write("<an_partBioDbhFiberA>");
    oOut.write("<an_pbdfaVal species=\"Species_1\">0.20</an_pbdfaVal>");
    oOut.write("<an_pbdfaVal species=\"Species_2\">1.20</an_pbdfaVal>");
    oOut.write("</an_partBioDbhFiberA>");
    oOut.write("<an_partBioDbhFiberB>");
    oOut.write("<an_pbdfbVal species=\"Species_1\">0.21</an_pbdfbVal>");
    oOut.write("<an_pbdfbVal species=\"Species_2\">1.21</an_pbdfbVal>");
    oOut.write("</an_partBioDbhFiberB>");
    oOut.write("<an_partBioDbhCelluloseA>");
    oOut.write("<an_pbdcaVal species=\"Species_1\">0.22</an_pbdcaVal>");
    oOut.write("<an_pbdcaVal species=\"Species_2\">1.22</an_pbdcaVal>");
    oOut.write("</an_partBioDbhCelluloseA>");
    oOut.write("<an_partBioDbhCelluloseB>");
    oOut.write("<an_pbdcbVal species=\"Species_1\">0.23</an_pbdcbVal>");
    oOut.write("<an_pbdcbVal species=\"Species_2\">1.23</an_pbdcbVal>");
    oOut.write("</an_partBioDbhCelluloseB>");
    oOut.write("</analysis>");
    oOut.write("</paramFile>");;

    oOut.close();
    return sFileName;
  }
  
  /**
   * Writes a valid file for testing parameter file reading of Ripley's K
   * calculator behavior parameters.
   * @return String Filename written.
   * @throws IOException if there is a problem writing the file.
   */
  protected String WriteRipleysKXMLFile1() throws IOException {
  	String sFileName = "\\loratestxml1.xml";
    FileWriter oOut = new FileWriter(sFileName);

    WritePlotAndTrees(oOut);
    
    oOut.write("<behaviorList>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>Ripley's K</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("</behavior>");
    oOut.write("</behaviorList>");
    oOut.write("<analysis>");
    oOut.write("<an_RipleysKMaxDistance>15</an_RipleysKMaxDistance>");
    oOut.write("<an_RipleysKDistanceInc>5</an_RipleysKDistanceInc>");
    oOut.write("</analysis>");
    oOut.write("</paramFile>");

    oOut.close();
    return sFileName;
  }
  
  /**
   * Writes a valid file for testing parameter file reading of Relative 
   * Neighborhood Density calculator (Condit's Omega) behavior parameters.
   * @return String Filename written.
   * @throws IOException if there is a problem writing the file.
   */
  protected String WriteConditsOmegaXMLFile1() throws IOException {
  	String sFileName = "\\loratestxml1.xml";
    FileWriter oOut = new FileWriter(sFileName);

    WritePlotAndTrees(oOut);
    oOut.write("<behaviorList>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>Condit's Omega</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("</behavior>");
    oOut.write("</behaviorList>");
    oOut.write("<analysis>");
    oOut.write("<an_ConditsOmegaMaxDistance>15</an_ConditsOmegaMaxDistance>");
    oOut.write("<an_ConditsOmegaDistanceInc>5</an_ConditsOmegaDistanceInc>");
    oOut.write("</analysis>");
    oOut.write("</paramFile>");

    oOut.close();
    return sFileName;
  }

  /**
   * Writes a file with multiple analysis behaviors represented.
   * @return String Filename written.
   * @throws IOException if there is a problem writing the file.
   */
  protected String WriteComboXMLFile1() throws IOException {
    String sFileName = "\\loratestxml1.xml";
    FileWriter oOut = new FileWriter(sFileName);

    WritePlotAndTrees(oOut);
    
    oOut.write("<behaviorList>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>tree volume calculator</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("<applyTo species=\"Species_2\" type=\"Adult\"/>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Adult\"/>");
    oOut.write("</behavior>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>tree biomass calculator</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Adult\"/>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Sapling\"/>");
    oOut.write("<applyTo species=\"Species_2\" type=\"Adult\"/>");
    oOut.write("<applyTo species=\"Species_2\" type=\"Sapling\"/>");
    oOut.write("</behavior>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>tree bole volume calculator</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("<applyTo species=\"Species_2\" type=\"Adult\"/>");
    oOut.write("<applyTo species=\"Species_2\" type=\"Sapling\"/>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Adult\"/>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Sapling\"/>");
    oOut.write("</behavior>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>Merch Value Calculator</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Adult\"/>");
    oOut.write("<applyTo species=\"Species_2\" type=\"Adult\"/>");
    oOut.write("</behavior>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>Carbon Value Calculator</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Adult\"/>");
    oOut.write("<applyTo species=\"Species_2\" type=\"Adult\"/>");
    oOut.write("</behavior>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>Partitioned DBH Biomass</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Sapling\"/>");
    oOut.write("<applyTo species=\"Species_2\" type=\"Adult\"/>");
    oOut.write("</behavior>");
    oOut.write("</behaviorList>");
    oOut.write("<analysis>");
    oOut.write("<volume>");
    oOut.write("<vo_taperA>");
    oOut.write("<vo_taVal species=\"Species_1\">0.843142</vo_taVal>");
    oOut.write("<vo_taVal species=\"Species_2\">0.8</vo_taVal>");
    oOut.write("</vo_taperA>");
    oOut.write("<vo_taperB>");
    oOut.write("<vo_tbVal species=\"Species_1\">1.00813</vo_tbVal>");
    oOut.write("<vo_tbVal species=\"Species_2\">0.9</vo_tbVal>");
    oOut.write("</vo_taperB>");
    oOut.write("<vo_taperC>");
    oOut.write("<vo_tcVal species=\"Species_1\">0.031402</vo_tcVal>");
    oOut.write("<vo_tcVal species=\"Species_2\">0.032</vo_tcVal>");
    oOut.write("</vo_taperC>");
    oOut.write("<vo_taperD>");
    oOut.write("<vo_tdVal species=\"Species_1\">0.319773</vo_tdVal>");
    oOut.write("<vo_tdVal species=\"Species_2\">0.321</vo_tdVal>");
    oOut.write("</vo_taperD>");
    oOut.write("<vo_taperF>");
    oOut.write("<vo_tfVal species=\"Species_1\">-0.532732</vo_tfVal>");
    oOut.write("<vo_tfVal species=\"Species_2\">-0.51</vo_tfVal>");
    oOut.write("</vo_taperF>");
    oOut.write("<vo_taperG>");
    oOut.write("<vo_tgVal species=\"Species_1\">0.585361</vo_tgVal>");
    oOut.write("<vo_tgVal species=\"Species_2\">0.59</vo_tgVal>");
    oOut.write("</vo_taperG>");
    oOut.write("<vo_taperI>");
    oOut.write("<vo_tiVal species=\"Species_1\">1.0315</vo_tiVal>");
    oOut.write("<vo_tiVal species=\"Species_2\">1.04</vo_tiVal>");
    oOut.write("</vo_taperI>");
    oOut.write("<vo_taperJ>");
    oOut.write("<vo_tjVal species=\"Species_1\">0.018622</vo_tjVal>");
    oOut.write("<vo_tjVal species=\"Species_2\">0.02</vo_tjVal>");
    oOut.write("</vo_taperJ>");
    oOut.write("<vo_taperK>");
    oOut.write("<vo_tkVal species=\"Species_1\">-0.225146</vo_tkVal>");
    oOut.write("<vo_tkVal species=\"Species_2\">-0.23</vo_tkVal>");
    oOut.write("</vo_taperK>");
    oOut.write("<vo_barkA>");
    oOut.write("<vo_baVal species=\"Species_1\">0.2</vo_baVal>");
    oOut.write("<vo_baVal species=\"Species_2\">0.01</vo_baVal>");
    oOut.write("</vo_barkA>");
    oOut.write("<vo_barkB>");
    oOut.write("<vo_bbVal species=\"Species_1\">0.8</vo_bbVal>");
    oOut.write("<vo_bbVal species=\"Species_2\">1</vo_bbVal>");
    oOut.write("</vo_barkB>");
    oOut.write("<vo_barkC>");
    oOut.write("<vo_bcVal species=\"Species_1\">0.01</vo_bcVal>");
    oOut.write("<vo_bcVal species=\"Species_2\">0.001</vo_bcVal>");
    oOut.write("</vo_barkC>");
    oOut.write("<vo_stumpHeight>13</vo_stumpHeight>");
    oOut.write("<vo_minUsableDiam>3</vo_minUsableDiam>");
    oOut.write("<vo_segmentLength>2</vo_segmentLength>");
    oOut.write("</volume>");
    oOut.write("<bi_a>");
    oOut.write("<bi_aVal species=\"Species_1\">-0.99</bi_aVal>");
    oOut.write("<bi_aVal species=\"Species_2\">-1.01</bi_aVal>");
    oOut.write("</bi_a>");
    oOut.write("<bi_b>");
    oOut.write("<bi_bVal species=\"Species_1\">2.2</bi_bVal>");
    oOut.write("<bi_bVal species=\"Species_2\">2.41</bi_bVal>");
    oOut.write("</bi_b>");
    oOut.write("<bi_c>");
    oOut.write("<bi_cVal species=\"Species_1\">0.9</bi_cVal>");
    oOut.write("<bi_cVal species=\"Species_2\">1</bi_cVal>");
    oOut.write("</bi_c>");
    oOut.write("<bi_d>");
    oOut.write("<bi_dVal species=\"Species_1\">1.1</bi_dVal>");
    oOut.write("<bi_dVal species=\"Species_2\">1.2</bi_dVal>");
    oOut.write("</bi_d>");
    oOut.write("<bi_e>");
    oOut.write("<bi_eVal species=\"Species_1\">0.1</bi_eVal>");
    oOut.write("<bi_eVal species=\"Species_2\">0.2</bi_eVal>");
    oOut.write("</bi_e>");
    oOut.write("<bi_eqID>");
    oOut.write("<bi_eiVal species=\"Species_1\">4</bi_eiVal>");
    oOut.write("<bi_eiVal species=\"Species_2\">5</bi_eiVal>");
    oOut.write("</bi_eqID>");
    oOut.write("<bi_dbhUnits>");
    oOut.write("<bi_duVal species=\"Species_1\">0</bi_duVal>");
    oOut.write("<bi_duVal species=\"Species_2\">1</bi_duVal>");
    oOut.write("</bi_dbhUnits>");
    oOut.write("<bi_biomassUnits>");
    oOut.write("<bi_buVal species=\"Species_1\">2</bi_buVal>");
    oOut.write("<bi_buVal species=\"Species_2\">1</bi_buVal>");
    oOut.write("</bi_biomassUnits>");
    oOut.write("<bi_useCorrectionFactor>");
    oOut.write("<bi_ucfVal species=\"Species_1\">0</bi_ucfVal>");
    oOut.write("<bi_ucfVal species=\"Species_2\">1</bi_ucfVal>");
    oOut.write("</bi_useCorrectionFactor>");
    oOut.write("<bi_correctionFactorValue>");
    oOut.write("<bi_cfvVal species=\"Species_1\">1.11</bi_cfvVal>");
    oOut.write("<bi_cfvVal species=\"Species_2\">1.12</bi_cfvVal>");
    oOut.write("</bi_correctionFactorValue>");
    oOut.write("<an_boleB0>");
    oOut.write("<an_bb0Val species=\"Species_1\">0.11</an_bb0Val>");
    oOut.write("<an_bb0Val species=\"Species_2\">0.24</an_bb0Val>");
    oOut.write("</an_boleB0>");
    oOut.write("<an_boleB1>");
    oOut.write("<an_bb1Val species=\"Species_1\">-0.05977</an_bb1Val>");
    oOut.write("<an_bb1Val species=\"Species_2\">-0.05895</an_bb1Val>");
    oOut.write("</an_boleB1>");
    oOut.write("<an_boleB2>");
    oOut.write("<an_bb2Val species=\"Species_1\">2.0498</an_bb2Val>");
    oOut.write("<an_bb2Val species=\"Species_2\">2.0362</an_bb2Val>");
    oOut.write("</an_boleB2>");
    oOut.write("<an_boleB3>");
    oOut.write("<an_bb3Val species=\"Species_1\">0.4965</an_bb3Val>");
    oOut.write("<an_bb3Val species=\"Species_2\">0.4947</an_bb3Val>");
    oOut.write("</an_boleB3>");
    oOut.write("<an_boleB4>");
    oOut.write("<an_bb4Val species=\"Species_1\">2.0198</an_bb4Val>");
    oOut.write("<an_bb4Val species=\"Species_2\">2.0172</an_bb4Val>");
    oOut.write("</an_boleB4>");
    oOut.write("<an_boleB5>");
    oOut.write("<an_bb5Val species=\"Species_1\">0.3468</an_bb5Val>");
    oOut.write("<an_bb5Val species=\"Species_2\">0.3366</an_bb5Val>");
    oOut.write("</an_boleB5>");
    oOut.write("<an_boleFormClasses>");
    oOut.write("<an_bfcVal species=\"Species_1\">80</an_bfcVal>");
    oOut.write("<an_bfcVal species=\"Species_2\">79</an_bfcVal>");
    oOut.write("</an_boleFormClasses>");
    oOut.write("<an_merchValueFormClasses>");
    oOut.write("<an_mvfcVal species=\"Species_1\">78</an_mvfcVal>");
    oOut.write("<an_mvfcVal species=\"Species_2\">79</an_mvfcVal>");
    oOut.write("</an_merchValueFormClasses>");
    oOut.write("<an_merchValuePricePer1KFeet>");
    oOut.write("<an_mvpp1kfVal species=\"Species_1\">1.23</an_mvpp1kfVal>");
    oOut.write("<an_mvpp1kfVal species=\"Species_2\">0.89</an_mvpp1kfVal>");
    oOut.write("</an_merchValuePricePer1KFeet>");
    oOut.write("<an_carbonPercentBiomassCarbon>");
    oOut.write("<an_cpbcVal species=\"Species_1\">0</an_cpbcVal>");
    oOut.write("<an_cpbcVal species=\"Species_2\">100</an_cpbcVal>");
    oOut.write("</an_carbonPercentBiomassCarbon>");
    oOut.write("<an_carbonPricePerMetricTonCarbon>11</an_carbonPricePerMetricTonCarbon>");
    oOut.write("<an_partBioDbhLeafA>");
    oOut.write("<an_pbdlaVal species=\"Species_1\">1.523</an_pbdlaVal>");
    oOut.write("<an_pbdlaVal species=\"Species_2\">1.792</an_pbdlaVal>");
    oOut.write("</an_partBioDbhLeafA>");
    oOut.write("<an_partBioDbhLeafB>");
    oOut.write("<an_pbdlbVal species=\"Species_1\">-3.49</an_pbdlbVal>");
    oOut.write("<an_pbdlbVal species=\"Species_2\">3.758</an_pbdlbVal>");
    oOut.write("</an_partBioDbhLeafB>");
    oOut.write("<an_partBioDbhBranchA>");
    oOut.write("<an_pbdbraVal species=\"Species_1\">2.062</an_pbdbraVal>");
    oOut.write("<an_pbdbraVal species=\"Species_2\">1.982</an_pbdbraVal>");
    oOut.write("</an_partBioDbhBranchA>");
    oOut.write("<an_partBioDbhBranchB>");
    oOut.write("<an_pbdbrbVal species=\"Species_1\">-5.963</an_pbdbrbVal>");
    oOut.write("<an_pbdbrbVal species=\"Species_2\">3.69</an_pbdbrbVal>");
    oOut.write("</an_partBioDbhBranchB>");
    oOut.write("<an_partBioDbhBoleA>");
    oOut.write("<an_pbdboaVal species=\"Species_1\">2.543</an_pbdboaVal>");
    oOut.write("<an_pbdboaVal species=\"Species_2\">2.559</an_pbdboaVal>");
    oOut.write("</an_partBioDbhBoleA>");
    oOut.write("<an_partBioDbhBoleB>");
    oOut.write("<an_pbdbobVal species=\"Species_1\">-3.139</an_pbdbobVal>");
    oOut.write("<an_pbdbobVal species=\"Species_2\">2.832</an_pbdbobVal>");
    oOut.write("</an_partBioDbhBoleB>");
    oOut.write("<an_partBioHeightLeafA>");
    oOut.write("<an_pbhlaVal species=\"Species_1\">1.7</an_pbhlaVal>");
    oOut.write("<an_pbhlaVal species=\"Species_2\">1.3</an_pbhlaVal>");
    oOut.write("</an_partBioHeightLeafA>");
    oOut.write("<an_partBioHeightLeafB>");
    oOut.write("<an_pbhlbVal species=\"Species_1\">6.8</an_pbhlbVal>");
    oOut.write("<an_pbhlbVal species=\"Species_2\">-2.3</an_pbhlbVal>");
    oOut.write("</an_partBioHeightLeafB>");
    oOut.write("<an_partBioHeightBoleA>");
    oOut.write("<an_pbhboaVal species=\"Species_1\">5.9</an_pbhboaVal>");
    oOut.write("<an_pbhboaVal species=\"Species_2\">6.9</an_pbhboaVal>");
    oOut.write("</an_partBioHeightBoleA>");
    oOut.write("<an_partBioHeightBoleB>");
    oOut.write("<an_pbhbobVal species=\"Species_1\">-1.6</an_pbhbobVal>");
    oOut.write("<an_pbhbobVal species=\"Species_2\">-1.1</an_pbhbobVal>");
    oOut.write("</an_partBioHeightBoleB>");
    oOut.write("</analysis>");
    oOut.write("</paramFile>");

    oOut.close();
    return sFileName;
  }
  
  /**
   * Writes a file with 6.8 partitioned biomass parameters, to test backwards
   * compatibility.
   * @return String Filename written.
   * @throws IOException if there is a problem writing the file.
   */
  protected String Write68PartitionedBiomassXMLFile() throws IOException {
    String sFileName = "\\loratestxml1.xml";
    FileWriter oOut = new FileWriter(sFileName);

    WritePlotAndTrees(oOut);
    
    oOut.write("<behaviorList>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>Partitioned DBH Biomass</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Sapling\"/>");
    oOut.write("<applyTo species=\"Species_2\" type=\"Adult\"/>");
    oOut.write("</behavior>");
    oOut.write("</behaviorList>");
    oOut.write("<analysis>");
    oOut.write("<an_partBioDbhLeafA>");
    oOut.write("<an_pbdlaVal species=\"Species_1\">1.523</an_pbdlaVal>");
    oOut.write("<an_pbdlaVal species=\"Species_2\">1.792</an_pbdlaVal>");
    oOut.write("</an_partBioDbhLeafA>");
    oOut.write("<an_partBioDbhLeafB>");
    oOut.write("<an_pbdlbVal species=\"Species_1\">-3.49</an_pbdlbVal>");
    oOut.write("<an_pbdlbVal species=\"Species_2\">3.758</an_pbdlbVal>");
    oOut.write("</an_partBioDbhLeafB>");
    oOut.write("<an_partBioDbhBranchA>");
    oOut.write("<an_pbdbraVal species=\"Species_1\">2.062</an_pbdbraVal>");
    oOut.write("<an_pbdbraVal species=\"Species_2\">1.982</an_pbdbraVal>");
    oOut.write("</an_partBioDbhBranchA>");
    oOut.write("<an_partBioDbhBranchB>");
    oOut.write("<an_pbdbrbVal species=\"Species_1\">-5.963</an_pbdbrbVal>");
    oOut.write("<an_pbdbrbVal species=\"Species_2\">3.69</an_pbdbrbVal>");
    oOut.write("</an_partBioDbhBranchB>");
    oOut.write("<an_partBioDbhBoleA>");
    oOut.write("<an_pbdboaVal species=\"Species_1\">2.543</an_pbdboaVal>");
    oOut.write("<an_pbdboaVal species=\"Species_2\">2.559</an_pbdboaVal>");
    oOut.write("</an_partBioDbhBoleA>");
    oOut.write("<an_partBioDbhBoleB>");
    oOut.write("<an_pbdbobVal species=\"Species_1\">-3.139</an_pbdbobVal>");
    oOut.write("<an_pbdbobVal species=\"Species_2\">2.832</an_pbdbobVal>");
    oOut.write("</an_partBioDbhBoleB>");
    oOut.write("<an_partBioHeightLeafA>");
    oOut.write("<an_pbhlaVal species=\"Species_1\">1.7</an_pbhlaVal>");
    oOut.write("<an_pbhlaVal species=\"Species_2\">1.3</an_pbhlaVal>");
    oOut.write("</an_partBioHeightLeafA>");
    oOut.write("<an_partBioHeightLeafB>");
    oOut.write("<an_pbhlbVal species=\"Species_1\">6.8</an_pbhlbVal>");
    oOut.write("<an_pbhlbVal species=\"Species_2\">-2.3</an_pbhlbVal>");
    oOut.write("</an_partBioHeightLeafB>");
    oOut.write("<an_partBioHeightBoleA>");
    oOut.write("<an_pbhboaVal species=\"Species_1\">5.9</an_pbhboaVal>");
    oOut.write("<an_pbhboaVal species=\"Species_2\">6.9</an_pbhboaVal>");
    oOut.write("</an_partBioHeightBoleA>");
    oOut.write("<an_partBioHeightBoleB>");
    oOut.write("<an_pbhbobVal species=\"Species_1\">-1.6</an_pbhbobVal>");
    oOut.write("<an_pbhbobVal species=\"Species_2\">-1.1</an_pbhbobVal>");
    oOut.write("</an_partBioHeightBoleB>");
    oOut.write("</analysis>");
    oOut.write("</paramFile>");

    oOut.close();
    return sFileName;
  }

  /**
   * Writes a valid file for testing parameter file reading of bole volume
   * calculator behavior parameters.
   * @return String Filename written.
   * @throws IOException if there is a problem writing the file.
   */
  protected String WriteBoleVolumeXMLFile1() throws java.io.IOException {
    String sFileName = "\\loratestxml1.xml";
    FileWriter oOut = new FileWriter(sFileName);

    WriteBoleVolumeCommonStuff(oOut);

    oOut.write("<an_boleFormClasses>");
    oOut.write("<an_bfcVal species=\"Species_2\">80</an_bfcVal>");
    oOut.write("<an_bfcVal species=\"Species_3\">79</an_bfcVal>");
    oOut.write("</an_boleFormClasses>");
    oOut.write("</analysis>");
    oOut.write("</paramFile>");

    oOut.close();
    return sFileName;
  }

  /**
   * Writes a valid file for testing parameter file reading of dimension
   * analysis behavior parameters.
   * @return String Filename written.
   * @throws IOException if there is a problem writing the file.
   */
  protected String WriteDimensionAnalysisXMLFile1() throws java.io.IOException {
    String sFileName = "\\loratestxml1.xml";
    FileWriter oOut = new FileWriter(sFileName);

    WriteDimensionAnalysisCommonStuff(oOut);
    oOut.write("<analysis>");
    oOut.write("<bi_a>");
    oOut.write("<bi_aVal species=\"Species_1\">-0.99</bi_aVal>");
    oOut.write("<bi_aVal species=\"Species_2\">-1.01</bi_aVal>");
    oOut.write("<bi_aVal species=\"Species_3\">-1.09</bi_aVal>");
    oOut.write("</bi_a>");
    oOut.write("<bi_b>");
    oOut.write("<bi_bVal species=\"Species_1\">2.2</bi_bVal>");
    oOut.write("<bi_bVal species=\"Species_2\">2.41</bi_bVal>");
    oOut.write("<bi_bVal species=\"Species_3\">2.35</bi_bVal>");
    oOut.write("</bi_b>");
    oOut.write("<bi_c>");
    oOut.write("<bi_cVal species=\"Species_1\">0.9</bi_cVal>");
    oOut.write("<bi_cVal species=\"Species_2\">1</bi_cVal>");
    oOut.write("<bi_cVal species=\"Species_3\">1.12</bi_cVal>");
    oOut.write("</bi_c>");
    oOut.write("<bi_d>");
    oOut.write("<bi_dVal species=\"Species_1\">1.1</bi_dVal>");
    oOut.write("<bi_dVal species=\"Species_2\">1.2</bi_dVal>");
    oOut.write("<bi_dVal species=\"Species_3\">1.3</bi_dVal>");
    oOut.write("</bi_d>");
    oOut.write("<bi_e>");
    oOut.write("<bi_eVal species=\"Species_1\">0.1</bi_eVal>");
    oOut.write("<bi_eVal species=\"Species_2\">0.2</bi_eVal>");
    oOut.write("<bi_eVal species=\"Species_3\">0.3</bi_eVal>");
    oOut.write("</bi_e>");
    oOut.write("<bi_eqID>");
    oOut.write("<bi_eiVal species=\"Species_1\">4</bi_eiVal>");
    oOut.write("<bi_eiVal species=\"Species_2\">5</bi_eiVal>");
    oOut.write("<bi_eiVal species=\"Species_3\">6</bi_eiVal>");
    oOut.write("</bi_eqID>");
    oOut.write("<bi_dbhUnits>");
    oOut.write("<bi_duVal species=\"Species_1\">0</bi_duVal>");
    oOut.write("<bi_duVal species=\"Species_2\">1</bi_duVal>");
    oOut.write("<bi_duVal species=\"Species_3\">2</bi_duVal>");
    oOut.write("</bi_dbhUnits>");
    oOut.write("<bi_biomassUnits>");
    oOut.write("<bi_buVal species=\"Species_1\">2</bi_buVal>");
    oOut.write("<bi_buVal species=\"Species_2\">1</bi_buVal>");
    oOut.write("<bi_buVal species=\"Species_3\">0</bi_buVal>");
    oOut.write("</bi_biomassUnits>");
    oOut.write("<bi_useCorrectionFactor>");
    oOut.write("<bi_ucfVal species=\"Species_1\">0</bi_ucfVal>");
    oOut.write("<bi_ucfVal species=\"Species_2\">1</bi_ucfVal>");
    oOut.write("<bi_ucfVal species=\"Species_3\">1</bi_ucfVal>");
    oOut.write("</bi_useCorrectionFactor>");
    oOut.write("<bi_correctionFactorValue>");
    oOut.write("<bi_cfvVal species=\"Species_1\">1.11</bi_cfvVal>");
    oOut.write("<bi_cfvVal species=\"Species_2\">1.12</bi_cfvVal>");
    oOut.write("<bi_cfvVal species=\"Species_3\">1.13</bi_cfvVal>");
    oOut.write("</bi_correctionFactorValue>");
    oOut.write("</analysis>");
    oOut.write("</paramFile>");

    oOut.close();
    return sFileName;
  }

  /**
   * Writes a file where the stump height is a negative number.  Tests
   * volume calculator behavior validation.
   * @return String Filename written.
   * @throws IOException if there is a problem writing the file.
   */
  protected String WriteVolumeXMLErrorFile1() throws java.io.IOException {
    String sFileName = "\\loratestxml1.xml";
    FileWriter oOut = new FileWriter(sFileName);

    WriteVolumeCommonStuff(oOut);

    oOut.write("<vo_stumpHeight>-10</vo_stumpHeight>");
    oOut.write("<vo_minUsableDiam>5</vo_minUsableDiam>");
    oOut.write("<vo_segmentLength>1</vo_segmentLength>");
    oOut.write("</volume>");
    oOut.write("</paramFile>");
    oOut.close();
    return sFileName;
  }

  /**
   * Writes a file where the minimum usable diameter is a negative number.  Tests
   * volume calculator behavior validation.
   * @return String Filename written.
   * @throws IOException if there is a problem writing the file.
   */
  protected String WriteVolumeXMLErrorFile2() throws java.io.IOException {
    String sFileName = "\\loratestxml1.xml";
    FileWriter oOut = new FileWriter(sFileName);

    WriteVolumeCommonStuff(oOut);

    oOut.write("<vo_stumpHeight>0</vo_stumpHeight>");
    oOut.write("<vo_minUsableDiam>-5</vo_minUsableDiam>");
    oOut.write("<vo_segmentLength>1</vo_segmentLength>");
    oOut.write("</volume>");
    oOut.write("</paramFile>");
    oOut.close();
    return sFileName;
  }

  /**
   * Writes a file where the segment length is a negative number.  Tests
   * volume calculator behavior validation.
   * @return String Filename written.
   * @throws IOException if there is a problem writing the file.
   */
  protected String WriteVolumeXMLErrorFile3() throws java.io.IOException {
    String sFileName = "\\loratestxml1.xml";
    FileWriter oOut = new FileWriter(sFileName);

    WriteVolumeCommonStuff(oOut);

    oOut.write("<vo_stumpHeight>0</vo_stumpHeight>");
    oOut.write("<vo_minUsableDiam>5</vo_minUsableDiam>");
    oOut.write("<vo_segmentLength>-1</vo_segmentLength>");
    oOut.write("</volume>");
    oOut.write("</paramFile>");
    oOut.close();
    return sFileName;
  }

  /**
   * Writes an error file for testing parameter file reading of bole volume
   * calculator behavior parameters.  This has a form class of less than 60.
   * @return String Filename written.
   * @throws IOException if there is a problem writing the file.
   */
  protected String WriteBoleVolumeErrorXMLFile1() throws IOException {
    String sFileName = "\\loratestxml1.xml";
    FileWriter oOut = new FileWriter(sFileName);

    WriteBoleVolumeCommonStuff(oOut);

    oOut.write("<an_boleFormClasses>");
    oOut.write("<an_bfcVal species=\"Species_2\">80</an_bfcVal>");
    oOut.write("<an_bfcVal species=\"Species_3\">49</an_bfcVal>");
    oOut.write("</an_boleFormClasses>");
    oOut.write("</analysis>");
    oOut.write("</paramFile>");

    oOut.close();
    return sFileName;
  }

  /**
   * Writes an error file for testing parameter file reading of bole volume
   * calculator behavior parameters.  This has a form class of more than 100.
   * @return String Filename written.
   * @throws IOException if there is a problem writing the file.
   */
  protected String WriteBoleVolumeErrorXMLFile2() throws java.io.IOException {
    String sFileName = "\\loratestxml1.xml";
    FileWriter oOut = new FileWriter(sFileName);

    WriteBoleVolumeCommonStuff(oOut);

    oOut.write("<an_boleFormClasses>");
    oOut.write("<an_bfcVal species=\"Species_2\">80</an_bfcVal>");
    oOut.write("<an_bfcVal species=\"Species_3\">149</an_bfcVal>");
    oOut.write("</an_boleFormClasses>");
    oOut.write("</analysis>");
    oOut.write("</paramFile>");

    oOut.close();
    return sFileName;
  }

  /**
   * Writes common stuff for parameter files validating the volume calculator
   * behavior.
   * @param oOut FileWriter File to write to.
   * @throws IOException if there is a problem writing to the file.
   */
  private void WriteVolumeCommonStuff(FileWriter oOut) throws IOException {
  	WritePlotAndTrees(oOut);
    oOut.write("<behaviorList>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>tree volume calculator</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("<applyTo species=\"Species_2\" type=\"Adult\"/>");
    oOut.write("<applyTo species=\"Species_2\" type=\"Sapling\"/>");
    oOut.write("<applyTo species=\"Species_3\" type=\"Adult\"/>");
    oOut.write("<applyTo species=\"Species_3\" type=\"Sapling\"/>");
    oOut.write("</behavior>");
    oOut.write("</behaviorList>");
    oOut.write("<volume>");
    oOut.write("<vo_taperA>");
    oOut.write("<vo_taVal species=\"Species_2\">0.843142</vo_taVal>");
    oOut.write("<vo_taVal species=\"Species_3\">0.8</vo_taVal>");
    oOut.write("</vo_taperA>");
    oOut.write("<vo_taperB>");
    oOut.write("<vo_tbVal species=\"Species_2\">1.00813</vo_tbVal>");
    oOut.write("<vo_tbVal species=\"Species_3\">0.9</vo_tbVal>");
    oOut.write("</vo_taperB>");
    oOut.write("<vo_taperC>");
    oOut.write("<vo_tcVal species=\"Species_2\">0.031402</vo_tcVal>");
    oOut.write("<vo_tcVal species=\"Species_3\">0.032</vo_tcVal>");
    oOut.write("</vo_taperC>");
    oOut.write("<vo_taperD>");
    oOut.write("<vo_tdVal species=\"Species_2\">0.319773</vo_tdVal>");
    oOut.write("<vo_tdVal species=\"Species_3\">0.321</vo_tdVal>");
    oOut.write("</vo_taperD>");
    oOut.write("<vo_taperF>");
    oOut.write("<vo_tfVal species=\"Species_2\">-0.532732</vo_tfVal>");
    oOut.write("<vo_tfVal species=\"Species_3\">-0.51</vo_tfVal>");
    oOut.write("</vo_taperF>");
    oOut.write("<vo_taperG>");
    oOut.write("<vo_tgVal species=\"Species_2\">0.585361</vo_tgVal>");
    oOut.write("<vo_tgVal species=\"Species_3\">0.59</vo_tgVal>");
    oOut.write("</vo_taperG>");
    oOut.write("<vo_taperI>");
    oOut.write("<vo_tiVal species=\"Species_2\">1.0315</vo_tiVal>");
    oOut.write("<vo_tiVal species=\"Species_3\">1.04</vo_tiVal>");
    oOut.write("</vo_taperI>");
    oOut.write("<vo_taperJ>");
    oOut.write("<vo_tjVal species=\"Species_2\">0.018622</vo_tjVal>");
    oOut.write("<vo_tjVal species=\"Species_3\">0.02</vo_tjVal>");
    oOut.write("</vo_taperJ>");
    oOut.write("<vo_taperK>");
    oOut.write("<vo_tkVal species=\"Species_2\">-0.225146</vo_tkVal>");
    oOut.write("<vo_tkVal species=\"Species_3\">-0.23</vo_tkVal>");
    oOut.write("</vo_taperK>");
    oOut.write("<vo_barkA>");
    oOut.write("<vo_baVal species=\"Species_2\">0.2</vo_baVal>");
    oOut.write("<vo_baVal species=\"Species_3\">0.01</vo_baVal>");
    oOut.write("</vo_barkA>");
    oOut.write("<vo_barkB>");
    oOut.write("<vo_bbVal species=\"Species_2\">0.8</vo_bbVal>");
    oOut.write("<vo_bbVal species=\"Species_3\">1</vo_bbVal>");
    oOut.write("</vo_barkB>");
    oOut.write("<vo_barkC>");
    oOut.write("<vo_bcVal species=\"Species_2\">0.01</vo_bcVal>");
    oOut.write("<vo_bcVal species=\"Species_3\">0.001</vo_bcVal>");
    oOut.write("</vo_barkC>");
  }

  /**
   * Writes common stuff for parameter files validating the bole volume
   * calculator behavior.
   * @param oOut FileWriter File to write to.
   * @throws IOException if there is a problem writing to the file.
   */
  private void WriteBoleVolumeCommonStuff(FileWriter oOut) throws java.io.
      IOException {
    WritePlotAndTrees(oOut);
    
    oOut.write("<behaviorList>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>tree bole volume calculator</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("<applyTo species=\"Species_2\" type=\"Adult\"/>");
    oOut.write("<applyTo species=\"Species_2\" type=\"Sapling\"/>");
    oOut.write("<applyTo species=\"Species_3\" type=\"Adult\"/>");
    oOut.write("<applyTo species=\"Species_3\" type=\"Sapling\"/>");
    oOut.write("</behavior>");
    oOut.write("</behaviorList>");
    oOut.write("<analysis>");
    oOut.write("<an_boleB0>");
    oOut.write("<an_bb0Val species=\"Species_2\">0.11</an_bb0Val>");
    oOut.write("<an_bb0Val species=\"Species_3\">0.24</an_bb0Val>");
    oOut.write("</an_boleB0>");
    oOut.write("<an_boleB1>");
    oOut.write("<an_bb1Val species=\"Species_2\">-0.05977</an_bb1Val>");
    oOut.write("<an_bb1Val species=\"Species_3\">-0.05895</an_bb1Val>");
    oOut.write("</an_boleB1>");
    oOut.write("<an_boleB2>");
    oOut.write("<an_bb2Val species=\"Species_2\">2.0498</an_bb2Val>");
    oOut.write("<an_bb2Val species=\"Species_3\">2.0362</an_bb2Val>");
    oOut.write("</an_boleB2>");
    oOut.write("<an_boleB3>");
    oOut.write("<an_bb3Val species=\"Species_2\">0.4965</an_bb3Val>");
    oOut.write("<an_bb3Val species=\"Species_3\">0.4947</an_bb3Val>");
    oOut.write("</an_boleB3>");
    oOut.write("<an_boleB4>");
    oOut.write("<an_bb4Val species=\"Species_2\">2.0198</an_bb4Val>");
    oOut.write("<an_bb4Val species=\"Species_3\">2.0172</an_bb4Val>");
    oOut.write("</an_boleB4>");
    oOut.write("<an_boleB5>");
    oOut.write("<an_bb5Val species=\"Species_2\">0.3468</an_bb5Val>");
    oOut.write("<an_bb5Val species=\"Species_3\">0.3366</an_bb5Val>");
    oOut.write("</an_boleB5>");
  }

  /**
   * Writes common stuff for parameter files validating the dimension analysis
   * behavior.
   * @param oOut FileWriter File to write to.
   * @throws IOException if there is a problem writing to the file.
   */
  private void WriteDimensionAnalysisCommonStuff(FileWriter oOut) throws java.io.
      IOException {
  	WritePlotAndTrees(oOut);
  	
    oOut.write("<behaviorList>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>tree biomass calculator</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Adult\"/>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Sapling\"/>");
    oOut.write("<applyTo species=\"Species_2\" type=\"Adult\"/>");
    oOut.write("<applyTo species=\"Species_2\" type=\"Sapling\"/>");
    oOut.write("<applyTo species=\"Species_3\" type=\"Adult\"/>");
    oOut.write("<applyTo species=\"Species_3\" type=\"Sapling\"/>");
    oOut.write("</behavior>");
    oOut.write("</behaviorList>");
  }

  /**
   * Writes a file with no analysis behaviors.
   * @return String Filename written.
   * @throws IOException if there is a problem writing the file.
   */
  protected String WriteNeutralFile() throws IOException {
    String sFileName = "\\loratestxml1.xml";
    FileWriter oOut = new FileWriter(sFileName);

    WritePlotAndTrees(oOut);
    
    oOut.write("<behaviorList>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>radiallight</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Adult\"/>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Sapling\"/>");
    oOut.write("<applyTo species=\"Species_2\" type=\"Adult\"/>");
    oOut.write("<applyTo species=\"Species_2\" type=\"Sapling\"/>");
    oOut.write("<applyTo species=\"Species_3\" type=\"Adult\"/>");
    oOut.write("<applyTo species=\"Species_3\" type=\"Sapling\"/>");
    oOut.write("</behavior>");
    oOut.write("</behaviorList>");
    oOut.write("</paramFile>");

    oOut.close();
    return sFileName;
  }
  
  /**
   * Writes plot and trees with three species.
   * @param oOut FileWriter File to write to.
   * @throws IOException if there is a problem writing to the file.
   */
  private void WritePlotAndTrees(FileWriter oOut) throws IOException {
    oOut.write("<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>");
    oOut.write("<paramFile fileCode=\"06010101\">");
    oOut.write("<plot>");
    oOut.write("<timesteps>2</timesteps>");
    oOut.write("<yearsPerTimestep>1</yearsPerTimestep>");
    oOut.write("<randomSeed>1</randomSeed>");
    oOut.write("<plot_lenX>200.0</plot_lenX>");
    oOut.write("<plot_lenY>200.0</plot_lenY>");
    oOut.write("<plot_latitude>55.37</plot_latitude>");
    oOut.write("</plot>");
    oOut.write("<trees>");
    oOut.write("<tr_speciesList>");
    oOut.write("<tr_species speciesName=\"Species_1\" />");
    oOut.write("<tr_species speciesName=\"Species_2\" />");
    oOut.write("<tr_species speciesName=\"Species_3\" />");
    oOut.write("</tr_speciesList>");
    oOut.write("<tr_seedDiam10Cm>0.1</tr_seedDiam10Cm>");
    oOut.write("<tr_minAdultDBH>");
    oOut.write("<tr_madVal species=\"Species_1\">0.0</tr_madVal>");
    oOut.write("<tr_madVal species=\"Species_2\">0.0</tr_madVal>");
    oOut.write("<tr_madVal species=\"Species_3\">0.0</tr_madVal>");
    oOut.write("</tr_minAdultDBH>");
    oOut.write("</trees>");
    oOut.write("<allometry>");
    oOut.write("<tr_canopyHeight>");
    oOut.write("<tr_chVal species=\"Species_1\">39.48</tr_chVal>");
    oOut.write("<tr_chVal species=\"Species_2\">39.54</tr_chVal>");
    oOut.write("<tr_chVal species=\"Species_3\">45</tr_chVal>");
    oOut.write("</tr_canopyHeight>");
    oOut.write("<tr_stdAsympCrownRad>");
    oOut.write("<tr_sacrVal species=\"Species_1\">0.0549</tr_sacrVal>");
    oOut.write("<tr_sacrVal species=\"Species_2\">0.0549</tr_sacrVal>");
    oOut.write("<tr_sacrVal species=\"Species_3\">0.0549</tr_sacrVal>");
    oOut.write("</tr_stdAsympCrownRad>");
    oOut.write("<tr_stdCrownRadExp>");
    oOut.write("<tr_screVal species=\"Species_1\">1.0</tr_screVal>");
    oOut.write("<tr_screVal species=\"Species_2\">1.0</tr_screVal>");
    oOut.write("<tr_screVal species=\"Species_3\">1.0</tr_screVal>");
    oOut.write("</tr_stdCrownRadExp>");
    oOut.write("<tr_conversionDiam10ToDBH>");
    oOut.write("<tr_cdtdVal species=\"Species_1\">0.8008</tr_cdtdVal>");
    oOut.write("<tr_cdtdVal species=\"Species_2\">0.8008</tr_cdtdVal>");
    oOut.write("<tr_cdtdVal species=\"Species_3\">0.8008</tr_cdtdVal>");
    oOut.write("</tr_conversionDiam10ToDBH>");
    oOut.write("<tr_stdAsympCrownHt>");
    oOut.write("<tr_sachVal species=\"Species_1\">0.389</tr_sachVal>");
    oOut.write("<tr_sachVal species=\"Species_2\">0.389</tr_sachVal>");
    oOut.write("<tr_sachVal species=\"Species_3\">0.389</tr_sachVal>");
    oOut.write("</tr_stdAsympCrownHt>");
    oOut.write("<tr_stdCrownHtExp>");
    oOut.write("<tr_scheVal species=\"Species_1\">1.0</tr_scheVal>");
    oOut.write("<tr_scheVal species=\"Species_2\">1.0</tr_scheVal>");
    oOut.write("<tr_scheVal species=\"Species_3\">1.0</tr_scheVal>");
    oOut.write("</tr_stdCrownHtExp>");
    oOut.write("<tr_slopeOfHeight-Diam10>");
    oOut.write("<tr_sohdVal species=\"Species_1\">0.03418</tr_sohdVal>");
    oOut.write("<tr_sohdVal species=\"Species_2\">0.03418</tr_sohdVal>");
    oOut.write("<tr_sohdVal species=\"Species_3\">0.03418</tr_sohdVal>");
    oOut.write("</tr_slopeOfHeight-Diam10>");
    oOut.write("<tr_slopeOfAsymHeight>");
    oOut.write("<tr_soahVal species=\"Species_1\">0.0299</tr_soahVal>");
    oOut.write("<tr_soahVal species=\"Species_2\">0.0241</tr_soahVal>");
    oOut.write("<tr_soahVal species=\"Species_3\">0.0263</tr_soahVal>");
    oOut.write("</tr_slopeOfAsymHeight>");
    oOut.write("</allometry>");
  }
}
