package javawrapper;

import junit.framework.*;

/**
 * Tests the DetailedOutputSettings class.
 * <p>Copyright: Copyright (c) Charles D. Canham 2003</p>
 * <p>Company: Institute of Ecosystem Studies</p>
 * @author Lora E. Murphy
 * @version 1.0
 *
 * <br>Edit history:
 * <br>------------------
 * <br>April 28, 2004: Submitted in beta version (LEM)
 */
public class TestDetailedGridSettings
    extends TestCase {

  /** Object to test on */
  private DetailedGridSettings m_oDetailedGridSettings = null;
  private DetailedGridSettings m_oCopyTarget = null;

  /**
   * Does setup.
   * @throws Exception Won't.
   */
  protected void setUp() throws Exception {
    super.setUp();
    m_oDetailedGridSettings = new DetailedGridSettings("Test Grid Settings");
    assertEquals(0, m_oDetailedGridSettings.GetNumberOfBools());
    assertEquals(0, m_oDetailedGridSettings.GetNumberOfChars());
    assertEquals(0, m_oDetailedGridSettings.GetNumberOfFloats());
    assertEquals(0, m_oDetailedGridSettings.GetNumberOfInts());
    assertEquals(0, m_oDetailedGridSettings.GetNumberOfPackageBools());
    assertEquals(0, m_oDetailedGridSettings.GetNumberOfPackageChars());
    assertEquals(0, m_oDetailedGridSettings.GetNumberOfPackageFloats());
    assertEquals(0, m_oDetailedGridSettings.GetNumberOfPackageInts());
    assertEquals(0, m_oDetailedGridSettings.GetSaveFrequency());
    assertEquals("Test Grid Settings", m_oDetailedGridSettings.GetName());
  }

  /**
   * Does teardown
   * @throws Exception Won't.
   */
  protected void tearDown() throws Exception {
    m_oDetailedGridSettings = null;
    super.tearDown();
  }

  protected void TestCloning() {
    //test cloning
    m_oCopyTarget = (DetailedGridSettings) m_oDetailedGridSettings.clone();
    assertEquals(1, m_oCopyTarget.GetNumberOfBools());
    assertEquals(2, m_oCopyTarget.GetNumberOfChars());
    assertEquals(3, m_oCopyTarget.GetNumberOfFloats());
    assertEquals(4, m_oCopyTarget.GetNumberOfInts());
    assertEquals(1, m_oCopyTarget.GetNumberOfPackageBools());
    assertEquals(2, m_oCopyTarget.GetNumberOfPackageChars());
    assertEquals(3, m_oCopyTarget.GetNumberOfPackageFloats());
    assertEquals(4, m_oCopyTarget.GetNumberOfPackageInts());
    assertEquals(5, m_oCopyTarget.GetSaveFrequency());
    assertEquals(m_oCopyTarget.GetName(), m_oDetailedGridSettings.GetName());

    //Data member names
    assertEquals(m_oDetailedGridSettings.GetBool(0).GetCodeName(),
                 m_oCopyTarget.GetBool(0).GetCodeName());
    assertEquals(m_oDetailedGridSettings.GetFloat(0).GetDisplayName(),
                 m_oCopyTarget.GetFloat(0).GetDisplayName());
    assertEquals(m_oDetailedGridSettings.GetInt(0).GetDisplayName(),
                 m_oCopyTarget.GetInt(0).GetDisplayName());
    assertEquals(m_oDetailedGridSettings.GetChar(0).GetDisplayName(),
                 m_oCopyTarget.GetChar(0).GetDisplayName());
    assertEquals(m_oDetailedGridSettings.GetPackageBool(0).GetCodeName(),
                 m_oCopyTarget.GetPackageBool(0).GetCodeName());
    assertEquals(m_oDetailedGridSettings.GetPackageFloat(0).GetDisplayName(),
                 m_oCopyTarget.GetPackageFloat(0).GetDisplayName());
    assertEquals(m_oDetailedGridSettings.GetPackageInt(0).GetDisplayName(),
                 m_oCopyTarget.GetPackageInt(0).GetDisplayName());
    assertEquals(m_oDetailedGridSettings.GetPackageChar(0).GetDisplayName(),
                 m_oCopyTarget.GetPackageChar(0).GetDisplayName());

    //Make sure the clone was deep
    DataMember oMember = m_oDetailedGridSettings.GetBool(0);
    int iSaveCode = oMember.GetCode();
    oMember.SetCode(iSaveCode+4);

    assertEquals(iSaveCode, m_oCopyTarget.GetBool(0).GetCode());
  }

  protected void TestRemoval() {
    /////////////////////////////////
    //Test data member removal
    /////////////////////////////////

    //Invalid index
    m_oDetailedGridSettings.RemoveBool(-1);
    assertEquals(1, m_oDetailedGridSettings.GetNumberOfBools());
    m_oDetailedGridSettings.RemoveBool(1);
    assertEquals(1, m_oDetailedGridSettings.GetNumberOfBools());
    //Good index
    m_oDetailedGridSettings.RemoveBool(0);
    assertEquals(0, m_oDetailedGridSettings.GetNumberOfBools());
    assertEquals(2, m_oDetailedGridSettings.GetNumberOfChars());
    assertEquals(3, m_oDetailedGridSettings.GetNumberOfFloats());
    assertEquals(4, m_oDetailedGridSettings.GetNumberOfInts());
    //Double-checking deep cloning...
    assertEquals(1, m_oCopyTarget.GetNumberOfBools());

    //Invalid index
    m_oDetailedGridSettings.RemoveChar(-1);
    assertEquals(2, m_oDetailedGridSettings.GetNumberOfChars());
    m_oDetailedGridSettings.RemoveChar(5);
    assertEquals(2, m_oDetailedGridSettings.GetNumberOfChars());
    //Good index
    m_oDetailedGridSettings.RemoveChar(0);
    assertEquals(0, m_oDetailedGridSettings.GetNumberOfBools());
    assertEquals(1, m_oDetailedGridSettings.GetNumberOfChars());
    assertEquals(3, m_oDetailedGridSettings.GetNumberOfFloats());
    assertEquals(4, m_oDetailedGridSettings.GetNumberOfInts());
    assertEquals("test char 2", m_oDetailedGridSettings.GetChar(0).GetDisplayName());
    //Double-checking deep cloning...
    assertEquals(2, m_oCopyTarget.GetNumberOfChars());

    //Invalid index
    m_oDetailedGridSettings.RemoveFloat(-1);
    assertEquals(3, m_oDetailedGridSettings.GetNumberOfFloats());
    m_oDetailedGridSettings.RemoveFloat(5);
    assertEquals(3, m_oDetailedGridSettings.GetNumberOfFloats());
    //Good index
    m_oDetailedGridSettings.RemoveFloat(2);
    assertEquals(0, m_oDetailedGridSettings.GetNumberOfBools());
    assertEquals(1, m_oDetailedGridSettings.GetNumberOfChars());
    assertEquals(2, m_oDetailedGridSettings.GetNumberOfFloats());
    assertEquals(4, m_oDetailedGridSettings.GetNumberOfInts());
    assertEquals("test float 1", m_oDetailedGridSettings.GetFloat(0).GetDisplayName());
    assertEquals("test float 2", m_oDetailedGridSettings.GetFloat(1).GetDisplayName());
    //Double-checking deep cloning...
    assertEquals(3, m_oCopyTarget.GetNumberOfFloats());

    //Invalid index
    m_oDetailedGridSettings.RemoveInt(-1);
    assertEquals(4, m_oDetailedGridSettings.GetNumberOfInts());
    m_oDetailedGridSettings.RemoveInt(5);
    assertEquals(4, m_oDetailedGridSettings.GetNumberOfInts());
    //Good index
    m_oDetailedGridSettings.RemoveInt(2);
    assertEquals(0, m_oDetailedGridSettings.GetNumberOfBools());
    assertEquals(1, m_oDetailedGridSettings.GetNumberOfChars());
    assertEquals(2, m_oDetailedGridSettings.GetNumberOfFloats());
    assertEquals(3, m_oDetailedGridSettings.GetNumberOfInts());
    assertEquals("test int 1", m_oDetailedGridSettings.GetInt(0).GetDisplayName());
    assertEquals("test int 2", m_oDetailedGridSettings.GetInt(1).GetDisplayName());
    assertEquals("test int 4", m_oDetailedGridSettings.GetInt(2).GetDisplayName());
    //Double-checking deep cloning...
    assertEquals(4, m_oCopyTarget.GetNumberOfInts());

    //////////////////////
    // Repeat with packages
    //////////////////////
    //Invalid index
    m_oDetailedGridSettings.RemovePackageBool(-1);
    assertEquals(1, m_oDetailedGridSettings.GetNumberOfPackageBools());
    m_oDetailedGridSettings.RemovePackageBool(1);
    assertEquals(1, m_oDetailedGridSettings.GetNumberOfPackageBools());
    //Good index
    m_oDetailedGridSettings.RemovePackageBool(0);
    assertEquals(0, m_oDetailedGridSettings.GetNumberOfPackageBools());
    assertEquals(2, m_oDetailedGridSettings.GetNumberOfPackageChars());
    assertEquals(3, m_oDetailedGridSettings.GetNumberOfPackageFloats());
    assertEquals(4, m_oDetailedGridSettings.GetNumberOfPackageInts());
    //Double-checking deep cloning...
    assertEquals(1, m_oCopyTarget.GetNumberOfPackageBools());

    //Invalid index
    m_oDetailedGridSettings.RemovePackageChar(-1);
    assertEquals(2, m_oDetailedGridSettings.GetNumberOfPackageChars());
    m_oDetailedGridSettings.RemovePackageChar(5);
    assertEquals(2, m_oDetailedGridSettings.GetNumberOfPackageChars());
    //Good index
    m_oDetailedGridSettings.RemovePackageChar(0);
    assertEquals(0, m_oDetailedGridSettings.GetNumberOfPackageBools());
    assertEquals(1, m_oDetailedGridSettings.GetNumberOfPackageChars());
    assertEquals(3, m_oDetailedGridSettings.GetNumberOfPackageFloats());
    assertEquals(4, m_oDetailedGridSettings.GetNumberOfPackageInts());
    assertEquals("test package char 2", m_oDetailedGridSettings.GetPackageChar(0).GetDisplayName());
    //Double-checking deep cloning...
    assertEquals(2, m_oCopyTarget.GetNumberOfPackageChars());

    //Invalid index
    m_oDetailedGridSettings.RemovePackageFloat(-1);
    assertEquals(3, m_oDetailedGridSettings.GetNumberOfPackageFloats());
    m_oDetailedGridSettings.RemovePackageFloat(5);
    assertEquals(3, m_oDetailedGridSettings.GetNumberOfPackageFloats());
    //Good index
    m_oDetailedGridSettings.RemovePackageFloat(2);
    assertEquals(0, m_oDetailedGridSettings.GetNumberOfPackageBools());
    assertEquals(1, m_oDetailedGridSettings.GetNumberOfPackageChars());
    assertEquals(2, m_oDetailedGridSettings.GetNumberOfPackageFloats());
    assertEquals(4, m_oDetailedGridSettings.GetNumberOfPackageInts());
    assertEquals("test package float 1", m_oDetailedGridSettings.GetPackageFloat(0).GetDisplayName());
    assertEquals("test package float 2", m_oDetailedGridSettings.GetPackageFloat(1).GetDisplayName());
    //Double-checking deep cloning...
    assertEquals(3, m_oCopyTarget.GetNumberOfPackageFloats());

    //Invalid index
    m_oDetailedGridSettings.RemovePackageInt(-1);
    assertEquals(4, m_oDetailedGridSettings.GetNumberOfPackageInts());
    m_oDetailedGridSettings.RemovePackageInt(5);
    assertEquals(4, m_oDetailedGridSettings.GetNumberOfPackageInts());
    //Good index
    m_oDetailedGridSettings.RemovePackageInt(2);
    assertEquals(0, m_oDetailedGridSettings.GetNumberOfPackageBools());
    assertEquals(1, m_oDetailedGridSettings.GetNumberOfPackageChars());
    assertEquals(2, m_oDetailedGridSettings.GetNumberOfPackageFloats());
    assertEquals(3, m_oDetailedGridSettings.GetNumberOfPackageInts());
    assertEquals("test package int 1", m_oDetailedGridSettings.GetPackageInt(0).GetDisplayName());
    assertEquals("test package int 2", m_oDetailedGridSettings.GetPackageInt(1).GetDisplayName());
    assertEquals("test package int 4", m_oDetailedGridSettings.GetPackageInt(2).GetDisplayName());
    //Double-checking deep cloning...
    assertEquals(4, m_oCopyTarget.GetNumberOfPackageInts());
  }

  /**
   * This concentrates on adding using both a code name and a display name.
   */
  protected void TestAdding() {
    String sDisplayName = "test bool 1";
    String sCodeName = "bool1";
    m_oDetailedGridSettings.AddBool(sCodeName, sDisplayName);
    assertEquals(1, m_oDetailedGridSettings.GetNumberOfBools());
    assertEquals(0, m_oDetailedGridSettings.GetNumberOfPackageBools());
    assertEquals(sCodeName, m_oDetailedGridSettings.GetBool(0).GetCodeName());
    assertEquals(sDisplayName, m_oDetailedGridSettings.GetBool(0).GetDisplayName());

    sDisplayName = "test char 1";
    sCodeName = "char1";
    m_oDetailedGridSettings.AddChar(sCodeName, sDisplayName);
    assertEquals(1, m_oDetailedGridSettings.GetNumberOfChars());
    assertEquals(0, m_oDetailedGridSettings.GetNumberOfPackageChars());
    assertEquals(sCodeName, m_oDetailedGridSettings.GetChar(0).GetCodeName());
    assertEquals(sDisplayName, m_oDetailedGridSettings.GetChar(0).GetDisplayName());

    sDisplayName = "test char 2";
    sCodeName = "char2";
    m_oDetailedGridSettings.AddChar(sCodeName, sDisplayName);
    assertEquals(2, m_oDetailedGridSettings.GetNumberOfChars());
    assertEquals(0, m_oDetailedGridSettings.GetNumberOfPackageChars());
    assertEquals("test char 1", m_oDetailedGridSettings.GetChar(0).GetDisplayName());
    assertEquals("char1", m_oDetailedGridSettings.GetChar(0).GetCodeName());
    assertEquals(sCodeName, m_oDetailedGridSettings.GetChar(1).GetCodeName());
    assertEquals(sDisplayName, m_oDetailedGridSettings.GetChar(1).GetDisplayName());

    sDisplayName = "test float 1";
    sCodeName = "float1";
    m_oDetailedGridSettings.AddFloat(sCodeName, sDisplayName);
    assertEquals(1, m_oDetailedGridSettings.GetNumberOfFloats());
    assertEquals(0, m_oDetailedGridSettings.GetNumberOfPackageFloats());
    assertEquals(sCodeName, m_oDetailedGridSettings.GetFloat(0).GetCodeName());
    assertEquals(sDisplayName, m_oDetailedGridSettings.GetFloat(0).GetDisplayName());

    sDisplayName = "test float 2";
    sCodeName = "float2";
    m_oDetailedGridSettings.AddFloat(sCodeName, sDisplayName);
    assertEquals(2, m_oDetailedGridSettings.GetNumberOfFloats());
    assertEquals(0, m_oDetailedGridSettings.GetNumberOfPackageFloats());
    assertEquals("test float 1", m_oDetailedGridSettings.GetFloat(0).GetDisplayName());
    assertEquals("float1", m_oDetailedGridSettings.GetFloat(0).GetCodeName());
    assertEquals(sCodeName, m_oDetailedGridSettings.GetFloat(1).GetCodeName());
    assertEquals(sDisplayName, m_oDetailedGridSettings.GetFloat(1).GetDisplayName());

    sDisplayName = "test float 3";
    sCodeName = "float3";
    m_oDetailedGridSettings.AddFloat(sCodeName, sDisplayName);
    assertEquals(3, m_oDetailedGridSettings.GetNumberOfFloats());
    assertEquals(0, m_oDetailedGridSettings.GetNumberOfPackageFloats());
    assertEquals("test float 1", m_oDetailedGridSettings.GetFloat(0).GetDisplayName());
    assertEquals("float1", m_oDetailedGridSettings.GetFloat(0).GetCodeName());
    assertEquals("test float 2", m_oDetailedGridSettings.GetFloat(1).GetDisplayName());
    assertEquals("float2", m_oDetailedGridSettings.GetFloat(1).GetCodeName());
    assertEquals(sCodeName, m_oDetailedGridSettings.GetFloat(2).GetCodeName());
    assertEquals(sDisplayName, m_oDetailedGridSettings.GetFloat(2).GetDisplayName());

    sDisplayName = "test int 1";
    sCodeName = "int1";
    m_oDetailedGridSettings.AddInt(sCodeName, sDisplayName);
    assertEquals(1, m_oDetailedGridSettings.GetNumberOfInts());
    assertEquals(0, m_oDetailedGridSettings.GetNumberOfPackageInts());
    assertEquals(sCodeName, m_oDetailedGridSettings.GetInt(0).GetCodeName());
    assertEquals(sDisplayName, m_oDetailedGridSettings.GetInt(0).GetDisplayName());

    sDisplayName = "test int 2";
    sCodeName = "int2";
    m_oDetailedGridSettings.AddInt(sCodeName, sDisplayName);
    assertEquals(2, m_oDetailedGridSettings.GetNumberOfInts());
    assertEquals(0, m_oDetailedGridSettings.GetNumberOfPackageInts());
    assertEquals("test int 1", m_oDetailedGridSettings.GetInt(0).GetDisplayName());
    assertEquals("int1", m_oDetailedGridSettings.GetInt(0).GetCodeName());
    assertEquals(sCodeName, m_oDetailedGridSettings.GetInt(1).GetCodeName());
    assertEquals(sDisplayName, m_oDetailedGridSettings.GetInt(1).GetDisplayName());

    sDisplayName = "test int 3";
    sCodeName = "int3";
    m_oDetailedGridSettings.AddInt(sCodeName, sDisplayName);
    assertEquals(3, m_oDetailedGridSettings.GetNumberOfInts());
    assertEquals(0, m_oDetailedGridSettings.GetNumberOfPackageInts());
    assertEquals("test int 1", m_oDetailedGridSettings.GetInt(0).GetDisplayName());
    assertEquals("int1", m_oDetailedGridSettings.GetInt(0).GetCodeName());
    assertEquals("test int 2", m_oDetailedGridSettings.GetInt(1).GetDisplayName());
    assertEquals("int2", m_oDetailedGridSettings.GetInt(1).GetCodeName());
    assertEquals(sCodeName, m_oDetailedGridSettings.GetInt(2).GetCodeName());
    assertEquals(sDisplayName, m_oDetailedGridSettings.GetInt(2).GetDisplayName());

    sDisplayName = "test int 4";
    sCodeName = "int4";
    m_oDetailedGridSettings.AddInt(sCodeName, sDisplayName);
    assertEquals(4, m_oDetailedGridSettings.GetNumberOfInts());
    assertEquals(0, m_oDetailedGridSettings.GetNumberOfPackageInts());
    assertEquals("test int 1", m_oDetailedGridSettings.GetInt(0).GetDisplayName());
    assertEquals("int1", m_oDetailedGridSettings.GetInt(0).GetCodeName());
    assertEquals("test int 2", m_oDetailedGridSettings.GetInt(1).GetDisplayName());
    assertEquals("int2", m_oDetailedGridSettings.GetInt(1).GetCodeName());
    assertEquals("test int 3", m_oDetailedGridSettings.GetInt(2).GetDisplayName());
    assertEquals("int3", m_oDetailedGridSettings.GetInt(2).GetCodeName());
    assertEquals(sCodeName, m_oDetailedGridSettings.GetInt(3).GetCodeName());
    assertEquals(sDisplayName, m_oDetailedGridSettings.GetInt(3).GetDisplayName());

    sDisplayName = "test package bool 1";
    sCodeName = "packagebool1";
    m_oDetailedGridSettings.AddPackageBool(sCodeName, sDisplayName);
    assertEquals(1, m_oDetailedGridSettings.GetNumberOfBools());
    assertEquals(1, m_oDetailedGridSettings.GetNumberOfPackageBools());
    assertEquals(sCodeName, m_oDetailedGridSettings.GetPackageBool(0).GetCodeName());
    assertEquals(sDisplayName, m_oDetailedGridSettings.GetPackageBool(0).GetDisplayName());

    sDisplayName = "test package char 1";
    sCodeName = "packagechar1";
    m_oDetailedGridSettings.AddPackageChar(sCodeName, sDisplayName);
    assertEquals(2, m_oDetailedGridSettings.GetNumberOfChars());
    assertEquals(1, m_oDetailedGridSettings.GetNumberOfPackageChars());
    assertEquals(sCodeName, m_oDetailedGridSettings.GetPackageChar(0).GetCodeName());
    assertEquals(sDisplayName, m_oDetailedGridSettings.GetPackageChar(0).GetDisplayName());

    sDisplayName = "test package char 2";
    sCodeName = "packagechar2";
    m_oDetailedGridSettings.AddPackageChar(sCodeName, sDisplayName);
    assertEquals(2, m_oDetailedGridSettings.GetNumberOfPackageChars());
    assertEquals(2, m_oDetailedGridSettings.GetNumberOfPackageChars());
    assertEquals("test package char 1", m_oDetailedGridSettings.GetPackageChar(0).GetDisplayName());
    assertEquals("packagechar1", m_oDetailedGridSettings.GetPackageChar(0).GetCodeName());
    assertEquals(sCodeName, m_oDetailedGridSettings.GetPackageChar(1).GetCodeName());
    assertEquals(sDisplayName, m_oDetailedGridSettings.GetPackageChar(1).GetDisplayName());

    sDisplayName = "test package float 1";
    sCodeName = "packagefloat1";
    m_oDetailedGridSettings.AddPackageFloat(sCodeName, sDisplayName);
    assertEquals(3, m_oDetailedGridSettings.GetNumberOfFloats());
    assertEquals(1, m_oDetailedGridSettings.GetNumberOfPackageFloats());
    assertEquals(sCodeName, m_oDetailedGridSettings.GetPackageFloat(0).GetCodeName());
    assertEquals(sDisplayName, m_oDetailedGridSettings.GetPackageFloat(0).GetDisplayName());

    sDisplayName = "test package float 2";
    sCodeName = "packagefloat2";
    m_oDetailedGridSettings.AddPackageFloat(sCodeName, sDisplayName);
    assertEquals(3, m_oDetailedGridSettings.GetNumberOfFloats());
    assertEquals(2, m_oDetailedGridSettings.GetNumberOfPackageFloats());
    assertEquals("test package float 1", m_oDetailedGridSettings.GetPackageFloat(0).GetDisplayName());
    assertEquals("packagefloat1", m_oDetailedGridSettings.GetPackageFloat(0).GetCodeName());
    assertEquals(sCodeName, m_oDetailedGridSettings.GetPackageFloat(1).GetCodeName());
    assertEquals(sDisplayName, m_oDetailedGridSettings.GetPackageFloat(1).GetDisplayName());

    sDisplayName = "test package float 3";
    sCodeName = "packagefloat3";
    m_oDetailedGridSettings.AddPackageFloat(sCodeName, sDisplayName);
    assertEquals(3, m_oDetailedGridSettings.GetNumberOfFloats());
    assertEquals(3, m_oDetailedGridSettings.GetNumberOfPackageFloats());
    assertEquals("test package float 1", m_oDetailedGridSettings.GetPackageFloat(0).GetDisplayName());
    assertEquals("packagefloat1", m_oDetailedGridSettings.GetPackageFloat(0).GetCodeName());
    assertEquals("test package float 2", m_oDetailedGridSettings.GetPackageFloat(1).GetDisplayName());
    assertEquals("packagefloat2", m_oDetailedGridSettings.GetPackageFloat(1).GetCodeName());
    assertEquals(sCodeName, m_oDetailedGridSettings.GetPackageFloat(2).GetCodeName());
    assertEquals(sDisplayName, m_oDetailedGridSettings.GetPackageFloat(2).GetDisplayName());

    sDisplayName = "test package int 1";
    sCodeName = "packageint1";
    m_oDetailedGridSettings.AddPackageInt(sCodeName, sDisplayName);
    assertEquals(4, m_oDetailedGridSettings.GetNumberOfInts());
    assertEquals(1, m_oDetailedGridSettings.GetNumberOfPackageInts());
    assertEquals(sCodeName, m_oDetailedGridSettings.GetPackageInt(0).GetCodeName());
    assertEquals(sDisplayName, m_oDetailedGridSettings.GetPackageInt(0).GetDisplayName());

    sDisplayName = "test package int 2";
    sCodeName = "packageint2";
    m_oDetailedGridSettings.AddPackageInt(sCodeName, sDisplayName);
    assertEquals(4, m_oDetailedGridSettings.GetNumberOfInts());
    assertEquals(2, m_oDetailedGridSettings.GetNumberOfPackageInts());
    assertEquals("test package int 1", m_oDetailedGridSettings.GetPackageInt(0).GetDisplayName());
    assertEquals("packageint1", m_oDetailedGridSettings.GetPackageInt(0).GetCodeName());
    assertEquals(sCodeName, m_oDetailedGridSettings.GetPackageInt(1).GetCodeName());
    assertEquals(sDisplayName, m_oDetailedGridSettings.GetPackageInt(1).GetDisplayName());

    sDisplayName = "test package int 3";
    sCodeName = "packageint3";
    m_oDetailedGridSettings.AddPackageInt(sCodeName, sDisplayName);
    assertEquals(4, m_oDetailedGridSettings.GetNumberOfInts());
    assertEquals(3, m_oDetailedGridSettings.GetNumberOfPackageInts());
    assertEquals("test package int 1", m_oDetailedGridSettings.GetPackageInt(0).GetDisplayName());
    assertEquals("packageint1", m_oDetailedGridSettings.GetPackageInt(0).GetCodeName());
    assertEquals("test package int 2", m_oDetailedGridSettings.GetPackageInt(1).GetDisplayName());
    assertEquals("packageint2", m_oDetailedGridSettings.GetPackageInt(1).GetCodeName());
    assertEquals(sCodeName, m_oDetailedGridSettings.GetPackageInt(2).GetCodeName());
    assertEquals(sDisplayName, m_oDetailedGridSettings.GetPackageInt(2).GetDisplayName());

    sDisplayName = "test package int 4";
    sCodeName = "packageint4";
    m_oDetailedGridSettings.AddPackageInt(sCodeName, sDisplayName);
    assertEquals(4, m_oDetailedGridSettings.GetNumberOfInts());
    assertEquals(4, m_oDetailedGridSettings.GetNumberOfPackageInts());
    assertEquals("test package int 1", m_oDetailedGridSettings.GetPackageInt(0).GetDisplayName());
    assertEquals("packageint1", m_oDetailedGridSettings.GetPackageInt(0).GetCodeName());
    assertEquals("test package int 2", m_oDetailedGridSettings.GetPackageInt(1).GetDisplayName());
    assertEquals("packageint2", m_oDetailedGridSettings.GetPackageInt(1).GetCodeName());
    assertEquals("test package int 3", m_oDetailedGridSettings.GetPackageInt(2).GetDisplayName());
    assertEquals("packageint3", m_oDetailedGridSettings.GetPackageInt(2).GetCodeName());
    assertEquals(sCodeName, m_oDetailedGridSettings.GetPackageInt(3).GetCodeName());
    assertEquals(sDisplayName, m_oDetailedGridSettings.GetPackageInt(3).GetDisplayName());

    m_oDetailedGridSettings.SetSaveFrequency(5);
    assertEquals(5, m_oDetailedGridSettings.GetSaveFrequency());
  }

  /**
   * Tests detailed output settings functionality. Order of test functions
   * is important.
   */
  public void testEverything() {

    TestAdding();
    TestCloning();
    TestRemoval();

  }
}
